Package: ReliaLearnR
Type: Package
Title: Learning Modules for Reliability Analysis
Version: 0.5
URL: https://paulgovan.github.io/ReliaLearnR/,
        https://github.com/paulgovan/ReliaLearnR
BugReports: https://github.com/paulgovan/ReliaLearnR/issues
Authors@R: person("Paul", "Govan", email = "paul.govan2@gmail.com", role = c("aut", "cre", "cph"), 
  comment = c(ORCID = "0000-0002-1821-8492"))
Description: Learning modules for reliability analysis including modules for Reliability,
  Availability, and Maintainability (RAM) Analysis, Life Data Analysis, Reliability Testing,
  Repairable Systems Analysis, and Reliability Block Diagrams.
Depends: R (>= 4.1.0)
Imports: learnr (>= 0.11.0), ReliaGrowR, WeibullR, WeibullR.ALT
Suggests: DiagrammeR, knitr, mockery, ReliaPlotR, ReliaShiny,
        rmarkdown, testthat (>= 3.0.0)
License: Apache License (>= 2)
Encoding: UTF-8
RoxygenNote: 7.3.3
VignetteBuilder: knitr
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-05-27 12:36:06 UTC; paulgovan
Author: Paul Govan [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-1821-8492>)
Maintainer: Paul Govan <paul.govan2@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-27 13:00:02 UTC
Built: R 4.5.2; ; 2026-05-27 14:38:51 UTC; unix

# Generated by roxygen2: do not edit by hand

export(avail)
export(fr)
export(lda)
export(mtbf)
export(mttf)
export(ram)
export(rbd)
export(rel)
export(rs)
export(rt)
import(ReliaGrowR)
import(WeibullR)
import(WeibullR.ALT)
importFrom(learnr,run_tutorial)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# ReliaLearnR <a href="https://paulgovan.github.io/ReliaLearnR/"><img src="man/figures/logo.png" align="right" height="136" alt="ReliaLearnR website" /></a>

<!-- badges: start -->

[![Project Status: Active – The project has reached a stable, usable
state and is being actively
developed.](https://www.repostatus.org/badges/latest/active.svg)](https://www.repostatus.org/#active)
[![CRAN
status](https://www.r-pkg.org/badges/version/ReliaLearnR)](https://CRAN.R-project.org/package=ReliaLearnR)
[![R-CMD-check](https://github.com/paulgovan/ReliaLearnR/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/paulgovan/ReliaLearnR/actions/workflows/R-CMD-check.yaml)
[![Codecov test
coverage](https://codecov.io/gh/paulgovan/ReliaLearnR/graph/badge.svg)](https://app.codecov.io/gh/paulgovan/ReliaLearnR)
[![](http://cranlogs.r-pkg.org/badges/last-month/ReliaLearnR)](https://cran.r-project.org/package=ReliaLearnR)
[![](http://cranlogs.r-pkg.org/badges/grand-total/ReliaLearnR)](https://cran.r-project.org/package=ReliaLearnR)
[![DOI](https://jose.theoj.org/papers/10.21105/jose.00302/status.svg)](https://doi.org/10.21105/jose.00302)
<!-- badges: end -->

Welcome to **ReliaLearnR!** This package provides interactive learning
modules on fundamental concepts in reliability analysis. The modules are
built using the `learnr` package and cover topics such as life data
analysis, reliability testing, repairable systems analysis, reliability
block diagrams, and reliability, availability, and maintainability (RAM)
concepts. The package also includes helper functions for common RAM
calculations.

## Installation

To install ReliaLearnR in R:

``` r
install.packages('ReliaLearnR')
```

To install the development version:

``` r
# install.packages("pak")
pak::pak("paulgovan/ReliaLearnR")
```

Note: You may be prompted to update dependent packages before
installing. To do so, type 1 (All) when prompted.

## Recommended Background

ReliaLearnR is designed for students and professionals who are
interested in learning the fundamentals of reliability analysis. No
prior experience is required, but a basic understanding of R and
statistics is helpful. For a complete beginners’ guide to R, check out
the resources at <https://education.rstudio.com/learn/beginner/>.

## Usage

The package includes five interactive learning modules. To launch the
modules, load the package and call the respective function:

- `ram()` - A quick reference for common Reliability, Availability, and
  Maintainability (RAM) concepts
- `rbd()` - An introduction to Reliability Block Diagrams
- `lda()` - An introduction to Life Data Analysis
- `rt()` - An introduction to Reliability Testing
- `rs()` - An introduction to Repairable Systems Analysis

The modules can also be accessed in a browser at
[paulgovan.shinyapps.io/RAMAnalysis/](https://paulgovan.shinyapps.io/RAMAnalysis/),
[govan.shinyapps.io/ReliabilityBlocks/](https://govan.shinyapps.io/ReliabilityBlocks/)
[paulgovan.shinyapps.io/LifeDataAnalysis/](https://paulgovan.shinyapps.io/LifeDataAnalysis/)
and
[paulgovan.shinyapps.io/ReliabilityTesting/](https://paulgovan.shinyapps.io/ReliabilityTesting/),
[govan.shinyapps.io/RepairableSystems/](https://govan.shinyapps.io/RepairableSystems/)

![](https://github.com/paulgovan/ReliaLearnR/blob/master/inst/paper/ReliaLearnR.png?raw=true)<!-- -->

The package also includes several helper functions for common RAM
calculations. These functions can be used independently of the learning
modules:

- `rel()` - reliability function
- `avail()` - availability function
- `mttf()` - mean time to failure
- `mtbf()` - mean time between failure
- `fr()` - failure rate

## Companion Book

A static companion reference book is available alongside the interactive
tutorials. It covers the same material in a readable, searchable, and
printable format, with additional chapters on interactive visualization
with [ReliaPlotR](https://CRAN.R-project.org/package=ReliaPlotR) and the
web-based app
[ReliaShiny](https://CRAN.R-project.org/package=ReliaShiny).

📖 **[Read the Companion
Book](https://paulgovan.github.io/ReliaLearnR/book/)**

The book source lives in the `book/` directory of this repository and is
rendered automatically on every push to `main`.

## Design

The learning modules are designed to be interactive and engaging, with a
focus on practical applications. Each module includes a mix of
instructional content, code examples, and exercises to reinforce
learning. The modules are self-paced, allowing learners to progress at
their own speed.

The original learning modules were provided in a series of workshops,
where each workshop covered a specific module over a 1-2 hour period.
These workshops were designed to be completed in a classroom setting
with an instructor. The current version of the modules has been adapted
for self-paced learning, but they can still be used in a classroom
setting with an instructor.

To adopt the modules for classroom use, instructors can either access
them via the project website or install the package and use the
functions directly. Instructors can also modify the modules to fit their
specific needs, as the source code is available on the project
repository.

## Motivation

This project began as an effort to build upon a reliability program
developed at a major technology company. The original program proved to
provide a strong foundation, providing a structured learning opportunity
that helped many early-career professionals understand and apply the
fundamental concepts of reliability engineering. Over time, however, the
proprietary nature of the program limited accessibility and
adaptability.

Recognizing the importance of keeping reliability learning both relevant
and accessible, this project was initiated to create an open-source
framework for teaching reliability analysis. By leveraging this
framework, this project aims to reach a broader audience, encourage
collaboration, and ensure that learning resources can evolve as needs
and priorities change.

## Code of Conduct

Please note that the ReliaLearnR project is released with a [Contributor
Code of
Conduct](https://contributor-covenant.org/version/2/1/CODE_OF_CONDUCT.html).
By contributing to this project, you agree to abide by its terms.

# Internal validation helper used by all exported RAM functions.
.validate_inputs <- function(x, y, x_name = "first argument", y_name = "second argument") {
  if (anyNA(x) || anyNA(y)) stop("NA values not allowed in inputs")
  if (!is.numeric(x)) stop(paste0('"', x_name, '" must be numeric'))
  if (!is.numeric(y)) stop(paste0('"', y_name, '" must be numeric'))
  if (any(x < 0) || any(y < 0)) stop("inputs must be non-negative")
  if (length(x) == 0 || length(y) == 0) stop("inputs must not be empty vectors")
  if (length(x) > 1 && length(y) > 1 && length(x) != length(y))
    stop("inputs must have the same length or one must be a scalar")
}


#' Reliability (1 - outage / total)
#'
#' Reliability is the probability that an item will perform its intended function
#' without failure over a specified period under stated conditions. This function
#' computes reliability given outage times and total times.
#'
#' @param outageTime Numeric scalar or numeric vector of forced outage times.
#' @param totalTime Numeric scalar or numeric vector of total times (same units as outageTime).
#' @return Numeric scalar: reliability for the period (between 0 and 1).
#' @examples
#' rel(100, 1000)
#' rel(c(10,20), c(500, 600))
#' @export
rel <- function(outageTime, totalTime) {
  .validate_inputs(outageTime, totalTime, "outageTime", "totalTime")

  total <- sum(totalTime)
  if (total == 0) stop("sum(totalTime) is zero (cannot divide by zero)")

  r <- 1 - (sum(outageTime) / total)
  # clamp to [0,1] in case of rounding/overdrawn outages
  r <- max(0, min(1, r))
  return(r)
}


#' Availability (1 - unavailability / total)
#'
#' Availability is the proportion of time a system is in a functioning condition.
#' This function computes availability given unavailable times and total times.
#'
#' @param unavailTime Numeric scalar or numeric vector of unavailable times.
#' @param totalTime Numeric scalar or numeric vector of total times (same units as unavailTime).
#' @return Numeric scalar: availability for the period (between 0 and 1).
#' @examples
#' avail(100, 1000)
#' avail(c(5,10), c(500,600))
#' @export
avail <- function(unavailTime, totalTime) {
  .validate_inputs(unavailTime, totalTime, "unavailTime", "totalTime")

  total <- sum(totalTime)
  if (total == 0) stop("sum(totalTime) is zero (cannot divide by zero)")

  a <- 1 - (sum(unavailTime) / total)
  a <- max(0, min(1, a))
  return(a)
}

#' Mean Time Between Failures (MTBF) for repairable items.
#'
#' MTBF = total operating time / number of failures. The MTBF is the expected time
#' between consecutive failures. It is commonly used for repairable items. The
#' behavior is the same as mttf here; keep separate name for semantic clarity.
#'
#' @param failures Numeric scalar or numeric vector of failure counts (non-negative).
#' @param totalTime Numeric scalar or numeric vector of total operating times.
#' @return Numeric scalar: MTBF. If number of failures is zero, returns Inf (with a warning).
#' @examples
#' mtbf(5, 1000)
#' mtbf(c(2,3), c(500,500))
#' @export
mtbf <- function(failures, totalTime) {
  .validate_inputs(failures, totalTime, "failures", "totalTime")

  total_failures <- sum(failures)
  total_time     <- sum(totalTime)

  if (total_time == 0) stop("sum(totalTime) is zero (cannot compute MTBF)")
  if (total_failures == 0L) {
    warning("zero failures observed; MTBF is infinite")
    return(Inf)
  }

  return(total_time / total_failures)
}


#' Mean Time To Failure (MTTF)
#'
#' For non-repairable items MTTF = total operating time / number of failures. The
#' MTTF is the expected time to the first failure. It is commonly used for non-repairable items.
#' The behavior implemented is the same as mtbf here; keep separate name for semantic clarity.
#'
#' @param failures Numeric scalar or numeric vector of failure counts (non-negative).
#' @param totalTime Numeric scalar or numeric vector of total operating times.
#' @return Numeric scalar: MTTF. If number of failures is zero, returns Inf (with a warning).
#' @examples
#' mttf(5, 1000)
#' mttf(c(2,3), c(500,500))
#' @export
mttf <- function(failures, totalTime) {
  .validate_inputs(failures, totalTime, "failures", "totalTime")

  total_failures <- sum(failures)
  total_time     <- sum(totalTime)

  if (total_time == 0) stop("sum(totalTime) is zero (cannot compute MTTF)")
  if (total_failures == 0L) {
    warning("zero failures observed; MTTF is infinite")
    return(Inf)
  }
  return(total_time / total_failures)
}


#' Failure rate (lambda)
#'
#' Failure rate is the frequency with which an engineered system or component fails,
#' expressed in failures per unit of time. This function computes failure rate given
#' failure counts and total operating times.
#'
#' @param failures Numeric scalar or numeric vector of failure counts (non-negative).
#' @param totalTime Numeric scalar or numeric vector of total operating times.
#' @return Numeric scalar: failures per unit time (failures / totalTime).
#' @examples
#' fr(75, 5000)
#' fr(c(10,5), c(1000,2000))
#' @export
fr <- function(failures, totalTime) {
  .validate_inputs(failures, totalTime, "failures", "totalTime")

  total_failures <- sum(failures)
  total_time     <- sum(totalTime)

  if (total_time == 0) stop("sum(totalTime) is zero (cannot compute failure rate)")

  return(total_failures / total_time)
}

test_that("rel() errors on NA, non-numeric, negative, and zero totalTime", {
  expect_error(rel(NA, 100), "NA values not allowed in inputs")
  expect_error(rel(10, NA), "NA values not allowed in inputs")
  expect_error(rel("a", 100), "\"outageTime\" must be numeric")
  expect_error(rel(10, "b"), "\"totalTime\" must be numeric")
  expect_error(rel(-1, 100), "inputs must be non-negative")
  expect_error(rel(1, -100), "inputs must be non-negative")
  expect_error(rel(1, 0), "sum\\(totalTime\\) is zero")
})

test_that("avail() errors on NA, non-numeric, negative, and zero totalTime", {
  expect_error(avail(NA, 100), "NA values not allowed in inputs")
  expect_error(avail(10, NA), "NA values not allowed in inputs")
  expect_error(avail("a", 100), "\"unavailTime\" must be numeric")
  expect_error(avail(10, "b"), "\"totalTime\" must be numeric")
  expect_error(avail(-1, 100), "inputs must be non-negative")
  expect_error(avail(1, -100), "inputs must be non-negative")
  expect_error(avail(1, 0), "sum\\(totalTime\\) is zero")
})

test_that("mtbf() errors on NA, non-numeric, negative, and zero totalTime", {
  expect_error(mtbf(NA, 100), "NA values not allowed in inputs")
  expect_error(mtbf(1, NA), "NA values not allowed in inputs")
  expect_error(mtbf("x", 100), "\"failures\" must be numeric")
  expect_error(mtbf(1, "y"), "\"totalTime\" must be numeric")
  expect_error(mtbf(-1, 100), "inputs must be non-negative")
  expect_error(mtbf(1, -100), "inputs must be non-negative")
  expect_error(mtbf(1, 0), "sum\\(totalTime\\) is zero")
})

test_that("mttf() errors on NA, non-numeric, negative, and zero totalTime", {
  expect_error(mttf(NA, 100), "NA values not allowed in inputs")
  expect_error(mttf(1, NA), "NA values not allowed in inputs")
  expect_error(mttf("x", 100), "\"failures\" must be numeric")
  expect_error(mttf(1, "y"), "\"totalTime\" must be numeric")
  expect_error(mttf(-1, 100), "inputs must be non-negative")
  expect_error(mttf(1, -100), "inputs must be non-negative")
  expect_error(mttf(1, 0), "sum\\(totalTime\\) is zero")
})

test_that("fr() errors on NA, non-numeric, negative, and zero totalTime", {
  expect_error(fr(NA, 100), "NA values not allowed in inputs")
  expect_error(fr(1, NA), "NA values not allowed in inputs")
  expect_error(fr("x", 100), "\"failures\" must be numeric")
  expect_error(fr(1, "y"), "\"totalTime\" must be numeric")
  expect_error(fr(-1, 100), "inputs must be non-negative")
  expect_error(fr(1, -100), "inputs must be non-negative")
  expect_error(fr(1, 0), "sum\\(totalTime\\) is zero")
})

test_that("rel() and avail() handle boundary cases correctly", {
  # outage = total -> reliability = 0
  expect_equal(rel(500, 500), 0)
  # unavail = 0 -> availability = 1
  expect_equal(avail(0, 500), 1)
  # tiny floating point overdrawn outage (clamped to 0)
  expect_equal(rel(100.0001, 100), 0)
  # tiny floating point negative (clamped to 1)
  expect_equal(avail(1e-12, 100), 1)
})

test_that("rel() and avail() error when totalTime sums to zero", {
  expect_error(rel(1, 0), "sum\\(totalTime\\) is zero")
  expect_error(avail(1, 0), "sum\\(totalTime\\) is zero")
  # also with vectors summing to zero
  expect_error(rel(c(1,2), c(0, -2)), "inputs must be non-negative")
})

test_that("mtbf()/mttf() handle vector zero failures properly", {
  expect_warning(mtbf(c(0,0), c(100,200)), "zero failures")
  expect_true(is.infinite(suppressWarnings(mtbf(c(0,0), c(100,200)))))
  expect_warning(mttf(c(0,0), c(50,150)), "zero failures")
  expect_true(is.infinite(suppressWarnings(mttf(c(0,0), c(50,150)))))
})

test_that("fr() errors when total time is zero", {
  expect_error(fr(1, 0), "sum\\(totalTime\\) is zero")
  expect_error(fr(c(1,2), c(0,0)), "sum\\(totalTime\\) is zero")
})

test_that("all functions accept numeric vectors and collapse correctly", {
  expect_equal(rel(c(10,10), c(50,50)), 1 - (20 / 100))
  expect_equal(avail(c(1,1,1), c(10,10,10)), 1 - (3 / 30))
  expect_equal(mtbf(c(1,2), c(50,150)), 200 / 3)
  expect_equal(mttf(c(2,2), c(100,100)), 200 / 4)
  expect_equal(fr(c(1,2,3), c(10,10,20)), 6 / 40)
})

test_that("all functions reject empty vectors", {
  expect_error(rel(numeric(0), 100),   "must not be empty")
  expect_error(rel(100, numeric(0)),   "must not be empty")
  expect_error(avail(numeric(0), 100), "must not be empty")
  expect_error(mtbf(numeric(0), 100),  "must not be empty")
  expect_error(mttf(numeric(0), 100),  "must not be empty")
  expect_error(fr(numeric(0), 100),    "must not be empty")
})

test_that("all functions reject mismatched vector lengths", {
  expect_error(rel(c(1,2,3), c(100,200)),   "same length")
  expect_error(avail(c(1,2,3), c(100,200)), "same length")
  expect_error(mtbf(c(1,2,3), c(100,200)),  "same length")
  expect_error(mttf(c(1,2,3), c(100,200)),  "same length")
  expect_error(fr(c(1,2,3), c(100,200)),    "same length")
})

test_that("scalar paired with vector is allowed", {
  expect_no_error(rel(10, c(100, 200)))
  expect_no_error(rel(c(10, 20), 1000))
})

#' Launch the RAM Analysis Tutorial
#'
#' This function launches an interactive tutorial on Reliability, Availability,
#' and Maintainability (RAM) analysis.
#' @importFrom learnr run_tutorial
#' @export
#' @seealso \url{https://paulgovan.github.io/ReliaLearnR/}
#' @return This function does not return a value.
#' @examples
#' if (interactive()) {
#'   ram()
#' }
ram <- function() {
  learnr::run_tutorial("ram", package = "ReliaLearnR")
}

test_that("ram runs without error in interactive mode", {
  skip_if_not(interactive(), "Only runs interactively")
  expect_error(ram(), NA)
})

test_that("lda runs without error in interactive mode", {
  skip_if_not(interactive(), "Only runs interactively")
  expect_error(lda(), NA)
})

test_that("rt runs without error in interactive mode", {
  skip_if_not(interactive(), "Only runs interactively")
  expect_error(rt(), NA)
})

test_that("rs runs without error in interactive mode", {
  skip_if_not(interactive(), "Only runs interactively")
  expect_error(rs(), NA)
})

test_that("rbd runs without error in interactive mode", {
  skip_if_not(interactive(), "Only runs interactively")
  expect_error(rbd(), NA)
})

#' Launch the Reliability Block Diagram Tutorial
#'
#' This function launches an interactive tutorial on Reliability Block Diagrams
#' and system reliability, covering series, parallel, mixed, and k-out-of-n
#' configurations as well as an introduction to Fault Tree Analysis.
#' @importFrom learnr run_tutorial
#' @export
#' @seealso \url{https://paulgovan.github.io/ReliaLearnR/}
#' @return This function does not return a value.
#' @examples
#' if (interactive()) {
#'   rbd()
#' }
rbd <- function() {
  learnr::run_tutorial("rbd", package = "ReliaLearnR")
}

#' Launch the Reliability Testing Tutorial
#'
#' This function launches the Reliability Testing tutorial
#' @importFrom learnr run_tutorial
#' @import WeibullR WeibullR.ALT ReliaGrowR
#' @export
#' @seealso \url{https://paulgovan.github.io/ReliaLearnR/}
#' @return This function does not return a value.
#' @examples
#' if (interactive()) {
#'   rt()
#' }
rt <- function() {
  learnr::run_tutorial("rt", package = "ReliaLearnR")
}

#' Launch the Life Data Analysis Tutorial
#'
#' This function launches an interactive tutorial for life data analysis.
#'
#' @importFrom learnr run_tutorial
#' @import WeibullR
#' @export
#' @seealso \url{https://paulgovan.github.io/ReliaLearnR/}
#' @return This function does not return a value.
#' @examples
#' if (interactive()) {
#'   lda()
#' }
lda <- function() {
  learnr::run_tutorial("lda", package = "ReliaLearnR")
}

---
title: "Reliability Testing"
output: 
  learnr::tutorial:
    progressive: true
    allow_skip: true
runtime: shiny_prerendered
---

```{r setup, include=FALSE}
library(learnr)
library(ReliaGrowR)
library(WeibullR.ALT)
```

## Introduction

Welcome to the tutorial on **Reliability Testing**! In this tutorial, we will explore key concepts and models used in reliability testing, including the Crow-AMSAA model, Duane model, and Accelerated Life Testing (ALT) methods such as the Arrhenius and Power Law models. These models help engineers and reliability professionals assess and improve the reliability of products and systems over time. 

## Learning Objectives

By the end of this tutorial, learners will be able to:

* Define key reliability growth concepts, including the Crow-AMSAA and Duane models.
* Explain fundamental principles of accelerated life testing, including the Arrhenius and Power Law models.
* Fit reliability growth models to real-world datasets using R.
* Apply the Crow-AMSAA model to assess reliability growth.
* Conduct accelerated life tests with real-world datasets, utilizing R for analysis.
* Interpret accelerated life testing plots to identify trends.
* Analyze the impact of stress factors on product reliability using Arrhenius and Power Law models.

## What is Reliability Testing?

Reliability testing is a process to ensure that a product or system performs its intended functions under specified conditions over a defined period. For example, in the manufacturing industry, we may want to design a machine that runs without breaking down for 5 to 10 years.

## Why is Reliability Testing important?

Reliability testing is important for several reasons: 

* Helps identify potential failure modes and weaknesses in a product or system.
* Ensures that products meet quality and safety standards.
* Enhances customer satisfaction by delivering reliable products.
* Reduces costs associated with warranty claims, repairs, and downtime.

## Reliability Growth Analysis

**Reliability Growth Analysis (RGA)** is a method used to monitor and improve the reliability of a product or system over time. It involves analyzing failure data collected during testing phases to identify trends and patterns in reliability performance. 

## The Duane Model

**The Duane Model**, developed by Duane in 1964, is one of the earliest and most widely used models for reliability growth analysis. The model is particularly useful due to its simplicity and ease of interpretation. The Duane plot is a graphical representation that helps visualize whether failure rates are improving, stable, or worsening over time.

Specifically, the Duane Model is a log-log plot of the cumulative Mean Time Between Failures (MTBF) vs cumulative time, where the MTBF is calculated as the total operating time divided by the number of failures observed up to that time.

The slope of the line on the plot indicates the rate of reliability growth:

* A positive slope means that the system is improving (reliability is growing, the failure rate is decreasing).
* A zero slope means there is no change in reliability (the system is stable).
* A negative slope indicates that reliability is worsening (the failure rate is increasing).

### The Duane Model: Reliability Growth vs Deterioration

To illustrate the Duane Model, take the following example. Adjust the Beta value 
input to see the failure rate both decreasing and increasing.

```{r echo=FALSE}
numericInput("beta", label = h3("Beta Value:"), value = 1, min = 0, max = 5, step = 0.1, width = '50%') # Numeric input for beta value
plotOutput("duanePlot") # Plot output for the Duane plot
```

```{r, context="server"}
# Plotting the Duane Model based on user input for beta value
output$duanePlot <- renderPlot({
  x <- seq(1, 100, by = .1)
  y <- 0.1 * x ^ (input$beta - 1)
  log_x <- log(x)
  log_y <- log(y)
  mainTitle <- if (input$beta == 1) {
    "Duane Plot with Stable Failure Rate"
  } else if (input$beta < 1) {
    "Duane Plot with Increasing Failure Rate"
  } else {
    "Duane Plot with Decreasing Failure Rate"
  }
  plot(log_x, log_y, type = "l", col = "blue", xlab = "Time", ylab = "Cumulative MTBF", main = mainTitle)
})
```

As you adjust the Beta value, observe how the slope of the line changes. When
Beta is greater than 1, the slope is positive, indicating that the system is improving. 
When Beta is less than 1, the slope is negative, indicating that the system is worsening. 
When Beta equals 1, the slope is zero, indicating that the system is stable.

### The Duane Model Equation

The cumulative MTBF at any time \( t \) can be expressed as:

\[
CMBTF(t) = K \cdot t^{\beta - 1}
\]

Where:

- \( CMBTF(t) \) is the cumulative MTBF at time \( t \),
- \( K \) is a constant related to the initial failure rate,
- \( t \) is the cumulative operating time,
- \( \beta \) is the reliability growth parameter (slope of the line on the log-log plot).

  - If \( \beta > 1 \), the system reliability is improving (the failure rate decreases over time).
  - If \( \beta = 1 \), the system reliability is constant (no improvement or degradation).
  - If \( \beta < 1 \), the system reliability is deteriorating (the failure rate increases over time).

## ReliaGrowR

### Getting Started with ReliaGrowR

For this tutorial, we will use **ReliaGrowR**: an R package for Reliability Growth Analysis.

First, check if ReliaGrowR is installed in R and install if not.

```{r eval=FALSE}
pak::pkg_install("ReliaGrowR")
```

As an example, suppose a manufacturer wants to analyze the reliability growth of a new product. The product has been tested over a period of time, and the manufacturer has collected data on the cumulative operating time and the number of failures that occurred during that time. The cumulative time and failure data are as follows: 

```{r echo=TRUE}
times <- c(100, 200, 300, 400, 500)
failures <- c(1, 2, 1, 3, 2)
```

We can use the `duane` function with the cumulative time and failure data to fit a Duane model. Then we can use the `plot` function to visualize the results. The `main`, `xlab`, and `ylab` parameters are used to customize the plot title and axis labels.

```{r echo=TRUE}
fit <- duane(times, failures)
plot(fit, main = "Duane Model Example", xlab = "Cumulative Time", ylab = "Cumulative MTBF")
```

The plot shows a Duane Model fit to the data. The slope of the line indicates whether the system is improving, stable, or worsening. In this case, the slope is greater than 1, indicating that the system is improving over time.

```{r quiz1, echo=FALSE}
quiz(caption = "Quiz: Duane Model",
  question("What does a Duane Plot with a slope greater than 1 indicate?",
    answer("The system's reliability is improving", correct = TRUE, 
           message = "A slope greater than 1 indicates improvement."),
    answer("The system's reliability is deteriorating", 
           message = "A slope less than 1 indicates deterioration."),
    answer("The system has no change in reliability", 
           message = "A slope of 1 indicates no change in reliability."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("What does a Duane Plot with a slope less than 1 indicate?",
    answer("The system's reliability is improving",
           message = "A slope greater than 1 indicates improvement."),
    answer("The system's reliability is deteriorating", correct = TRUE, 
           message = "A slope less than 1 indicates deterioration."),
    answer("The system has no change in reliability", 
           message = "A slope of 1 indicates no change in reliability."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("In the previous example, what is the approximate cumulative MTBF at time 1000?",
    answer("200", message = "200 is too high, try again."),
    answer("100", message = "100 is too low, try again."),
    answer("120", message = "120 is slightly too low, try again."),
    answer("150", correct = TRUE, message = "The cumulative MTBF at time 1000 is approximately 150."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("In the previous example, how would you describe the reliability of the system?",
    answer("The system reliability is improving", correct = TRUE, 
           message = "The slope is greater than 1, so the system is improving."),
    answer("The system reliability is worsening", 
           message = "The slope is not less than 1, so the system is not worsening."),
    answer("The system reliability is stable", 
           message = "The slope is not close to 1, so the system is not stable."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  )
)
```

### Exercise: Fit a Duane Model

A new system was tested and the following cumulative failure counts were recorded at each cumulative test time. Fit a Duane model using `duane()` and plot the result.

```{r ex-rt1, exercise=TRUE}
times    <- c(200, 450, 750, 1100, 1500, 2000)
failures <- c(3, 2, 2, 1, 1, 1)

```

```{r ex-rt1-hint}
# Use duane() with cumulative times and per-interval failure counts, then plot
# fit <- duane(time = times, event = failures)
# plot(fit)
```

```{r ex-rt1-solution}
times    <- c(200, 450, 750, 1100, 1500, 2000)
failures <- c(3, 2, 2, 1, 1, 1)
fit <- duane(time = times, event = failures)
plot(fit, main = "Duane Reliability Growth Plot",
     xlab = "Cumulative Test Time", ylab = "Cumulative Failures")
```

## The Crow-AMSAA Model

The **Crow-AMSAA Model** is another widely used model for reliability growth analysis, developed by Larry Crow and the  Army Materiel Systems Analysis Activity (AMSAA) organization in the 1970s. This model is particularly useful for systems that undergo continuous improvements and modifications during their development and testing phases. The Crow-AMSAA model is based on the concept of a Non-Homogeneous Poisson Process (NHPP), which allows for the modeling of failure behavior that changes over time.

Specifically, the Crow-AMSAA Model is a model of the cumulative number of failures vs cumulative time. 

Similar to the Duane Model, the shape of the model indicates the rate of reliability growth:

* A model fit with downward curvature means that the system is improving (reliability is growing, the failure rate is decreasing).
* A linear model means there is no change in reliability (the system is stable).
* A model fit with upward curvature indicates that reliability is worsening (the failure rate is increasing).

### The Crow-AMSAA Model: Reliability Growth vs Deterioration

To illustrate the Crow-AMSAA model, take the following example. Adjust the Beta 
value input to see the failure rate both decreasing and increasing.

```{r echo=FALSE}
numericInput("beta", label = h3("Beta Value:"), value = 1, min = 0, max = 5, step = 0.1, width = '50%') # Numeric input for beta value
plotOutput("crowPlot") # Plot output for the Crow-AMSAA plot
```

```{r, context="server"}
# Plotting the Crow-AMSAA Model based on user input for beta value
output$crowPlot <- renderPlot({
  x <- seq(1, 100, by = .1)
  y <- 0.5 * x ^ input$beta
  mainTitle <- if (input$beta == 1) {
    "Crow-AMSAA Model with Stable Failure Rate"
  } else if (input$beta < 1) {
    "Crow-AMSAA Model with Decreasing Failure Rate"
  } else {
    "Crow-AMSAA Model with Increasing Failure Rate"
  }
  plot(x, y, type = "l", col = "darkgreen", xlab = "Time", ylab = "Cumulative Failures", main = mainTitle)
})
```

As you adjust the Beta value, observe how the curvature of the line changes. When
Beta is greater than 1, the line curves upward, indicating that the system is worsening. 
When Beta is less than 1, the line curves downward, indicating that the system is improving. 
When Beta equals 1, the line is linear, indicating that the system is stable.

### The Crow-AMSAA Model Equation:

The cumulative number of failures up to time \( t \) is given by:

\[N(t) = \lambda_0 \cdot t^{\beta}\]

Where:

* \( N(t) \) is the cumulative number of failures,
* \( \lambda_0 \) is the scale parameter.

* The Shape Parameter (\( \beta \)) indicates whether the system is improving or deteriorating:

  * If \( \beta \) > 1, failures are increasing over time, indicating that reliability is worsening.
  * If \( \beta \) < 1, failures are decreasing, indicating that reliability is improving over time.
  * \( \beta \) = 1 implies a constant failure rate (no growth or degradation).

### Crow-AMSAA with ReliaGrowR

As an example, suppose the same manufacturer from the Duane Model example also wants to analyze the reliability growth of a new product using the Crow-AMSAA model. The cumulative time and failure data are as follows:

```{r}
times <- c(100, 200, 300, 400, 500)
failures <- c(1, 2, 1, 3, 2)
```

To run a Crow-AMSAA model with ReliaGrowR, use the `rga` function to fit the model and the `plot` function to plot the results:

```{r}
result <- rga(times, failures)
plot(result, main = "Crow-AMSAA Model Example", xlab = "Cumulative Time", ylab = "Cumulative Failures")
```

The plot shows a Crow-AMSAA Model fit to the data. The curvature of the line indicates whether the system is improving, stable, or worsening. In this case, the slope is curving downward, indicating that the system is improving over time.

```{r quiz2, echo=FALSE}
quiz(caption = "Quiz: Crow-AMSAA Model",
  question("What does a Crow-AMSAA Model with a Beta greater than 1 indicate?",
    answer("The system's reliability is improving", 
           message = "A Beta less than 1 indicates improvement."),
    answer("The system's reliability is deteriorating", correct = TRUE, 
           message = "A Beta greater than 1 indicates deterioration."),
    answer("The system has no change in reliability", 
           message = "A Beta of 1 indicates no change in reliability."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("What does a Crow-AMSAA Model with a Beta less than 1 indicate?",
    answer("The system's reliability is improving", correct = TRUE, 
           message = "A Beta less than 1 indicates improvement."),
    answer("The system's reliability is deteriorating", 
           message = "A Beta greater than 1 indicates deterioration."),
    answer("The system has no change in reliability", 
           message = "A Beta of 1 indicates no change in reliability."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("In the previous example, what is the cumulative number of failures at time 1000?",
    answer("4", message = "4 is too low, try again."),
    answer("8", message = "8 is slightly too high, try again."),
    answer("6", message = "6 is slightly too low, try again."),
    answer("7", correct = TRUE, 
           message = "The cumulative number of failures at time 1000 is approximately 7."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("In the previous example, how would you describe the reliability of this system?",
    answer("The reliability of the system is improving", correct = TRUE, 
           message = "The slope is curving downward, so the reliability is improving."),
    answer("The reliability of the system is worsening", 
           message = "The slope is not curving upward, so the reliability is not worsening."),
    answer("The reliability of the system is stable", 
           message = "The slope is not linear, so the reliability is not stable."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  )
)
```

### Exercise: Fit a Crow-AMSAA Model

A development test recorded the following cumulative failure counts at cumulative test times. Use `rga()` with `model_type = "Crow-AMSAA"` to fit the model and plot the result.

```{r ex-rt2, exercise=TRUE}
times    <- c(50, 120, 220, 360, 530, 740, 1000)
failures <- c(4, 3, 3, 2, 2, 1, 1)

```

```{r ex-rt2-hint}
# Use rga() with model_type = "Crow-AMSAA", then plot
# fit <- rga(time = times, event = failures, model_type = "Crow-AMSAA")
# plot(fit)
```

```{r ex-rt2-solution}
times    <- c(50, 120, 220, 360, 530, 740, 1000)
failures <- c(4, 3, 3, 2, 2, 1, 1)
fit <- rga(time = times, event = failures, model_type = "Crow-AMSAA")
plot(fit, main = "Crow-AMSAA Reliability Growth",
     xlab = "Cumulative Test Time", ylab = "Cumulative Failures")
```

### The Piecewise NHPP Model

**The Piecewise NHPP model** is another extension of the Crow-AMSAA model that allows for different phases of reliability growth or deterioration. This model is particularly useful when a system undergoes significant changes in its failure behavior over time, such as during different stages of development or after major design changes.

For a Piecewise NHPP model, the cumulative number of failures is modeled as a piecewise function, where each segment has its own parameters. Formally, for time t within phase i, the relationship between the cumulative number of failures and cumulative time is given by:

\[
N_i(t) = N(t_{i-1}) + \lambda_i (t - t_{i-1})^{\beta_i}, \, \text{for } t_{i-1} < t \leq t_i
\]

where \(N_i(t)\) is the cumulative number of failures by time \(t\) in phase \(i\), \(\lambda_i\) is the scale parameter for phase \(i\), and \(\beta_i\) is the shape parameter for phase \(i\).

### Example

To illustrate the Piecewise NHPP model, take the following example. Suppose a manufacturer has tested a product over a period of time and collected data on the cumulative operating time and the number of failures that occurred during that time. At about time 500, a significant design change was made to the product, which is expected to impact its reliability. The cumulative time, failure data, and breakpoint are as follows:

```{r}
times <- c(25, 55, 97, 146, 201, 268, 341, 423, 513, 609, 710, 820, 940, 1072, 1217)
failures <- c(1, 1, 2, 4, 4, 1, 1, 2, 1, 4, 1, 1, 3, 3, 4)
breaks <- 500
```

To run a Piecewise NHPP model with ReliaGrowR, use the `rga` function with `model_type` set to "Piecewise NHPP" and `breaks` set to the breakpoint.

```{r}
result <- rga(times, failures, model_type = "Piecewise NHPP", breaks = breaks)
plot(result, main = "Piecewise NHPP Model Example", xlab = "Cumulative Time", ylab = "Cumulative Failures")
```

The plot shows a Piecewise NHPP Model fit to the data, with a breakpoint at time 500. The two segments of the model have different slopes, indicating that the failure behavior changed after the design change. The first segment has a higher slope, than the second segment, indicating that the failure rate decreased after the design change. 

### The Piecewise NHPP with Change Point Detection

The **Piecewise NHPP with Change Point Detection** is an advanced model that identifies change points in the failure data to automatically segment the data into different phases of reliability growth or deterioration. This approach is particularly useful when the exact timing of changes in failure behavior is unknown or when multiple changes may occur over times. 

### Example

To illustrate the Piecewise NHPP with Change Point Detection, take the previous example but without specifying a breakpoint. The cumulative time and failure data are as follows:

```{r}
times <- c(25, 55, 97, 146, 201, 268, 341, 423, 513, 609, 710, 820, 940, 1072, 1217)
failures <- c(1, 1, 2, 4, 4, 1, 1, 2, 1, 4, 1, 1, 3, 3, 4)
```

To run a Piecewise NHPP model with Change Point Detection using ReliaGrowR, use the `rga` function with `model_type` set to "Piecewise NHPP" without specifying `breaks`.

```{r}
result <- rga(times, failures, model_type = "Piecewise NHPP")
plot(result, main = "Piecewise NHPP with Change Point Detection", xlab = "Cumulative Time", ylab = "Cumulative Failures")
```

The plot shows a Piecewise NHPP Model fit to the data, with change points automatically detected. The change point is close to time 500, which is consistent with the design change made to the product. 

## Accelerated Life Testing

**Accelerated Life Testing (ALT)** is a method used to estimate the life of a product or system under normal operating conditions by subjecting it to elevated stress levels in a controlled environment. The goal is to identify potential failure modes and predict how long the product will last under normal conditions without having to wait for failures to occur naturally over a longer time period.

### Acceleration Factor

Suppose you are testing a product that is expected to last a long time under normal conditions. Waiting for it to fail naturally could take months or even years. Instead, you can use ALT to accelerate the testing process by applying higher stress levels, such as increased temperature, voltage, or pressure, to induce failures more quickly.

The relationship between stress and life is often modeled using an exponential function, where the life of the product decreases as the stress level increases. The rate of this decrease is determined by the Acceleration Factor (AF), which quantifies how much faster the product fails under elevated stress compared to normal conditions. 

The plot below illustrates this concept. Adjust the Acceleration Factor (AF) to see how it impacts the expected life of the product.

```{r echo=FALSE}
sliderInput("af", label = h3("Acceleration Factor:"), value = 1, min = 1, max = 10, step = 0.1, width = '50%') # Slider input for acceleration factor
plotOutput("altPlot") # Plot output for the ALT plot
```

```{r, context="server"}
# Plotting the Accelerated Life Testing model based on user input for acceleration factor
output$altPlot <- renderPlot({
  y <- seq(1, 10, by = .1)
  x <- 3678.79 * exp(1 / y)
  life <- 3678.79 * exp(1 / input$af)
  plot(x, y, type = "l", col = "blue", xlab = "Life", ylab = "Stress Factor", main = "Stress Factor vs Life")
  abline(v = life, col = "red", lty = 2)
  legend("topright", legend = c("Expected Life = ", ceiling(life)))
})
```

In this plot, the blue curve represents the relationship between the stress and the expected life. The red dashed line indicates the expected life for the selected acceleration factor. As you increase the acceleration factor, the expected life decreases, demonstrating how elevated stress levels can significantly reduce the lifespan of a product.

## The Arrhenius Model

The **Arrhenius Model** is one of the most commonly used models in ALT. It describes the relationship between temperature and the rate of chemical reactions, which can be directly related to the degradation or failure rate of materials. The model is based on the Arrhenius equation, which states that the rate of a chemical reaction increases exponentially with an increase in temperature.

The Acceleration Factor (AF) for the Arrhenius model is given by:

$$ AF = \exp\left(\frac{E_a}{k} \left( \frac{1}{T_{\text{use}}} - \frac{1}{T_{\text{stress}}} \right) \right) $$

Where:

- \( E_a \) = Activation energy in electron volts (eV). The activation energy is a measure of the energy barrier that must be overcome for a chemical reaction to occur. Higher activation energies indicate that the reaction is more sensitive to temperature changes.
- \( k \) = Boltzmann constant (\( 8.617 \times 10^{-5} \, \text{eV/K} \)). The Boltzmann constant relates the average kinetic energy of particles in a gas with the temperature of the gas.
- \( T_{\text{use}} \) = Temperature at normal use condition in Kelvin
- \( T_{\text{stress}} \) = Temperature at accelerated stress condition in Kelvin

By using the Arrhenius model, you can determine how much faster a product will fail at an elevated temperature compared to its normal operating temperature. 

## WeibullR.ALT

For this tutorial, we will use **WeibullR.ALT**, an extension of the WeibullR package for Accelerated Life Testing.

First, check if WeibullR.ALT is installed in R and install if not.

```{r eval=FALSE}
pak::pkg_install("WeibullR.ALT")
```

To illustrate the Arrhenius Model, we will use the data set `MeekerData` that comes prepackaged with `WeibullR.ALT`. This data set contains time to failure data for carbon-film resistors tested at different temperature levels. The testing was conducted at 4 increasing temperature levels to accelerate the failure process. The temperature levels were 10, 40, 60, and 80 degrees Celsius.

First, load the data set and use the `head` function to view the first few rows.

```{r}
data <- MeekerData("table10")
head(data)
```

The data set contains columns for the time to failure (time), event type (event) (1 = failure, 0 = suspension), quantity of samples (qty) and temperature as the acceleration factor (TempC). 

Next, use the `alt.data` function to create a separate object for the data at each temperature level. Each data set is created by subsetting the original data set for each temperature level and selecting only the relevant columns (time, event, qty). The `stress` parameter is set to the corresponding temperature level. The `which` function is used to filter the rows based on the temperature level.

```{r}
data10C <- alt.data(data[which(data$TempC==10), 1:3], 
                    stress = data[which(data$TempC==10), 4][1])
data40C <- alt.data(data[which(data$TempC==40), 1:3], 
                    stress = data[which(data$TempC==40), 4][1])
data60C <- alt.data(data[which(data$TempC==60), 1:3], 
                    stress = data[which(data$TempC==60), 4][1])
data80C <- alt.data(data[which(data$TempC==80), 1:3], 
                    stress = data[which(data$TempC==80), 4][1])
```

Then, use the `alt.make` function to fit an Arrhenius-Weibull model to the data. Each data set is passed as a list to the `alt.make` function, along with the distribution type (`dist = "weibull"`), the acceleration model (`alt.model = "arrhenius"`), and the fitting method (`method.fit = "mle"`). The resulting object `arrobj` contains the fitted model.

```{r}
arrobj <- alt.make(list(data10C, data40C, data60C, data80C), dist = "weibull",
                alt.model = "arrhenius", method.fit = "mle")
```

The plot shows an Arrhenius-Weibull model fit to the data. The different colors represent the different temperature levels. Starting with a temperature level of 40 degrees C on the right, the curves shift to the left as the temperature increases, indicating that higher temperatures lead to shorter life.

Finally, calculate the AF at 80 degrees C using the Arrhenius Model. Assume that the Activation Energy is 0.7 eV. The code below calculates the AF using the formula provided earlier, with \( T_{\text{use}} \) set to 10 degrees C (283K) and \( T_{\text{stress}} \) set to 80 degrees C (353K).

```{r, results='asis'}
E_a <- 0.7
k <- 8.617E-5
T_use <- 10 + 273
T_stress <- 80 + 273
AF <- exp(E_a / k * (1 / T_use - 1 / T_stress))
cat(paste0("The Acceleration Factor at 80 degrees C is ", round(AF, 2)))
```

The AF at 80 degrees C is approximately 300. This means that the resistors tested at 80 degrees C are expected to fail 300 times faster than those tested at 10 degrees C.

```{r quiz3, echo=FALSE}
quiz(caption = "Quiz: Arrhenius Model",
  question("What does the Arrhenius-Weibull model indicate about the relationship between temperature and life?",
    answer("Higher temperatures lead to longer life", 
           message = "Higher temperatures actually lead to shorter life."),
    answer("Higher temperatures lead to shorter life", correct = TRUE, 
           message = "Higher temperatures lead to shorter life, as indicated by the leftward shift of the curves."),
    answer("Temperature has no impact on life", 
           message = "Temperature does have an impact on life, as shown by the different curves."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("In the Arrhenius Model, what does the Activation Energy (E_a) represent?",
    answer("The energy required to initiate a chemical reaction", correct = TRUE, 
           message = "Activation Energy is the energy required to initiate a chemical reaction."),
    answer("The temperature at which a material fails", 
           message = "Activation Energy is not the temperature at which a material fails."),
    answer("The rate of degradation of a material", 
           message = "Activation Energy is not the rate of degradation of a material."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("In the previous example, what is the approximate unreliability for 
           a duration of 3000 and an operating temperature of 60 (the green curve)?",
    answer("90%", message = "90% is the approximate unreliability for a duration of 
           3000 and an operating temperature of 80 (the purple curve)."),
    answer("10%", message = "10% is too low, try again."),
    answer("70%", message = "70% is the reliability at these levels, not the unreliability"),
    answer("30%", correct = TRUE, 
           message = "The approximate unreliability for a duration of 3000 and 
           an operating temperature of 60 is 30%."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("In the previous example, what is the approximate reliability for a duration of 1000 and an operating stress of 80 (the purple curve)?",
    answer("70%", message = "70% is too high, try again."),
    answer("30%", message = "30% is too low, try again."),
    answer("40%", message = "40% is the unreliability at these levels, not the reliability"),
    answer("60%", correct = TRUE, message = "The approximate reliability for a duration of 1000 and 
           an operating temperature of 80 is 60%."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  )
)
```

### Exercise: Compute an Arrhenius Acceleration Factor

A component is tested at 85°C to accelerate failures. The use temperature is 25°C. The activation energy for the failure mechanism is 0.65 eV. Compute the Acceleration Factor using the Arrhenius model.

```{r ex-rt3, exercise=TRUE}
E_a      <- 0.65        # activation energy (eV)
k        <- 8.617e-5    # Boltzmann's constant (eV/K)
T_use    <- 25 + 273.15 # use temperature in Kelvin
T_stress <- 85 + 273.15 # stress temperature in Kelvin

```

```{r ex-rt3-hint}
# AF = exp((E_a / k) * (1/T_use - 1/T_stress))
```

```{r ex-rt3-solution}
E_a      <- 0.65
k        <- 8.617e-5
T_use    <- 25 + 273.15
T_stress <- 85 + 273.15
AF <- exp((E_a / k) * (1 / T_use - 1 / T_stress))
cat("Acceleration Factor:", round(AF, 1), "\n")
```

## The Power Law Model

The **Power Law Model** is another commonly used model in ALT, particularly for mechanical systems and components. This model describes how a product's life is affected by a certain stress factor (such as temperature, voltage, or pressure). Unlike the Arrhenius model, which is primarily used for temperature-based stresses, the Power Law Model can be applied to a wide variety of stress factors.

The AF for the Power Law model is given by:

$$ AF = \left( \frac{S_{\text{stress}}}{S_{\text{use}}} \right)^{n} $$

Where:

- \( S_{\text{stress}} \) = Stress level during accelerated testing (e.g., voltage, load)
- \( S_{\text{use}} \) = Stress level during normal use conditions
- \( n \) = Stress exponent, indicating the sensitivity of the failure mechanism to the applied stress

To illustrate the Power Law Model, we will use the data set `NelsonData` that comes prepackaged with `WeibullR.ALT`. This data set contains time to failure data for electrical insulating fluids tested at different voltage stress levels. The testing was conducted at 7 increasing voltage levels to accelerate the failure process. 

First, load the data set..

```{r results='asis'}
data <- NelsonData("table3.1")
```

The data is a list of time to failure data at 7 different voltage stress levels, 26 kilovolts (KV), 28kV, 30kV, 32kV, 34kV, 36kV, and 38kV.

Next, use the `alt.data` function to create a separate object for the data at each stess level. The `stress` parameter is set to the corresponding voltage level.

```{r}
data26 <- alt.data(data$kV26, stress = 26)
data28 <- alt.data(data$kV28, stress = 28)
data30 <- alt.data(data$kV30, stress = 30)
data32 <- alt.data(data$kV32, stress = 32)
data34 <- alt.data(data$kV34, stress = 34)
data36 <- alt.data(data$kV36, stress = 36)
data38 <- alt.data(data$kV38, stress = 38)
```

Then, use the `alt.make` function to fit a Power-Weibull model to the data and plot the results. Similar to the previous example, each data set is passed as a list to the `alt.make` function, along with the distribution type (`dist = "weibull"`) and the acceleration model (`alt.model = "power"`). The resulting object `pwrobj` contains the fitted model.

```{r}
pwrobj <- alt.make(list(data26, data28, data30, data32, data34, data36, data38), 
                   dist = "weibull", alt.model = "power")
```

The plot shows a Power-Weibull model fit to the data. The different colors represent the different voltage stress levels. Starting with a stress level of 28kV on the right (dark green curve), the curves shift to the left as the stress increases, indicating that higher stress leads to shorter life.

```{r quiz4, echo=FALSE}
quiz(caption = "Quiz: Power Law Model",
  question("In the previous example, which subset of data does not seem to follow the Power Law model?",
              answer("The data with a stress level of 38kV", 
                     message = "The data with a stress level of 38kV seems to follow the same shape as the other subsets."),
              answer("The data with a stress level of 28kV", 
                     message = "The data with a stress level of 28kV seems to follow the same shape as the other subsets."),
              answer("The data with a stress level of 36kV", 
                     message = "The data with a stress level of 36kV seems to follow the same shape as the other subsets."),
              answer("The data with a stress level of 26kV", correct = TRUE, 
                     message = "The data with a stress level of 26kV has a different shape than the other subsets. Although this may be due to limited data at this stress level (only 3 failures)."),
              random_answer_order = TRUE,
              allow_retry = TRUE
     ),
   question("In the previous example, what is the relationship between the data with a stress level of 26kV (the dark green curve) and the data with a stress level of 30kV (the red curve)?",
            answer("The plot to the right (dark green) has a lower unreliability than the one to the left (red)", correct = TRUE),
            answer("The plot to the right (dark green) a higher unreliability than the one to the left (red)"),
            answer("The plot to the right (dark green) has a lower reliability than the one to the left (red)"),
            answer("The plot to the right (dark green) has a higher reliability than the one to the left (red)", correct = TRUE),
            random_answer_order = TRUE,
            allow_retry = TRUE
   )
)
```

## Multiple Stress Factors

The **Eyring Model** is an extension of the Arrhenius model and is commonly used in ALT to account for the combined effects of multiple stressors such as temperature, voltage, and humidity on the failure mechanisms of a product. This model is particularly useful when the product is exposed to environmental factors beyond just temperature.

The general form of the Eyring Model is:

\[\text{Life}(S) = A \cdot \exp\left(\frac{B}{S} + C \cdot S\right)\]

Where:

* \( \text{Life}(S) \) is the expected life under stress level \( S \),
* \( A \) is a constant,
* \( B \) and \( C \) are model parameters,
* \( S \) is the applied stress factor (e.g., temperature, voltage, humidity).

This module does not include an example of the Eyring Model, but the `WeibullR.ALT` package does support it. See the package documentation for more information.

## Parallelization

Parallelization is a technique used in ALT to simplify the analysis of data collected at multiple stress levels. By assuming that the shape factor (slope) of the life distribution is the same across all stress levels, the analysis can be streamlined, making it easier to interpret the results and draw conclusions about the relationship between stress and life.

To fit a parallel model to the Meeker data, use the `alt.parallel` function.
```{r}
prlobj <- alt.parallel(arrobj)
```

The plot shows a Parallel model fit to the data. All curves have the same slope, indicating that all subsets have the same shape factor.

Similarly, to fit a parallel model to the Nelson data, use the `alt.parallel` function.
```{r}
expobj <- alt.parallel(pwrobj)
```

## Relationship Plots

Another useful tool in ALT is the relationship plot. The relationship plot provides a visual representation of how the life of a product changes as the stress factor increases. A relationship plot is a log-linear plot with the stress factor on the x-axis and the log of the unreliability (failure probability) on the y-axis. 

A linear model is fit to the data, which shows patterns indicating how life changes as stress increases. The slope of the line provides insights into the relationship between stress and life:

* A **negative slope** (decreasing life with increasing stress) indicates that higher stress leads to shorter life. 
* A **positive slope** (increasing life with increasing stress) is less common but indicates that life improves with higher stress.
* A **flat slope** indicates that higher stress does not have an impact on life. 

To fit a relationship plot to the Meeker data, first fit a linear model to the data using the `alt.fit` function and then plot the results using the `plot` function.

```{r}
lnrobj <- alt.fit(prlobj)
plot(lnrobj, suppress.dev.new = TRUE)
```

The relationship plot shows a negative slope, indicating that higher temperature stress tends to lead to shorter life. Each line indicates a different probability of failure. For example, the P63.2 line indicates the stress vs life relationship for a 63.2% probability of failure. The P90 line indicates the stress vs life relationship for a 90% probability of failure. 

Similarly, to fit a relationship plot to the Nelson data, first fit a linear model to the data and then plot the results.

```{r}
lnrobj2 <- alt.fit(expobj)
plot(lnrobj2, suppress.dev.new = TRUE)
```

Similarly, the relationship plot shows a negative slope, indicating that higher voltage stress tends to lead to shorter life. 

```{r quiz5, echo=FALSE}
quiz(caption = "Quiz: Relationship Plots",
  question("In the Arrhenius Weibull example, what is the approximate P63.2 time to failure at a stress level of 50?",
    answer("1000", message = "1000 is too low, try again."),
    answer("5,000", correct = TRUE, 
           message = "The approximate P63.2 time to failure at a stress level of 50 is 5,000."),
    answer("50,000", message = "50,000 is too high, try again."),
    answer("100,000", message = "100,000 is way too high, try again."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("In the Arrhenius Weibull example, what does the slope of the relationship plot indicate about the relationship between stress and life?",
    answer("Higher stress does not seem to have an impact on life", 
           message = "A flat slope would indicate that higher stress does not have an impact on life."),
    answer("Higher stress tends to lead to longer life", message = "A positive slope would indicate that higher stress leads to longer life."),
    answer("Higher stress tends to lead to shorter life", correct = TRUE, 
           message = "The negative slope indicates that higher stress tends to lead to shorter life."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("In the Power Weibull example, what does the slope of the relationship plot indicate about the relationship between stress and life?",
    answer("Higher stress does not seem to have an impact on life", 
           message = "A flat slope would indicate that higher stress does not have an impact on life."),
    answer("Higher stress tends to lead to longer life", message = "A positive slope would indicate that higher stress leads to longer life."),
    answer("Higher stress tends to lead to shorter life", correct = TRUE, 
           message = "The negative slope indicates that higher stress tends to lead to shorter life."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("In the Power Weibull example, what is the approximate P90 time to failure at a stress level of 30?",
    answer("100", correct = TRUE, 
           message = "The approximate P90 time to failure at a stress level of 30 is 100."),
    answer("10", message = "10 is too low, try again."),
    answer("1000", message = "1000 is too high, try again."),
    answer("100,000", message = "100,000 is way too high, try again."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  )
)
```

## Summary

Congratulations on completing the Reliability Testing tutorial!

In this tutorial, we've introduced the concepts of Reliability Growth Analysis and Accelerated Life Testing. You have learned to calculate and visualize reliability using different types of models.

### To Get Help

To get help with a specific function in ReliaGrowR, type a question mark before the function name.

```{r echo=TRUE, eval=FALSE}
?rga
```

Similarly, to get help with a specific function in WeibullR.ALT, type:

```{r echo=TRUE, eval=FALSE}
?alt.make
```

For more help with the ReliaGrowR package, type:

```{r echo=TRUE, eval=FALSE}
help(package="ReliaGrowR")
```

And for more help with the WeibullR.ALT package, type:

```{r echo=TRUE, eval=FALSE}
help(package="WeibullR.ALT")
```

### Additional Resources

- The ReliaGrowR package documentation: <https://cran.r-project.org/package=ReliaGrowR>
- The WeibullR.ALT package documentation: <https://cran.r-project.org/package=WeibullR.ALT>
- The WeibullR package documentation: <https://cran.r-project.org/package=WeibullR>

### References

- Abernethy, R.B. (2004). The New Weibull Handbook. Fifth Edition.

- Aden-Buie G, Schloerke B, Allaire J, Rossell Hayes A (2023). learnr: Interactive Tutorials for R. https://rstudio.github.io/learnr/, https://github.com/rstudio/learnr.

- Crow, Larry H. (1975). Reliability Analysis for Complex, Repairable Systems. Army Material Systems Analysis Activity, 40. <https://apps.dtic.mil/sti/citations/ADA020296>

- Fasola S, Muggeo VMR, Kuchenhoff K. (2018). A heuristic, iterative algorithm for change-point detection in abrupt change models. Computational Statistics, 33, 997-1015.

- Govan P (2024). ReliaGrowR: Reliability Growth Analysis. R package version 0.1,
<https://cran.r-project.org/package=ReliaGrowR>.

- Guo, H., Mettas, A., Sarakakis, G. and Niu, P., (2010). Piecewise NHPP models with maximum likelihood estimation for repairable systems. 2010 Proceedings - Annual Reliability and Maintainability Symposium (RAMS), 1-7. <https://doi.org/10.1109/RAMS.2010.5448029>.
  
- Duane J. T. (1964). Learning Curve Approach to Reliability Monitoring. IEEE Transactions on Aerospace, 2, 563-566, <https://doi.org/10.1109/TA.1964.4319640>.

- Meeker W. Q., Escobar L. A., (1998). Statistical Methods for Reliability Data, Wiley-Interscience, New York.

- Nelson W, (1990). Accelerated Testing, Wiley-Interscience, New York.

- Silkworth, David. (2020). WeibullR: An R Package for Weibull Analysis for Reliability Engineers. 43-53. <https://doi.org/10.35566/isdsa2019c3>.

- Silkworth D, Symynck J (2022). WeibullR: Weibull Analysis for Reliability Engineering. R package version 1.2.1, <https://CRAN.R-project.org/package=WeibullR>.

- Silkworth D (2022). WeibullR.ALT: Accelerated Life Testing Using 'WeibullR'. R package version 0.7.2, <https://CRAN.R-project.org/package=WeibullR.ALT>.

---
title: "An Introduction to Life Data Analysis"
output: 
  learnr::tutorial:
    progressive: true
    allow_skip: true
runtime: shiny_prerendered
---

```{r setup, include=FALSE}
library(learnr)
library(WeibullR)
```

## Introduction

Welcome to the tutorial on **Life Data Analysis**! In this tutorial, you will learn about the basics of life data analysis and how it applies to various fields, such as engineering, manufacturing, and many others. 

## Learning Objectives

By the end of this module, learners will be able to:

* Describe the purpose of Weibull analysis in reliability engineering.
* Differentiate between types of data censoring, including right-censored and interval-censored data.
* Differentiate between different Weibull models (2-parameter Weibull, 3-parameter Weibull, and Weibayes).
* Apply Median Rank Regression (MRR) and Maximum Likelihood Estimation (MLE) estimation methods to sample datasets.
* Interpret results using plotting methods, including probability plots and contour plots.

## What is Life Data Analysis?

Life data analysis is the study of how systems function over time. Life data helps explain how long a system will last, when it will likely fail, and how often it will need maintenance. For example, in the manufacturing industry, life data analysis can be used to understand the lifespan of machines and equipment, helping to optimize maintenance schedules and reduce downtime.

## Why is Life Data Analysis Important?

Understanding life data analysis helps in predicting the lifespan and maintenance needs of various systems, thereby improving reliability and efficiency. By collecting and modeling life data, we can make predictions about how long machines will last or how long before they will need to be repaired or replaced. 

## Life Distributions

### The Weibull Distribution

The Weibull distribution is a continuous probability distribution used in life data analysis and many other fields. Life data analysis is often referred to as Weibull analysis because the Weibull distribution is commonly used to model life data. While other distributions can be used to model life data, this tutorial focuses on the Weibull distribution because of its wide application.

### Why the Weibull Distribution?

The Weibull is a **flexible** distribution that can fit many different types of data. By adjusting the distribution parameters, the Weibull can mimic other distributions such as the normal, exponential, and log-normal distributions.

### Mimicking Other Distributions

To illustrate, take the following normal distribution. The normal distribution is another common distribution used in statistics. The normal is symmetric and characterized by its mean and standard deviation.

```{r echo=FALSE}
x <- seq(0, 10, by = .1) # Create a sequence of x values
y <- dnorm(x, mean = 5, sd = 1.5) # Calculate the normal density function
plot(x,y, type = "l", col = "blue", xlab = "Variable", ylab = "Probability") # Plot the normal distribution
```

The Weibull can *mimic* the normal distribution by adjusting the shape and scale parameters. Adjust the scale parameter in the slider until the Weibull below looks similar to the Normal above.

```{r, echo=FALSE}
sliderInput("eta", "Scale:", min = 1, max = 10, value = 1, step = 0.5) # Slider for eta
plotOutput("distPlot") # Plot output
```

```{r, context="server"}
# Plot the Weibull distribution
output$distPlot <- renderPlot({
  x <- seq(0, 10, by = .1) # Create a sequence of x values
  y <- dweibull(x, shape = 3.67, scale = input$eta) # Calculate the Weibull density function
  plot(x,y, type = "l", col = "red", xlab = "Variable", ylab = "Probability") # Plot the Weibull distribution
})
```

<div id="filter-hint">
**Hint:** Set Eta equal to 5 as a starting point.
</div>

By adjusting the scale parameter, the Weibull can closely resemble the normal distribution.

### The Cumulative Distribution Function

In life data analysis, the cumulative distribution function (CDF) describes the probability of survival (or failure) prior to a certain time *t*. In other words, the CDF gives the probability that a unit will survive (or fail) up to time *t*.



For the Weibull distribution, the CDF is given by:

$$ R(t) = 1 - F(t) = e^{-(t/\eta)^\beta} $$

Where *R(t)* is the cumulative probability of *survival* prior to  time *t*,  *F(t)* is the cumulative probability of *failure* prior to time *t*, $\beta$ is the shape parameter and $\eta$ is the scale parameter.

Notice how *R(t)* and *F(t)* are complementary probabilities, meaning that they add up to 1. For example, if the probability of failure prior to time *t* is 30%, then the probability of survival prior to time *t* is 70%.

For the Weibull distribution, $\beta$ has a physical interpretation related to the failure rate, or the rate at which failures occur over time.

-   $\beta$ \< 1 - the failure rate is decreasing
-   $\beta$ = 1 - the failure rate is constant
-   $\beta$ \> 1 - the failure rate is increasing

By estimating $\beta$ from life data, we can gain insights into the failure behavior of the system being analyzed.

```{r quiz1, echo=FALSE}
quiz(caption = "Quiz: The Weibull Parameters",
  question("Which beta indicates an increasing failure rate?",
    answer("1", message = "Beta of 1 indicates a constant failure rate."),
    answer("0.5", message = "Beta less than 1 indicates a decreasing failure rate."),
    answer("2", correct = TRUE, message = "Correct! Beta greater than 1 indicates an increasing failure rate."),
    answer("0", message = "Beta cannot be 0."),
    random_answer_order = TRUE, 
    allow_retry = TRUE
  ),
  question("What does the parameter eta represent?",
    answer("shape parameter", message = "Eta is the scale parameter, not the shape parameter."),
    answer("scale parameter", correct = TRUE, message = "Correct! Eta is the scale parameter."),
    answer("failure rate", message = "Eta is not the failure rate."),
    answer("mean time between failures", message = "Eta is not the mean time between failures."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("What is the cumulative probability when t = eta?",
    answer("0%", message = "At t = eta, the cumulative probability is not 0%."),
    answer("50%", message = "At t = eta, the cumulative probability is not 50%."),
    answer("63.2%", correct = TRUE, message = "Correct! At t = eta, the cumulative probability is 63.2%."),
    answer("100%", message = "At t = eta, the cumulative probability is not 100%."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("What is the cumulative probability when t = 0?",
    answer("0%", correct = TRUE, message = "Correct! At t = 0, the cumulative probability is 0%."),
    answer("50%", message = "At t = 0, the cumulative probability is not 50%."),
    answer("63.2%", message = "At t = 0, the cumulative probability is not 63.2%."),
    answer("100%", message = "At t = 0, the cumulative probability is not 100%."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  )
)
```

## The Weibull Parameters

### Beta
 
Beta is the **shape** parameter of the Weibull distribution. Changing Beta changes 
the **shape** of the cumulative distribution function. To illustrate, use the slider 
below to adjust the shape of the Weibull. 

```{r, echo=FALSE}
sliderInput("beta", "Beta:", min = 0.5, max = 5, value = 1, step = 0.5) # Slider for beta
plotOutput("cumPlot") # Plot output
```

```{r, context="server"}
# Plot the cumulative distribution function
output$cumPlot <- renderPlot({
  x <- seq(0, 50, by = .1) # Create a sequence of x values
  plot(pweibull(x, shape = input$beta, scale = 10), col = "blue", type = "l", 
       xlab = "Variable", ylab = "Cumulative Probability") # Plot the CDF
})
```

As you adjust Beta, notice how the shape of the Weibull changes. Increasing Beta 
makes the curve steeper, indicating an increasing failure rate. Decreasing Beta 
makes the curve less steep, indicating a decreasing failure rate.

### Eta

Eta is the **scale** parameter of the Weibull distribution. Changing Eta changes 
the **scale** of the cumulative distribution function. To illustrate, use the slider 
below to adjust the scale of the Weibull. 

```{r, echo=FALSE}
sliderInput("eta2", "Eta:", min = 1, max = 20, value = 1) # Slider for eta
plotOutput("cummPlot2") # Plot output
```

```{r, context="server"}
# Plot the cumulative distribution function
output$cummPlot2 <- renderPlot({
  x <- seq(0, 50, by = .1) # Create a sequence of x values
  plot(pweibull(x, shape = 1, scale = input$eta2), col = "red", type = "l", xlab = "Variable", ylab = "Cumulative Probability") # Plot the CDF
})
```

As you adjust Eta, notice how the scale of the Weibull changes. Increasing Eta 
shifts the curve to the right, indicating a longer lifespan. Decreasing Eta shifts the
curve to the left, indicating a shorter lifespan. 

## WeibullR

### Getting Started with WeibullR

For this tutorial, we will use WeibullR: an R package for Weibull analysis.

First, check if WeibullR is installed in R and install if not.

```{r eval=FALSE}
pak::pkg_install("WeibullR")
```

As an example of Weibull analysis, suppose a factory has 5 machines that fail at different times. The factory wants to understand the reliability of the machines over time and predict when future failures may occur. The factory collects the failure data and uses Weibull analysis to model the reliability of the machines. The failure times are 30, 49, 82, 90, and 96 respectively. 

To run Weibull analysis in R, first create a vector with the failure data, then use the `MLEw2p` function with the failure data to fit a Weibull to the data. The argument `bounds=TRUE` adds confidence bounds to the fit, and `show=TRUE` displays the results.

```{r echo=TRUE}
failures<-c(30, 49, 82, 90, 96)
fit<-MLEw2p(failures, bounds=TRUE, show=TRUE)
```

### The Weibull Probability Plot

The Weibull Probability Plot above shows the time (or equivalent) on the horizontal axis and the cumulative probability of failure or unreliability on the vertical axis. We know from the cumulative distribution function that the complement of the failure probability is the survival probability or reliability. For example, if the unreliability at time 50 is 20%, then the reliability at time 50 is 100%-20% = 80%.

Note that the horizontal axis on a probability plot is log scale, and the vertical axis is log(log(1-p)) scale, where p is the cumulative probability of failure. By using this scale, the resulting plot of the Weibull data appears linear.

```{r quiz2, echo=FALSE}
quiz(caption = "Quiz: Interpreting the Probability Plot",
  question("In the previous example, what is the approximate unreliability at time 100?",
    answer("20%", message = "20% is too low."),
    answer("80%", message = "80% is slightly too low."),
    answer("10%", message = "10% is the reliability, not the unreliability."),
    answer("90%", correct = TRUE, message = "Correct! The unreliability at time 100 is approximately 90%."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("In the previous example, what is the approximate reliability at time 50?)",
    answer("20%", correct = TRUE, message = "Correct! The reliability at time 50 is approximately 20%."),
    answer("80%", message = "80% is the unreliability, not the reliability."),
    answer("10%", message = "10% is slightly too low."),
    answer("90%", message = "90% is too high."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("In the previous example, what is the approximate beta of the fit?",
    answer("1.5", message = "1.5 is too low."),
    answer("3.5", message = "3.5 is slightly too high."),
    answer("0.5", message = "0.5 is too low."),
    answer("3.2", correct = TRUE, message = "Correct! The beta of the fit is approximately 1.5."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("In the previous example, what is the approximate eta of the fit?",
    answer("77.81", correct = TRUE, message = "Correct! The eta of the fit is approximately 77.81."),
    answer("88.84", message = "88.84 is slightly too high."),
    answer("37.89", message = "37.89 is too low."),
    answer("389.7", message = "389.7 is too high."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  )
)
```

## Data Censoring

In life data analysis, data is often **censored**. Censoring occurs when the exact time to failure is not known for all units. There are several types of censoring, including right censoring and interval censoring.

### Right Censored Data

Right censored data includes **suspensions** or units that have operated for a period of time without failure. Suspensions are common in life data analysis because not all units fail during the analysis period.

To illustrate right censored data, suppose that the factory in the previous example has 3 additional machines that did not fail during the analysis period. The machines were suspended (i.e. removed from service) at times 100, 45, and 10 respectively.

To add suspensions to the previous example, create a vector with the suspension data and add it to the `MLEw2p` function.

```{r echo=TRUE}
suspensions<-c(100, 45, 10)
fit<-MLEw2p(failures, suspensions, bounds=TRUE, show=TRUE)
```

Note how the fit has changed slightly after adding suspensions. The fit now has a lower beta and higher eta. 

### Interval Censored Data

Interval censored data is data where the exact failure times are unknown, but are known to have occurred within certain intervals. An example of interval censored data is inspection data, where parts or machines are inspected periodically for failures. If a failure is detected during an inspection, the exact failure time is unknown, but it is known to have occurred between the last inspection time and the current inspection time.

To illustrate interval censored data, let's look at an example from Silkworth, 2020. The data consists of part cracks detected during periodic inspections. The left and right columns represent the inspection intervals, and the qty column represents the number of cracks detected in each interval. 

```{r echo=TRUE}
inspection_data<-data.frame(left=c(0, 6.12, 19.92, 29.64, 35.4, 39.72, 45.32, 52.32),
                            right=c(6.12, 19.92, 29.64, 35.4, 39.72, 45.32, 52.32, 63.48),
                            qty=c(5, 16, 12, 18, 18, 2, 6, 17))
```

A number of units did not fail during the inspection period. These units are considered suspensions and are added to the data set as a single entry with the time equal to the last inspection time, event equal to 0 (suspension), and qty equal to the number of suspensions.

```{r echo=TRUE}
suspensions<-data.frame(time=63.48, event=0, qty=73)
```

To run the interval censored example in WeibullR, use the `wblr` function with the suspension data and the `interval` argument to specify the inspection data. Then use the `wblr.fit` function with the `method.fit="mle"` argument to fit the model using maximum likelihood estimation. Next, use the `wblr.conf` function with the `method.conf="fm"` argument to add confidence bounds using the Fisher matrix method. The `col` and `lty` arguments specify the color and line type of the fit and confidence bounds respectively. Use the `plot` function to create the probability plot.

```{r echo=TRUE, warning=FALSE}
obj1<-wblr(suspensions, interval = inspection_data)
obj1<-wblr.fit(obj1, method.fit="mle", col="red") 
obj1<-wblr.conf(obj1, method.conf="fm", lty=2)
plot(obj1)
```

Instead of points, the interval censored data is shown as black horizontal lines representing the intervals where failures occurred. The solid red line is the model fit and the red dashed lines are the confidence bounds.

### Grouped Data

Life data is often compiled into groups (i.e. a group of machines failed or survived at a certain time). For example, in the interval censored example above, the inspection data is grouped into intervals with multiple failures in each interval. Likewise, the suspensions are grouped into a single entry with 73 suspensions at the last inspection time.

Let's look at the inspection data again. The knitr::kable function below displays the inspection data in a table format, and the head function shows the first few rows of the data.

``` {r echo=TRUE}
knitr::kable(
  head(inspection_data)
)
```

The `qty` column indicates how many units failed in each interval. For example, 5 units failed between time 0 and 6.12, 16 units failed between time 6.12 and 19.92, and so on. 

```{r quiz3, echo=FALSE}
quiz(caption = "Quiz: Interval Censored Data",
  question("In the interval censored example above, how many units are in the group of suspensions that did not fail?",
    answer("73", correct = TRUE, message = "Correct! There are 73 units in the group of suspensions that did not fail."),
    answer("17", message = "17 is the number of failures in the last interval."),
    answer("94", message = "94 is the total number of units (failures + suspensions)."),
    answer("63.48", message = "63.48 is the time of the last inspection."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("In the interval censored example, how many total failures occurred?",
    answer("73", message = "73 is the number of suspensions that did not fail."),
    answer("17", message = "17 is the number of failures in the last interval."),
    answer("94", correct = TRUE, message = "Correct! There are a total of 94 failures."),
    answer("63.48", message = "63.48 is the time of the last inspection."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("In the interval censored example, how many total units are in the data set?",
    answer("73", message = "73 is the number of suspensions that did not fail."),
    answer("17", message = "17 is the number of failures in the last interval."),
    answer("94", message = "94 is the total number of failures."),
    answer("167", correct = TRUE, message = "Correct! There are a total of 167 units (failures + suspensions)."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("In the interval censored example, how many failures occurred between time 19.92 and 35.4?",
    answer("12", message = "12 is the number of failures between time 6.12 and 19.92."),
    answer("18", correct = TRUE, message = "Correct! There are 18 failures between time 19.92 and 35.4."),
    answer("16", message = "16 is the number of failures between time 0 and 6.12."),
    answer("2", message = "2 is the number of failures between time 39.72 and 45.32."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  )
)
```

## Parameter Estimation Methods

Parameter estimation methods are used to estimate the parameters of a probability distribution (e.g. Beta and Eta for the Weibull distribution) based on the observed data. The two most common methods are Maximum Likelihood Estimation (MLE) and Median Rank Regression (MRR).

### Maximum Likelihood vs Rank Regression

-   **Median Rank Regression (MRR)** is based on estimating the parameters of the distribution by minimizing the sum of squared errors between the observed and predicted values. The MRR method is also known as the least squares method. 

-   **Maximum Likelihood Estimation (MLE)** is based on estimating the parameters of the distribution by maximizing the likelihood function. The MLE method finds the parameters that make the observed data most probable.

This tutorial does not go into the mathematical details of MRR and MLE, but both methods are widely used in life data analysis. WeibullR includes functions for both methods, such as the `MRRw2p`, `MRRw3p` functions for MRR and the `MLEw2p`, `MLEw3p` functions for MLE.

Let's rerun the right censored example and compare the MRR and MLE methods using the `wblr` function. In the first step, fit the MRR and model using the `wblr.fit` function with the `method.fit` argument set to "rr" for MRR. In the second step, fit the MLE model using the `method.fit` argument set to "mle" for MLE. Finally, use the `plot.wblr` function to create a multi-plot comparing both models.

```{r echo=TRUE}
MRRfit <- wblr.fit(wblr(failures, suspensions, col="blue"), method.fit="rr")
MLEfit <- wblr.fit(wblr(failures, suspensions, col="red"), method.fit="mle")
plot.wblr(list(MRRfit, MLEfit))
```

The blue model is the MRR fit and the red model is the MLE fit. While both models fit the data well,
there are some slight differences in the parameters and unreliability.

```{r quiz4, echo=FALSE}
quiz(caption = "Quiz: MRR vs MLE",
  question("What is the difference between the MRR and MLE fits (select all that apply)?",
    answer("The MLE fit has a higher beta", correct = TRUE),
    answer("The MRR fit has a higher beta"),
    answer("The MLE fit results in a higher unreliability at time 30"),
    answer("The MRR fit results in a higher unreliability at time 30", correct = TRUE),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("In the previous example, which fit has a slightly higher unreliability at time 100?",
    answer("MRR", message = "The MRR fit has a slightly lower unreliability at time 100."),
    answer("MLE", correct = TRUE, message = "The MLE fit has a slightly higher unreliability at time 100."),
    answer("Both are the same", message = "The fits are slightly different at time 100."),
    answer("Cannot be determined from the plot", message = "The unreliability can be determined from the plot."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  )
)
```

## Other Weibull Models

### The WeiBayes Model

A WeiBayes or one parameter (1P) Weibull has a fixed $\beta$ or shape parameter based on *prior* knowledge or experience. Weibayes is a Bayesian approach to Weibull analysis, where prior knowledge is combined with observed data to estimate the parameters of the Weibull distribution. Weibayes is useful when there is limited failure data or when the data does not fit a standard Weibull model well.

As an example, let's rerun the right censored example of a machine that failed at times 30, 49, 82, 90, and 96. In this case, the data is limited, so a WeiBayes model may be appropriate. First, recreate the failure and suspension data. Then fit the WeiBayes model while assuming a Beta of 2 and plot the results. In the code below, the `method.fit` argument specifies the WeiBayes model, and the `weibayes.beta` argument specifies the fixed Beta value.

```{r echo=TRUE, warning=FALSE}
failures<-c(30, 49, 82, 90, 96)
suspensions<-c(100, 45, 10)
obj <- wblr.fit(wblr(failures, suspensions, col="blue"), method.fit="weibayes", weibayes.beta=2)
plot(obj)
```

Note that confidence bounds are not shown for the WeiBayes model. Confidence bounds for the WeiBayes model are not well defined because the shape parameter is fixed.

### The 3-Parameter Weibull Model

The 3P Weibull has an additional parameter *t~0~* to represent a failure free period. The failure free period is the time before which no failures can occur. Often parts or machines must age, wear, fatigue, etc. before they can actually fail, and the 3P Weibull can model this behavior. The cumulative distribution function for the 3P Weibull is given below:

$$ R(t) = 1 - F(t) = e^-((t-t~0~)/\eta)^\beta $$
Where *t~0~* is the failure free period, $\beta$ is the shape parameter, and $\eta$ is the scale parameter.

Here is an example from "The New Weibull Handbook" by Robert Abernethy (2004). This example consists of 25 failure times from a machine that has a failure free period. Let's run this example as a 3P Weibull. First, create a vector of failure data. Then fit the 3P Weibull model and plot the results. The `dist="weibull3p"` argument specifies the 3P Weibull model.

```{r echo=TRUE}
failures<-c(3.46623, 3.732711, 4.052996, 4.628703, 4.8157, 5.84517, 5.888313, 5.892967,
            8.168362, 10.02799, 10.06062, 10.49785, 11.11493, 11.87369, 12.21122, 12.51854, 
            12.91357, 18.04246, 18.20712, 19.57305, 21.20873, 30.03917, 34.88001, 36.87355,
            53.91168)
fit<- wblr.conf(wblr.fit(wblr(failures), dist="weibull3p"), col="darkgreen")
plot(fit)
```

Note that the 3P Weibull model has a failure free period of approximately 3.3 time units.
The model appears curved at lower probabilities, which is typical of a 3P Weibull.
The curvature is due to the failure free period, which shifts the distribution to the right.

```{r quiz5, echo=FALSE}
quiz(caption = "Quiz: Other Weibull Models",
  question("In the Weibayes example, what is eta?",
    answer("88.84", correct = TRUE, message = "Correct! Eta is approximately 88.84."),
    answer("389.7", message = "389.7 is too high."),
    answer("87.89", message = "87.89 is slightly too low."),
    answer("37.89", message = "37.89 is too low."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("What is the failure free period in the 3P Weibull example?",
    answer("3.3", correct = TRUE, message = "Correct! The failure free period is approximately 3.3 time units."),
    answer("0.85", message = "0.85 is the beta parameter, not the failure free period."),
    answer("11.16", message = "11.16 is the eta parameter, not the failure free period."),
    answer("0", message = "The failure free period is greater than 0."),
    random_answer_order = TRUE
  ),
  question("In the 3P Weibull example, what is beta?",
    answer("0.85", correct = TRUE, message = "Correct! Beta is approximately 0.85."),
    answer("3.3", message = "3.3 is the failure free period, not beta."),
    answer("11.16", message = "11.16 is the eta parameter, not beta."),
    answer("1", message = "Beta is not equal to 1."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("In the 3P Weibull example, what is eta?",
    answer("11.16", correct = TRUE, message = "Correct! Eta is approximately 11.16."),
    answer("0.85", message = "0.85 is the beta parameter, not eta."),
    answer("3.3", message = "3.3 is the failure free period, not eta."),
    answer("1", message = "Eta is not equal to 1."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  )
)
```

## Multi-Plots

### Multi-Plots

Multi-plots are useful for comparing multiple Weibull models on the same plot. In this way, you can easily see the differences between the models and how they fit the data. To build a multi-plot in WeibullR, create multiple `wblr` objects and add them to a list. Then use the `plot.wblr` function to create the multi-plot.

First, recreate the failure data from the first example of a machine that failed at times 30, 49, 82, 90, and 96.

```{r echo=TRUE}
failures<-c(30, 49, 82, 90, 96)
obj1<-wblr.fit(wblr(failures), col="red")
```

Next, recreate the failure and suspension data from the right censored example of a machine that failed at times 30, 49, 82, 90, and 96 with suspensions at times 100, 45, and 10.

```{r echo=TRUE}
failures<-c(30, 49, 82, 90, 96)
suspensions<-c(100, 45, 10)
obj2<-wblr.fit(wblr(failures, suspensions, col="purple"))
```

Finally, add the 2 `wblr` objects to a list and plot both objects in a single chart.

```{r echo=TRUE}
plot.wblr(list(obj1, obj2))
```

The red model is the fit without suspensions and the purple model is the fit with suspensions. 
Note how the fit with suspensions has a slightly lower beta and higher eta.

```{r quiz6, echo=FALSE}
quiz(caption = "Quiz: Multi-Plots",
  question("What is the difference between obj1 and obj2 in the previous example (select all that apply)?",
    answer("obj1 includes suspensions"),
    answer("obj1 has a fit with a lower beta"),
    answer("obj2 includes suspensions", correct=TRUE),
    answer("obj2 has a fit with a lower beta", correct=TRUE),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("In the previous example, which fit has a higher unreliability at time 100?",
    answer("obj2", message = "obj2 has a slightly lower unreliability at time 100."),
    answer("obj1", correct = TRUE, message = "Correct! obj1 has a slightly higher unreliability at time 100."),
    answer("Both are the same", message = "The fits are slightly different at time 100."),
    answer("Cannot be determined from the plot", message = "The unreliability can be determined from the plot."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  )
)
```

## Competing Failure Modes

A part or a system may fail in different ways. A failure mode is a specific way in which a part or machine can fail. For example, a machine may fail due to corrosion, fatigue, or overload. Each failure mode may have a different distribution of time to failure. Competing failure modes occur when multiple failure modes are present in the same data set. 

When competing failure modes are present, a basic Weibull model tends to fit the data poorly. This is because the basic Weibull assumes that all failures are due to a single failure mode. 

To illustrate, let's simulate some data with 3 different failure modes, where each failure mode has a different beta and eta. The data.frame below contains 120 observations, where 20 observations are failures and 100 observations are suspensions. The failure_mode column indicates the failure mode for each observation.

```{r echo=TRUE}
set.seed(123)
data <- data.frame(
  time = c(
    rweibull(5, 0.5, 20), # Simulate times for Failure Mode A
    rweibull(10, 1, 10), # Simulate times for Failure Mode B
    rweibull(5, 2, 5),  # Simulate times for Failure Mode C
    rweibull(100, 2, 10) # Simulate times for Suspensions
    ),
  event = c(
    rep(1, 20), # Label Failures
    rep(0, 100) # Label Suspensions
    ),
  failure_mode = c(
    rep("A", 5), # Label Failure Mode A
    rep("B", 10), # Label Failure Mode B
    rep("C", 5), # Label Failure Mode C
    rep("", 100)) # No label for Suspensions
  )
```

Next, fit a basic Weibull model to the combined data and plot the results.

```{r echo=TRUE}
obj <- wblr.conf(wblr.fit(wblr(data)))
plot(obj, col="darkgreen", is.plot.legend = FALSE)
```

When the data is combined, the overall fit has a beta of approximately 1, which indicates random failures. However, we know that the data actually contains 3 different failure modes.

Now let's create a separate data set for each failure mode. The code below creates 3 copies of the original data set and sets the event column to 0 for all observations that do not belong to the respective failure mode.

```{r echo=TRUE}
dat1 <- data
dat2 <- data
dat3 <- data
dat1$event[dat1$failure_mode != "A"] <- 0
dat2$event[dat2$failure_mode != "B"] <- 0
dat3$event[dat3$failure_mode != "C"] <- 0
```

Finally, fit a `wblr` object for each failure mode and create a multi-plot. The `is.plot.legend=FALSE` argument removes the legend from the plot.

```{r echo=TRUE}
obj1 <- wblr.fit(wblr(dat1, col="blue"))
obj2 <- wblr.fit(wblr(dat2, col="red"))
obj3 <- wblr.fit(wblr(dat3, col="orange"))
plot.wblr(list(obj1, obj2, obj3), is.plot.legend = FALSE)
```

Note how each failure mode has a different beta and eta. Failure mode A (blue) has a beta less than 1, which means the failure rate is decreasing. Failure mode B (red) has a beta greater than 1, which means that the failure rate is increasing. Failure mode C (orange) has a beta close to 1, which means that the failure rate is approximately constant.

## Contour Plots

Another way to compare Weibull models is with contour plots. A contour plot is a graphical representation of the relationship between the Weibull parameters Beta and Eta. 
 
Once again, let's look at the first example of a machine that failed at times 30, 49, 82, 90, and 96. First, recreate the failure data. Then fit the Weibull model and build the contour plot at a probability level of 90%. The `wblr.conf` function with the `method="lrb"` argument computes the likelihood ratio bounds for the contour plot, and the `plot_contour` function creates the contour plot. The `CL=0.9` argument specifies a probability level of 90%.

```{r echo=TRUE, results='hide'}
failures<-c(30, 49, 82, 90, 96)
obj <- wblr.conf(wblr.fit(wblr(failures, col="blue"), method="mle"), method="lrb")
plot_contour(obj, CL=0.9)
```

The Weibull contour plot shows the confidence region for the Weibull parameters Beta and Eta at a probability level of 90%. The x-axis represents Eta, the y-axis represents Beta, and the contour lines represent the confidence levels. 

### Comparing Contour Plots

To compare multiple Weibull models, we can build contour plots for each model and overlay them on the same chart. If the models are similar, then the contours will overlap. If the models are different, then the contours will not overlap.

To visualize this concept, let's revisit the previous competing failure mode example. First, recreate the failure data with 3 different failure modes. Then create a separate `wblr` object for each failure mode and build the contour plot for each model at a probability level of 90%. The contours are then overlaid on the same chart. Note the use of the `xlim` argument to set the x-axis limits for better visualization.


```{r echo=TRUE, results='hide'}
obj1 <- wblr.conf(wblr.fit(wblr(dat1, col="blue"), method="mle"), method="lrb")
obj2 <- wblr.conf(wblr.fit(wblr(dat2, col="red"), method="mle"), method="lrb")
obj3 <- wblr.conf(wblr.fit(wblr(dat3, col="orange"), method="mle"), method="lrb")
plot_contour(list(obj1, obj2, obj3), CL=0.9, xlim = c(1, 1500))
```

The blue, red, and orange contours represent the 3 different failure modes. Note how the contours overlap, which indicates that the failure modes are not significantly different. If the contours did not overlap, then the failure modes would be significantly different.

```{r quiz7, echo=FALSE}
quiz(caption = "Quiz: Contour Plots",
  question("In the single contour plot example, what is the approximate beta at the center of the contour?",
    answer("Between 2 and 3", message = "3 is slightly too low."),
    answer("Between 75 and 80", message = "The approximate eta is between 75 and 80."),
    answer("Between 4 and 4.5", message = "4 is slightly too high."),
    answer("Between 3 and 3.5", correct = TRUE, message = "Correct! The approximate beta at the center of the contour is between 3 and 3.5."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("In the multi-contour plot example, at a probability level of 90%...",
    answer("obj3 (orange) is significantly different than the others", message = "obj3 (orange) overlaps with obj1 (blue) and obj2 (red), so they are not significantly different."),
    answer("obj2 (red) is significantly different than the others", message = "obj2 (red) overlaps with obj3 (orange), so they are not significantly different."),
    answer("None of the distributions are significantly different", correct=TRUE, message = "Correct! All of the distributions overlap, so they are not significantly different."),
    answer("All of the distributions are significantly different", message = "The distributions overlap, so they are not significantly different."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  )
)
```

## Summary

Congratulations on completing the tutorial on life data analysis!

In this tutorial, we've introduced the concepts of life data analysis and reliability. You have learned to calculate reliability using the Weibull distribution and visualize life data.

### To Get Help

To get help with a specific function in WeibullR, type a question mark before the function name.

```{r echo=TRUE, eval=FALSE}
?MRRw2p
```

For more help with the WeibullR package.

```{r echo=TRUE, eval=FALSE}
help(package="WeibullR")
```

### Additional Resources

- The WeibullR package documentation: <https://cran.r-project.org/package=WeibullR>

### References

- Abernethy, R.B. (2004) The New Weibull Handbook. Fifth Edition.

-   Aden-Buie G, Schloerke B, Allaire J, Rossell Hayes A (2023). learnr: Interactive Tutorials for R. https://rstudio.github.io/learnr/, https://github.com/rstudio/learnr.

-   Silkworth, David. (2020). WeibullR: An R Package for Weibull Analysis for Reliability Engineers. 43-53. <https://doi.org/10.35566/isdsa2019c3>.

-   Silkworth D, Symynck J (2022). *WeibullR: Weibull Analysis for Reliability Engineering*. R package version 1.2.1, <https://CRAN.R-project.org/package=WeibullR>.

---
title: 'ReliaLearnR: Learning Modules for Reliability Analysis'
tags:
  - R
  - life data analysis
  - Weibull analysis
  - reliability engineering
  - availability
  - maintainability
  - reliability testing
  - accelerated life testing
authors:
  - name: Paul B. Govan
    orcid: 0000-0002-1821-8492
    affiliation: 1
affiliations:
 - name: Senior Reliability Systems Engineer, Safety and Reliability, GE Aerospace
   index: 1
date: 31 August 2025
bibliography: paper.bib

---

# Statement of Need

Reliability analysis is an important aspect of engineering that focuses on ensuring
systems perform as expected over time. Key components of reliability analysis include
life data analysis, reliability, availability, and maintainability (RAM) analysis,
and reliability testing. These concepts are essential for engineers and professionals
involved in product design, manufacturing, and maintenance. However, many learning resources
for these topics rely on proprietary software, which can be expensive and inaccessible
to students and early-career professionals. 

`ReliaLearnR` [@ReliaLearnR] addresses this gap by providing an open-source
framework for learning reliability analysis using R [@R], a widely-used programming
language for statistical computing and data analysis. The primary objective of this
project is to introduce fundamental concepts of reliability analysis while providing
an open-source alternative for analyzing reliability data. The target audience for
this project includes engineering students and professionals who are interested
in learning the fundamentals of reliability analysis.

# Design

`ReliaLearnR` is written in R [@R] and utilizes `WeibullR` [@WeibullR] for Life Data 
Analysis [@Weibull], `WeibullR.alt` [@WeibullRalt] for Accelerated Life Testing [@WeibullRalt], 
`ReliaGrowR` [@ReliaGrowR] for Reliability Growth Analysis, and `learnr` 
[@learnr], a framework for building interactive learning modules.  

The learning modules are designed to be interactive and engaging, with a
focus on practical applications. Each module includes a mix of instructional content, 
code examples, and exercises to reinforce learning. The modules are self-paced,
allowing learners to progress at their own speed. 

The original learning modules were provided in a series of workshops, where each 
workshop covered a specific module over a 1-2 hour period. These workshops were 
designed to be completed in a classroom setting with an instructor. The current version 
of the modules has been adapted for self-paced learning, but they can still be used 
in a classroom setting with an instructor. 

To adopt the modules for classroom use, instructors can either access them 
via the project website or install the package and use the functions directly. 
Instructors can also modify the modules to fit their specific needs, as the source code is 
available on the project repository. 

# Usage

The package includes three interactive learning modules, which can be run locally or
hosted on a web server. Users can run the modules in any R environment 
by calling the `ram()`, `lda()`, or `rt()` functions.

`ram()` is a quick reference for common Reliability, Availability, and Maintainability
(RAM) concepts. The learning objectives include defining key reliability metrics, 
including reliability, availability, and failure rate, Describing the significance 
of MTTR, MTTF, and MTBF in reliability engineering, calculating probability of failure 
using given reliability data, interpreting $B_n$ or $L_n$ life values in the context 
of product reliability, differentiating between different reliability measures. 

`lda()` provides an introduction to Life Data Analysis. 
The learning objectives include describing the purpose of Weibull analysis in reliability 
engineering, differentiating between types of data censoring, including right-censored 
and interval-censored data, differentiating between different Weibull models (2-parameter 
Weibull, 3-parameter Weibull, and Weibayes, applying Median Rank Regression (MRR) 
and Maximum Likelihood Estimation (MLE) estimation methods to sample datasets, interpreting
results using plotting methods, including probability plots and contour plots. 

`rt()` provides an introduction to Reliability Testing. The 
learning objectives include defining key reliability growth concepts, including 
Crow-AMSAA and Duane models, fitting a reliability growth model to real-world data
using R, interpreting reliability growth plots and identifying trends, applying 
the Crow-AMSAA model to assess reliability growth, explaining fundamental concepts
of accelerated life testing, including the use of Arrhenius and Power Law Models, 
conducting an accelerated life test with real-world datasets, utilizing R for analysis,
analyzing plots that illustrate the relationships in accelerated life testing, identifying
key patterns and data trends, and utilizing Arrhenius and Power Law models to evaluate 
the impact of stress factors on product reliability. 

The modules can also be accessed in a browser at 
[paulgovan.shinyapps.io/RAMAnalysis/](https://paulgovan.shinyapps.io/RAMAnalysis/), 
[paulgovan.shinyapps.io/LifeDataAnalysis/](https://paulgovan.shinyapps.io/LifeDataAnalysis/), 
and [paulgovan.shinyapps.io/ReliabilityTesting/](https://paulgovan.shinyapps.io/ReliabilityTesting/).

![](https://github.com/paulgovan/ReliaLearnR/blob/master/inst/paper/ReliaLearnR.png?raw=true)<!-- -->

The package also includes several helper functions for common RAM calculations. These functions
make it easy to apply the concepts covered in the RAM module.

* `rel()` - reliability function
* `avail()` - availability function
* `mttf()` - mean time to failure
* `mtbf()` - mean time between failure
* `fr()` - failure rate

The project documentation includes installation instructions for `ReliaLearnR`
and the required dependencies, examples of running the programs, and references 
to previous work used to build the modules. The documentation also references more
resources for users interest in learning more. These resources include 
`ReliaPlotR` [@ReliaPlotR], an R package for interactive reliability analysis
plots, and `ReliaShinR` [@ReliaShiny], a shiny [@shiny] web application for 
reliability analysis.

Contributions are welcome from the community. Users can submit pull requests, 
report issues, or suggest enhancements through the repository, which includes contributing 
guidelines.

# Motivation

This project began as an effort to build upon a reliability program developed at 
a major technology company. The original program proved to provide a strong foundation, 
providing a structured learning opportunity that helped many early-career professionals 
understand and apply the fundamental concepts of reliability engineering.
Over time, however, the proprietary nature of the program limited accessibility 
and adaptability. 

Recognizing the importance of keeping reliability learning both relevant and accessible, 
this project was initiated to create an open-source framework for teaching reliability 
analysis. By leveraging this framework, this project aims to reach a broader audience, 
encourage collaboration, and ensure that learning resources can evolve as needs and 
priorities change.

# Acknowledgements

The author acknowledges the creators of the original Reliability Program that inspired
this project.

# References

---
title: "Reliability, Availability, and Maintainability (RAM)"
output: 
  learnr::tutorial:
    progressive: true
    allow_skip: true
runtime: shiny_prerendered
---

```{r setup, include=FALSE}
library(learnr)
library(WeibullR.learnr)
```

## Introduction

Welcome to the tutorial on **Reliability, Availability, and Maintainability (RAM)**! In this tutorial, you will learn about these fundamental concepts and their applications.

## Learning Objectives

By the end of this module, learners will be able to:

* Define key reliability metrics, including reliability, availability, and failure rate.
* Describe the significance of MTTR, MTTF, and MTBF in reliability engineering.
* Calculate probability of failure using given reliability data.
* Interpret $B_n$ or $L_n$ life values in the context of product reliability.
* Differentiate between different reliability measures.

## What is RAM?

The terms Reliability, Availability, and Maintainability are related but distinct concepts.

**Reliability**: The ability of an item to perform its intended function without failure over a specified period.

**Availability**: The proportion of time an item is in a functioning condition.

**Maintainability**: The ease and speed with which an item can be restored to operational status after a failure.

These concepts are interrelated in the sense that a highly reliable item is likely to be available more often, and an item that is easy to maintain can be restored to service quickly, thus improving availability.

## Reliability

Reliability is the probability that an item will not fail under defined conditions for a specified period. Specifically, reliability is defined as:

$$ Reliability = (1 - (Failed Time/Total Time))*100 $$
Where Failed Time is the time that the item was not functioning, and Total Time is the total time that the item was in operation.

For example, suppose a motor ran for 5 years total and was failed for 10 of those days. The reliability of the motor is: 

$$ Reliability = (1 - (10/(5*365)))*100 = 94.5% $$
So, the motor has a reliability of 94.5% over the 5-year period.

Unreliability is the opposite of reliability, that is, the probability that an item _will_ fail under defined conditions for a specified period. Unreliability is defined as:

$$ Unreliability = 1 - Reliability = (Failed Time/Total Time)*100 $$
Take the previous example, the unreliability of the motor is:

$$ Unreliability = 1 - 0.945 = 0.055 = (10/(5*365))*100 = 5.5% $$
The motor has an unreliability of 5.5% over the 5-year period.

```{r quiz1, echo=FALSE}
quiz(caption = "Exercise: Reliability", 
  question("A machine has run for 3 years total and was failed for 5 of those days. What is the reliability of the machine?",
    answer("90%", message = "Not quite, try again."),
    answer("0.5%", message = "0.5% is the unreliability, not the reliability."),
    answer("99.5%", correct = TRUE),
    answer("75%", message = "Not quite, try again."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("What is the unreliability of the machine?",
    answer("0.5%", correct = TRUE),
    answer("90%", message = "Not quite, try again."),
    answer("99.5%", message = "99.5% is the reliability, not the unreliability."),
    answer("75%", message = "Not quite, try again."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  )
)
```

## Availability

Availability is the probability that an item will be available for service. Unavailable time includes failed time and scheduled maintenance. Specifically, availability is defined as:

$$ Availability = (1 - (Unavailable Time/Total Time))*100 $$
Where Unavailable Time is the time that the item was not available for service, and Total Time is the total time that the item was in operation.

Unavailable time is different than standby time, where standby time is when the item is available for service, but is not being used. 

Suppose a motor ran for 5 years total, was failed for 10 days, and had scheduled maintenance for 15 days. The availability of the motor is:

$$ Availability = (1 - ((10+15)/(5*365)))*100 = 93.2% $$
So the motor has an availability of 93.2% over the 5-year period.

```{r availability, echo=FALSE}
quiz(caption = "Exercise: Availability",
     question("What is the difference between reliability and availability?",
       answer("Reliability includes scheduled maintenance, availability does not.", message = "Not quite, try again."),
       answer("Availability includes scheduled maintenance, reliability does not.", correct = TRUE),
       answer("There is no difference, they are the same.", message = "There is a slight difference, try again."),
       answer("Reliability is always larger than availability.", message = "Not always, try again."),
       random_answer_order = TRUE,
       allow_retry = TRUE
     ),
  question("A machine has run for 3 years total, was failed for 5 days, and had scheduled maintenance for 14 days. What is the availability of the machine?",
    answer("95%", message = "Not quite, try again."),
    answer("1.7%", message = "1.7% is too low, try again."),
    answer("98.3%", correct = TRUE),
    answer("50%", message = "50% is too low, try again."),
    random_answer_order = TRUE,
  allow_retry = TRUE
  )
)
```

## Mean Time to Repair

The Mean Time to Repair (MTTR) is a measure of maintainability and is defined as the average time required to repair a failed item. The MTTR is the ratio of total repair time by the number of repairs for a given item.

$$ MTTR = \sum_{i=1}^n RepairTime_i/RepairCount $$
Where $RepairTime_i$ is the time to repair for the *i*th repair, and $RepairCount$ is the total number of repairs.

Suppose 100 motors run for 5 years total. During that time, 5 failures occur with repair times in days of 5, 10, 15, 8, and 12. The MTTR is:

$$ MTTR = (5+10+15+8+12)/5 = 10 days $$

```{r quiz3, echo=FALSE}
quiz(caption = "Exercise: MTTR",
  question("What does MTTR stand for?",
    answer("Mean Time to Recovery", message = "Not quite, try again."),
    answer("Mean Time to Repair", correct = TRUE),
    answer("Mean Time to Reliability", message = "Not quite, try again."),
    answer("Mean Time to Reboot", message = "Not quite, try again."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("1000 motors run for 3 years total. During that time, 5 failures occur with repair times in days of 5, 10, 15, 8, and 12. What is the MTTR?",
    answer("15 days", message = "15 days is too high, try again."),
    answer("5 days", message = "5 days is too low, try again."),
    answer("10 days", correct = TRUE),
    answer("8 days", message = "8 days is too low, try again."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  )
)
```

## Mean Time to Failure

The Mean Time to Failure (MTTF) is a measure of the reliability for *non-repairable* items, meaning that once the item fails, it is not repaired, but replaced. The MTTF is the ratio of total time by the number of failures for a given item.

$$ MTTF = Total Time/FailureCount $$
Where *Total Time* is the total time that the item was in operation, and *FailureCount* is the total number of failures.

For example, suppose 100 motors run for 5 years total. During that time, 5 failures occur. The MTTF is:

$$ MTTF = (5*100)/5 = 100 years $$
An MTTF of 100 years means that, on average, a single motor is expected to run for 100 years before failing. This may seem high, but remember that this is an average across all motors, and some motors may fail much sooner while others may last much longer. 

## Mean Time Between Failures

Mean Time Between Failures (MTBF) is a measure of the reliability for *repairable systems*, meaning that once the item fails, it is repaired and put back into service. In this way, the item can fail multiple times. The MTBF is the ratio of total time by the number of failures for a given item.

The calculation is similar to the MTTF, just with a slightly different interpretation. 

Take an example where a motor fails 5 times during a total time of 10,000 hours. The MTBF is:

$$ MTBF = 10000/5 = 2000 hours $$
An MTBF of 2000 hours means that, on average, the motor is expected to run for 2000 hours between failures.

```{r quiz4, echo=FALSE}
quiz(caption = "Exercise: MTTF and MTBF",
  question("What is the difference between MTTF and MTBF?",
    answer("MTTF is for repairable systems, MTBF is for non-repairable systems.", message = "Not quite, try again."),
    answer("MTTF is for non-repairable items, MTBF is for repairable systems.", correct = TRUE),
    answer("There is no difference, they are the same.", message = "There is a slight difference, try again."),
    answer("MTTF is always larger than MTBF.", message = "Not always, try again."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("1000 motors run for 3 years total. During that time, 5 failures occur. What is the MTTF?",
    answer("200 years", message = "200 years is too low, try again."),
    answer("0.6 years", message = "0.6 years is too low, try again."),
    answer("600 years", correct = TRUE),
    answer("500 years", message = "500 years is slightly too low, try again."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("A machine fails 5 times during a total time of 45,000 hours. What is the MTBF of the machine?",
    answer("0.0001 hours", message = "0.0001 hours is the failure rate, not the MTBF."),
    answer("10000 hours", message = "10000 hours is too low, try again."),
    answer("9000 hours", correct = TRUE),
    answer("225,000 hours", message = "225,000 hours is too high, try again."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  )
)
```

## Failure Rate

The failure rate is the ratio of the number of failures by the total operational time for a given item and a specified period. Since the MTBF is the ratio of total time by the number of failures, the failure rate is the inverse of the MTBF. Specifically, the failure rate is defined as:

$$ \lambda = 1/MTBF $$
Where $\lambda$ is the failure rate.

The failure rate can also be calculated directly as the number of failures divided by the total operational time.

$$ \lambda = FailureCount/Total Time $$
Where *Total Time* is the total time that the item was in operation, and *FailureCount* is the total number of failures.

A key assumption of MTBF is that the failure rate is constant, meaning that the item has the same probability of failing at any time during its operation. In practice, this is not always true, as items may have higher failure rates at the beginning or end of their life. However, for many items, the constant failure rate assumption is a reasonable approximation.

To illustrate, suppose 100 motors run for 10 years total. During that time, 20 failures occur. The failure rate is:

$$ \lambda = 20/(10*100) = 0.0055 \text{ failures per year} $$
A failure rate of 0.0055 failures per year means that, on average, there are 0.0055 failures for every motor per year of operation.

## Probability of Failure

Failures do not often occur at predictable times and often occur randomly according to a probability distribution. A common assumption is that failures occur according to a Poisson process, meaning that the number of failures in a given time period follows a Poisson distribution. When failures follow a Poisson process, the time between failures follows an Exponential distribution.

For an exponential distribution, the probability of failure prior to time *t* can be calculated using the cumulative distribution function (CDF) of the exponential distribution. Specifically, the cumulative probability of *failure* prior to time *t*, ***F(t)***, is given by:

$$ F(t) = 1 - e^{-t\lambda} $$
Where $\lambda$ is the failure rate.

The cumulative probability of *survival* prior to time *t*, ***R(t)***, is the opposite of ***F(t)***, that is

$$ R(t) = 1 - F(t) $$
$$ R(t) = e^{-t\lambda} $$

For example, suppose a motor has a failure rate of 0.1 failures per year. The probability of survival at time 10 is:

$$ R(10) = e^{-10*0.1} = 0.3679 = 36.79% $$
So, there is a 36.79% chance that the motor will survive for 10 years without failing or a 63.21% chance that it will fail before 10 years.

## $B_n$ or $L_n$ Life

The $B_n$ life is defined as the time at which *n* % of a population are expected to fail (or 1-*n* % are expected to survive). For example, the B10 life is the time at which 10% of a population are expected to fail, B90 is when 90% are expected to fail, and so on. Some industries use the term $L_n$ life instead of $B_n$ life, but they are the same concept.

To calculate the $B_n$ life, we can solve for time *t* in the cumulative distribution function (CDF) of the exponential distribution. Specifically, we set ***F(t)*** equal to *n* % and solve for *t*.


$$ t = -ln(1-F(t))/\lambda $$

For example, suppose a factor motor has a failure rate of 0.2 failures per year. The B10 life is:

$$ B10 = -ln(1-0.1)/0.2 = 0.526 years $$
So at the B10 life of 0.526 years, 10% of the motors are expected to have failed.

```{r quiz5, echo=FALSE}
quiz(caption = "Exercise: Failure Rate, Probability of Failure, and Bn Life",
  question("What is the relationship between failure rate and MTBF?",
    answer("Failure rate is the square of MTBF.", message = "Not quite, try again."),
    answer("Failure rate is the inverse of MTBF.", correct = TRUE),
    answer("Failure rate is the same as MTBF.", message = "Not quite, try again."),
    answer("Failure rate is the logarithm of MTBF.", message = "Not quite, try again."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("100 motors run for a total time of 5000 hours in a year. Out of these motors, 75 have failures. What is the failure rate in hours?",
    answer("0.75", message = "0.75 is too high, try again."),
    answer("6667", message = "6667 is too high, try again."),
    answer("0.00015", correct = TRUE),
    answer("0.015", message = "0.015 is too high, try again."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("A machine has a failure probability that follows the exponential distribution with a failure rate of 0.1. What is the probability of survival at time 5?",
    answer("39.3%", message = "39.3% is the failure probability, not the survival probability."),
    answer("60.7%", correct = TRUE),
    answer("6.1%", message = "6.1% is too low, try again."),
    answer("93.9%", message = "93.9% is too high, try again."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("A machine has a failure probability that follows the exponential distribution with a failure rate of 0.1. What is the B10?",
    answer("0.105", message = "0.105 is too low, try again."),
    answer("1.05", correct = TRUE),
    answer("60.7%", message = "60.7% is a probability, not a time."),
    answer("39.3%", message = "39.3% is a probability, not a time."),
    random_answer_order = TRUE
  )
)
```

## Summary

Congratulations on completing the Reliability, Availability, and Maintainability (RAM) tutorial!

In this tutorial, we've introduced the concepts of Reliability, Availability, and Maintainability. You have learned to calculate reliability using different assumptions and explored the importance of these concepts in system design.

## References

- Abernethy, R.B. (2004) The New Weibull Handbook. Fifth Edition.

- Aden-Buie G, Schloerke B, Allaire J, Rossell Hayes A (2023). learnr: Interactive Tutorials for R. https://rstudio.github.io/learnr/, https://github.com/rstudio/learnr.

---
title: "Reliability Block Diagrams"
output:
  learnr::tutorial:
    progressive: true
    allow_skip: true
runtime: shiny_prerendered
---

```{r setup, include=FALSE}
library(learnr)
library(DiagrammeR)
```

## Introduction

Welcome to the tutorial on **Reliability Block Diagrams (RBDs)** and System Reliability! In the RAM module, you learned to calculate reliability metrics for individual components. This tutorial extends those concepts to *systems*, collections of components whose arrangement determines whether the system succeeds or fails.

RBDs are a graphical tool for modeling how component reliabilities combine to produce system-level reliability. They are widely used in reliability engineering to analyze designs, identify vulnerabilities, and evaluate the benefit of redundancy.

## Learning Objectives

By the end of this module, learners will be able to:

* Define reliability block diagrams and relate them to physical systems.
* Compute system reliability for series, parallel, and mixed configurations.
* Interpret k-out-of-n (voting) systems and calculate their reliability.
* Describe the relationship between RBDs and Fault Tree Analysis.
* Apply RBD calculations to a realistic multi-component system.

## What is a Reliability Block Diagram?

A **Reliability Block Diagram** represents each component in a system as a block with a known reliability value. The blocks are connected by lines that show how the components relate to each other functionally:

* A **path** from left (input) to right (output) through functioning blocks represents a working system.
* The RBD is *not* a wiring diagram. It shows logical dependencies, not physical connections.

For example, a pump system that requires a motor, a pump, and a valve all to function would be drawn as three blocks in series. A backup generator that can substitute for a primary generator would appear as two blocks in parallel.

RBDs directly connect to the RAM metrics you already know:

* Each block has a reliability $R_i(t) = e^{-\lambda_i t}$ (exponential model) or a Weibull $R_i(t)$ (see the Life Data Analysis module).
* The system reliability is derived from the block reliabilities using the formulas in this tutorial.

## Series Systems

In a **series** system, *all* components must function for the system to function. This is the most common configuration, think of a chain where every link must hold.

$$R_{sys} = R_1 \times R_2 \times \cdots \times R_n = \prod_{i=1}^{n} R_i$$

**Key insight**: A series system is *always less reliable* than its weakest component. Adding more components in series can only reduce system reliability.

### Example

A water pumping system requires three components to all be operational: a motor (R = 0.95), a pump (R = 0.90), and a control valve (R = 0.98).

```{r rbd-series, echo=FALSE}
grViz("
digraph series {
  rankdir = LR
  graph [bgcolor = transparent]
  node [shape = rectangle, style = filled, fillcolor = '#AED6F1',
        fontname = 'sans-serif', fontsize = 12, margin = '0.2,0.1']
  edge [arrowsize = 0.8]
  I [shape = point, width = 0.15, fillcolor = black]
  A [label = 'Motor\nR = 0.95']
  B [label = 'Pump\nR = 0.90']
  C [label = 'Valve\nR = 0.98']
  O [shape = point, width = 0.15, fillcolor = black]
  I -> A -> B -> C -> O
}
")

```{r echo=TRUE}
R_motor <- 0.95
R_pump  <- 0.90
R_valve <- 0.98

R_series <- R_motor * R_pump * R_valve
R_series
```

The system reliability is approximately 83.8%, lower than any individual component.

```{r quiz1, echo=FALSE}
quiz(caption = "Quiz: Series Systems",
  question("Four components in series have reliabilities 0.99, 0.97, 0.95, and 0.93. What is the system reliability?",
    answer("0.99", message = "That is only the most reliable component."),
    answer("0.846", correct = TRUE,
           message = "Correct! R_sys = 0.99 × 0.97 × 0.95 × 0.93 ≈ 0.846."),
    answer("0.960", message = "That is the average, not the product."),
    answer("1.00", message = "System reliability is always ≤ the weakest component."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("In a series system, which component has the greatest influence on system reliability?",
    answer("The most reliable component",
           message = "The weakest component limits the system, not the strongest."),
    answer("The least reliable component", correct = TRUE,
           message = "Correct! The least reliable component is the bottleneck — improving it gives the largest gain in system reliability."),
    answer("The component with the lowest failure rate",
           message = "The component with the lowest failure rate is the most reliable one — but in a series system it is the least reliable component that constrains the system."),
    answer("All components equally",
           message = "In a series system, the least reliable component has a disproportionate effect on system reliability."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  )
)
```

Calculate series reliability yourself.

```{r ex-series, exercise=TRUE}
# A conveyor belt system has 5 components in series.
# Component reliabilities: 0.98, 0.96, 0.99, 0.94, 0.97
# Calculate the system reliability.

```

```{r ex-series-hint}
# R_sys <- prod(c(0.98, 0.96, 0.99, 0.94, 0.97))
```

```{r ex-series-solution}
R_components <- c(0.98, 0.96, 0.99, 0.94, 0.97)
R_series <- prod(R_components)
R_series  # ~0.845
```

## Parallel Systems

In a **parallel** system, only *one* component needs to function for the system to succeed. The system fails only if *all* components fail simultaneously. This is called **active redundancy**.

$$R_{sys} = 1 - \prod_{i=1}^{n}(1 - R_i)$$

**Key insight**: Adding parallel components always increases system reliability. Redundancy is a powerful tool for critical systems.

### Example

A backup power system has a primary generator (R = 0.90) and a standby generator (R = 0.85). Either one alone keeps the system running.

```{r rbd-parallel, echo=FALSE}
grViz("
digraph parallel {
  rankdir = LR
  graph [bgcolor = transparent]
  node [shape = rectangle, style = filled, fillcolor = '#A9DFBF',
        fontname = 'sans-serif', fontsize = 12, margin = '0.2,0.1']
  edge [arrowsize = 0.8]
  I [shape = point, width = 0.15, fillcolor = black]
  A [label = 'Generator 1\nR = 0.90']
  B [label = 'Generator 2\nR = 0.85']
  O [shape = point, width = 0.15, fillcolor = black]
  I -> A -> O
  I -> B -> O
}
")

```{r echo=TRUE}
R_primary <- 0.90
R_standby <- 0.85

R_parallel <- 1 - (1 - R_primary) * (1 - R_standby)
R_parallel
```

The parallel system reliability is 98.5%, much higher than either generator alone.

Use the slider to explore how the number of redundant components affects system reliability.

```{r echo=FALSE}
sliderInput("n_comp", "Number of parallel components:", min = 1, max = 8, value = 2, step = 1)
numericInput("R_ind", "Individual component reliability:", value = 0.90, min = 0.50, max = 0.999, step = 0.01)
plotOutput("parallelPlot")
```

```{r context="server"}
output$parallelPlot <- renderPlot({
  n_vals <- 1:10
  Rc     <- input$R_ind
  R_par  <- 1 - (1 - Rc)^n_vals
  R_sel  <- 1 - (1 - Rc)^input$n_comp
  plot(n_vals, R_par, type = "b", col = "steelblue", lwd = 2, pch = 19,
       xlab = "Number of redundant components",
       ylab = "System Reliability",
       main = "Parallel System Reliability vs. Redundancy",
       ylim = c(0, 1))
  points(input$n_comp, R_sel, col = "red", pch = 19, cex = 2.5)
  abline(h = Rc, col = "gray50", lty = 2)
  legend("bottomright",
         legend = c("System reliability", paste0("n = ", input$n_comp),
                    paste0("Single component (", Rc, ")")),
         col = c("steelblue", "red", "gray50"),
         lty = c(1, NA, 2), pch = c(19, 19, NA))
})
```

```{r quiz2, echo=FALSE}
quiz(caption = "Quiz: Parallel Systems",
  question("Two components each have reliability 0.80. What is the reliability of a parallel system?",
    answer("0.80", message = "That is a single component's reliability."),
    answer("0.64", message = "0.64 = 0.8 × 0.8 is the series reliability."),
    answer("0.96", correct = TRUE,
           message = "Correct! R_sys = 1 − (1−0.80)² = 1 − 0.04 = 0.96."),
    answer("1.60", message = "Reliability cannot exceed 1."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("Adding a third identical component (R = 0.80) in parallel to the previous two-component system gives a system reliability of approximately:",
    answer("0.992", correct = TRUE,
           message = "Correct! R_sys = 1 − (1−0.80)³ = 1 − 0.008 = 0.992."),
    answer("0.96", message = "That is the two-component parallel reliability."),
    answer("0.80", message = "That is a single component's reliability."),
    answer("0.512", message = "0.512 = 0.8³ is three components in series, not parallel."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  )
)
```

Calculate parallel reliability yourself.

```{r ex-parallel, exercise=TRUE}
# A critical pump station has 3 pumps in parallel.
# Each pump has reliability 0.88.
# What is the system reliability?

```

```{r ex-parallel-hint}
# R_sys <- 1 - prod(1 - c(0.88, 0.88, 0.88))
```

```{r ex-parallel-solution}
R_components <- c(0.88, 0.88, 0.88)
R_parallel <- 1 - prod(1 - R_components)
R_parallel  # ~0.9983
```

## Mixed Systems

Most real systems combine series and parallel blocks. To analyze a **mixed** system, decompose it into subsystems and apply series and parallel rules step by step, working from the innermost blocks outward.

Use the selector below to compare how the topology changes across the three core configurations.

```{r echo=FALSE}
selectInput("topology", "Select topology:",
            choices = c("Series", "Parallel", "Mixed"), selected = "Mixed")
DiagrammeROutput("topologyDiagram")
```

```{r context="server"}
output$topologyDiagram <- renderGrViz({
  if (input$topology == "Series") {
    grViz("digraph { rankdir=LR; graph [bgcolor=transparent]
      node [shape=rectangle, style=filled, fillcolor='#AED6F1', fontname='sans-serif', fontsize=12]
      I [shape=point, width=0.15, fillcolor=black]
      A [label='Component A\nR=0.95']; B [label='Component B\nR=0.90']; C [label='Component C\nR=0.97']
      O [shape=point, width=0.15, fillcolor=black]
      I->A->B->C->O }")
  } else if (input$topology == "Parallel") {
    grViz("digraph { rankdir=LR; graph [bgcolor=transparent]
      node [shape=rectangle, style=filled, fillcolor='#A9DFBF', fontname='sans-serif', fontsize=12]
      I [shape=point, width=0.15, fillcolor=black]
      A [label='Component A\nR=0.95']; B [label='Component B\nR=0.90']
      O [shape=point, width=0.15, fillcolor=black]
      I->A->O; I->B->O }")
  } else {
    grViz("digraph { rankdir=LR; graph [bgcolor=transparent]
      node [shape=rectangle, style=filled, fillcolor='#F9E79F', fontname='sans-serif', fontsize=12]
      I [shape=point, width=0.15, fillcolor=black]
      S [label='Sensor\nR=0.95']; T [label='Transmitter\nR=0.97']
      A1 [label='Actuator 1\nR=0.90']; A2 [label='Actuator 2\nR=0.90']
      O [shape=point, width=0.15, fillcolor=black]
      I->S->T->A1->O; T->A2->O }")
  }
})
```

### Example

A safety system has:

* **Subsystem A**: a sensor (R = 0.95) and a transmitter (R = 0.97) in **series**.
* **Subsystem B**: two redundant actuators (each R = 0.90) in **parallel**.
* Subsystems A and B are in **series** (both must work).

```{r echo=TRUE}
# Subsystem A (series)
R_A <- 0.95 * 0.97
R_A

# Subsystem B (parallel)
R_B <- 1 - (1 - 0.90) * (1 - 0.90)
R_B

# Overall system (A and B in series)
R_system <- R_A * R_B
R_system
```

```{r quiz3, echo=FALSE}
quiz(caption = "Quiz: Mixed Systems",
  question("A system has two subsystems in series. Subsystem 1 has two components in parallel (each R = 0.85); Subsystem 2 is a single component with R = 0.92. What is the system reliability?",
    answer("0.85 × 0.92 = 0.782", message = "That ignores the parallel redundancy in Subsystem 1."),
    answer("[1 − (1−0.85)²] × 0.92 ≈ 0.899", correct = TRUE,
           message = "Correct! R_sub1 = 1 − 0.15² = 0.9775, then R_sys = 0.9775 × 0.92 ≈ 0.899."),
    answer("0.85 + 0.92 = 1.77", message = "Reliabilities cannot be added like this."),
    answer("0.92", message = "That ignores Subsystem 1 entirely."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("To analyze a complex mixed RBD, the recommended approach is to:",
    answer("Average all component reliabilities",
           message = "Averaging does not account for the series/parallel topology."),
    answer("Decompose into series and parallel subsystems, then apply the formulas step by step", correct = TRUE,
           message = "Correct! Work from the innermost blocks outward, replacing each parallel or series group with its equivalent single-block reliability."),
    answer("Use only the reliability of the most critical component",
           message = "This would ignore the contributions of all other components."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  )
)
```

## k-out-of-n Systems

A **k-out-of-n** system succeeds if at least *k* of its *n* identical components function. This generalizes series (k = n) and parallel (k = 1):

* **k = n**: all must work → series.
* **k = 1**: at least one works → parallel.
* **1 < k < n**: voting or load-sharing systems.

The reliability of a k-out-of-n system with identical components (each with reliability *p*) follows the binomial distribution:

$$R_{k/n} = \sum_{i=k}^{n} \binom{n}{i} p^i (1-p)^{n-i} = 1 - \text{pbinom}(k-1, n, 1-p)$$

### Example

A flight control system uses 3 redundant computers. At least 2 of the 3 must agree for the system to function safely (a 2-out-of-3 voter). Each computer has reliability R = 0.99.

```{r rbd-koon, echo=FALSE}
grViz("
digraph koon {
  rankdir = LR
  graph [bgcolor = transparent]
  node [shape = rectangle, style = filled, fillcolor = '#D7BDE2',
        fontname = 'sans-serif', fontsize = 12, margin = '0.2,0.1']
  edge [arrowsize = 0.8]
  I  [shape = point, width = 0.15, fillcolor = black]
  C1 [label = 'Computer 1\nR = 0.99']
  C2 [label = 'Computer 2\nR = 0.99']
  C3 [label = 'Computer 3\nR = 0.99']
  V  [label = 'Voter\n(2-of-3)', shape = diamond, fillcolor = '#F1948A', fontsize = 11]
  O  [shape = point, width = 0.15, fillcolor = black]
  I -> C1 -> V
  I -> C2 -> V
  I -> C3 -> V
  V -> O
}
")
```

```{r echo=TRUE}
n <- 3      # total components
k <- 2      # minimum required
p <- 0.99   # individual reliability

R_voting <- 1 - pbinom(k - 1, n, 1 - p)
R_voting
```

The 2-out-of-3 system has reliability 0.9997.

```{r quiz4, echo=FALSE}
quiz(caption = "Quiz: k-out-of-n Systems",
  question("A 2-out-of-4 system has 4 identical components each with R = 0.90. What R function computes its reliability?",
    answer("1 - pbinom(1, 4, 0.10)", correct = TRUE,
           message = "Correct! k=2, n=4, failure probability = 1−0.90 = 0.10. Use pbinom(k−1, n, 1−p) = pbinom(1, 4, 0.10)."),
    answer("pbinom(2, 4, 0.90)",
           message = "This gives P(X ≤ 2 successes), not the probability that at least 2 succeed."),
    answer("1 - pbinom(2, 4, 0.90)",
           message = "The third argument should be the failure probability (1−p), not p."),
    answer("prod(rep(0.90, 4))",
           message = "That formula gives the series reliability (all 4 must work), not 2-out-of-4."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("How does a 1-out-of-n system relate to a parallel system?",
    answer("They are different — a 1-out-of-n system requires only the first component to work",
           message = "A 1-out-of-n system requires at least one of any n components to work, which is exactly a parallel configuration."),
    answer("They are equivalent — both require only one component to function", correct = TRUE,
           message = "Correct! When k = 1, the k-out-of-n formula reduces to the parallel reliability formula."),
    answer("A 1-out-of-n system is less reliable than a parallel system",
           message = "They are the same configuration."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  )
)
```

Calculate k-out-of-n reliability in R.

```{r ex-koon, exercise=TRUE}
# A 3-out-of-5 redundant sensor array.
# Each sensor has reliability p = 0.95.
# Calculate the system reliability.
n <- 5
k <- 3
p <- 0.95

```

```{r ex-koon-hint}
# R_sys <- 1 - pbinom(k - 1, n, 1 - p)
```

```{r ex-koon-solution}
n <- 5
k <- 3
p <- 0.95
R_sys <- 1 - pbinom(k - 1, n, 1 - p)
R_sys  # ~0.9988
```

## System MTTF

For a **series** system of components with constant failure rates (exponential distribution), the system failure rate is the sum of component failure rates:

$$\lambda_{sys} = \sum_{i=1}^{n} \lambda_i \qquad MTTF_{sys} = \frac{1}{\lambda_{sys}}$$

For $n$ identical parallel components each with failure rate $\lambda$:

$$MTTF_{parallel} = \frac{1}{\lambda}\left(1 + \frac{1}{2} + \frac{1}{3} + \cdots + \frac{1}{n}\right)$$

### Example

Two pumps in parallel, each with $\lambda = 0.01$ failures/hour:

```{r echo=TRUE}
lambda <- 0.01

MTTF_single   <- 1 / lambda
MTTF_parallel <- (1 / lambda) * (1 + 1/2)

MTTF_single    # 100 hours
MTTF_parallel  # 150 hours — 50% longer with one spare
```

```{r quiz5, echo=FALSE}
quiz(caption = "Quiz: System MTTF",
  question("Three components in series have failure rates 0.02, 0.03, and 0.05 failures/hour. What is the system MTTF?",
    answer("10 hours", correct = TRUE,
           message = "Correct! λ_sys = 0.02 + 0.03 + 0.05 = 0.10, so MTTF = 1/0.10 = 10 hours."),
    answer("16.7 hours", message = "That is 1/0.06 — you may have missed one failure rate."),
    answer("100 hours", message = "100 hours is the MTTF of a single component with λ = 0.01."),
    answer("0.1 hours", message = "0.1 is the system failure rate, not the MTTF."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("Compared to a single component, the MTTF of two identical components in parallel is:",
    answer("Twice as large",
           message = "Not quite. For two identical parallel components, MTTF_parallel = (3/2) × MTTF_single — a 50% increase."),
    answer("1.5 times as large", correct = TRUE,
           message = "Correct! MTTF_parallel = (1/λ)(1 + 1/2) = 1.5 × MTTF_single."),
    answer("The same",
           message = "Redundancy always increases MTTF."),
    answer("Three times as large",
           message = "Three times applies to three identical parallel components using the harmonic series formula."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  )
)
```

## Introduction to Fault Tree Analysis

**Fault Tree Analysis (FTA)** is a top-down approach to reliability analysis that starts with an undesirable system event (the **top event**) and works backward to identify the combinations of component failures that could cause it.

While an RBD asks "What must work for the system to succeed?", a fault tree asks "What can cause the system to fail?"

### Gates

A fault tree uses logic gates to combine failure events:

* **AND gate** ($\land$): the top event occurs only if *all* input events occur simultaneously. This corresponds to **parallel** components in an RBD, all must fail to cause system failure.
* **OR gate** ($\lor$): the top event occurs if *any* input event occurs. This corresponds to **series** components in an RBD, any single failure causes system failure.

### RBD — Fault Tree Duality

Every RBD has a corresponding fault tree and vice versa:

| RBD configuration | Fault tree gate for system failure |
|:---:|:---:|
| Series (all must work) | OR gate (any failure causes system failure) |
| Parallel (any can work) | AND gate (all must fail for system failure) |

This duality means the two tools provide complementary views of the same system. RBDs are better for computing reliability; fault trees are better for tracing failure causes and identifying critical combinations.

The fault tree below illustrates a simple two-component system: the top event (system failure) occurs if *either* component fails — an OR gate, which corresponds to a series RBD.

```{r rbd-fta, echo=FALSE}
grViz("
digraph fta {
  rankdir = TB
  graph [bgcolor = transparent]
  node [fontname = 'sans-serif', fontsize = 12, style = filled]
  edge [arrowsize = 0.8]
  Top [label = 'System\nFailure',  shape = rectangle, fillcolor = '#E74C3C', fontcolor = white]
  OR  [label = 'OR Gate',          shape = diamond,   fillcolor = '#F39C12']
  F1  [label = 'Component A\nFails', shape = ellipse, fillcolor = '#AED6F1']
  F2  [label = 'Component B\nFails', shape = ellipse, fillcolor = '#AED6F1']
  Top -> OR
  OR  -> F1
  OR  -> F2
}
")
```

For complex fault trees, the R package `FaultTree` on CRAN provides tools for building and analyzing fault tree models programmatically.

```{r quiz6, echo=FALSE}
quiz(caption = "Quiz: Fault Tree Analysis",
  question("A fault tree has an AND gate combining two component failures. This corresponds to which RBD configuration?",
    answer("Series — both components in series",
           message = "A series RBD means any single failure causes system failure, which maps to an OR gate in a fault tree."),
    answer("Parallel — both components in parallel", correct = TRUE,
           message = "Correct! An AND gate means the system fails only if both components fail — exactly the parallel (redundant) RBD configuration."),
    answer("k-out-of-n with k = 2",
           message = "A 2-out-of-n system is more complex than a simple AND/OR gate."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("Which statement best distinguishes FTA from RBD?",
    answer("FTA models reliability; RBD models failure modes",
           message = "It is the other way around — FTA is failure-oriented and RBD is success-oriented."),
    answer("RBD models what must succeed for the system to work; FTA traces what can cause the system to fail", correct = TRUE,
           message = "Correct! RBDs are success-space models; fault trees are failure-space models. They are logically dual and provide complementary views."),
    answer("RBD applies only to series systems; FTA applies to parallel systems",
           message = "Both methods apply to any system topology."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  )
)
```

## Case Study: Industrial Cooling System

Let's apply everything to a realistic example. An industrial cooling system has the following architecture:

1. **Water supply subsystem**: two pumps in parallel (each R = 0.92), followed by a filter in series (R = 0.99).
2. **Control subsystem**: a primary controller (R = 0.97) and a backup controller in parallel (R = 0.95).
3. The water supply and control subsystems must both work (series at system level).

```{r echo=TRUE}
# Step 1 — Water supply subsystem
R_pump_parallel <- 1 - (1 - 0.92)^2
R_water <- R_pump_parallel * 0.99
R_water

# Step 2 — Control subsystem
R_control <- 1 - (1 - 0.97) * (1 - 0.95)
R_control

# Step 3 — Overall system (both subsystems in series)
R_system <- R_water * R_control
R_system
```

Now explore what happens if the filter is upgraded.

```{r ex-casestudy, exercise=TRUE}
# Modify the case study: the filter is upgraded to R = 0.999.
# Recalculate the system reliability with the improved filter.
R_pump1  <- 0.92
R_pump2  <- 0.92
R_filter <- 0.999   # upgraded from 0.99
R_ctrl1  <- 0.97
R_ctrl2  <- 0.95

```

```{r ex-casestudy-hint}
# R_pump_parallel <- 1 - (1 - R_pump1) * (1 - R_pump2)
# R_water  <- R_pump_parallel * R_filter
# R_control <- 1 - (1 - R_ctrl1) * (1 - R_ctrl2)
# R_system <- R_water * R_control
```

```{r ex-casestudy-solution}
R_pump1  <- 0.92
R_pump2  <- 0.92
R_filter <- 0.999
R_ctrl1  <- 0.97
R_ctrl2  <- 0.95

R_pump_parallel <- 1 - (1 - R_pump1) * (1 - R_pump2)
R_water   <- R_pump_parallel * R_filter
R_control <- 1 - (1 - R_ctrl1) * (1 - R_ctrl2)
R_system  <- R_water * R_control
R_system  # ~0.985
```

## Summary

Congratulations on completing the Reliability Block Diagrams and System Reliability tutorial!

**Key takeaways:**

* **Series**: $R_{sys} = \prod R_i$ — every component must work; reliability decreases with more components.
* **Parallel**: $R_{sys} = 1 - \prod(1-R_i)$ — redundancy; only one component needs to work.
* **Mixed**: decompose into subsystems, apply formulas step by step.
* **k-out-of-n**: `1 - pbinom(k-1, n, 1-p)` for voting / load-sharing systems.
* **Series MTTF**: $1/\sum\lambda_i$; **Parallel MTTF** of $n$ identical units: $(1/\lambda)\sum_{i=1}^{n}(1/i)$.
* **FTA duality**: series RBD ↔ OR gate; parallel RBD ↔ AND gate.

### References

- Abernethy, R.B. (2004) The New Weibull Handbook. Fifth Edition.

- Aden-Buie G, Schloerke B, Allaire J, Rossell Hayes A (2023). learnr: Interactive Tutorials for R. https://rstudio.github.io/learnr/, https://github.com/rstudio/learnr.

- Billinton R, Allan R (1992). *Reliability Evaluation of Engineering Systems*. Second Edition. Plenum Press, New York.

- Vesely W, Goldberg F, Roberts N, Haasl D (1981). *Fault Tree Handbook*. US Nuclear Regulatory Commission, NUREG-0492.

---
title: "An Introduction to Life Data Analysis"
output: 
  learnr::tutorial:
    progressive: true
    allow_skip: true
runtime: shiny_prerendered
---

```{r setup, include=FALSE}
library(learnr)
library(WeibullR)
```

## Introduction

Welcome to the tutorial on **Life Data Analysis**! In this tutorial, you will learn about the basics of life data analysis and how it applies to various fields, such as engineering, manufacturing, and many others. 

## Learning Objectives

By the end of this module, learners will be able to:

* Describe the purpose of Weibull analysis in reliability engineering.
* Differentiate between types of data censoring, including right-censored and interval-censored data.
* Differentiate between different Weibull models (2-parameter Weibull, 3-parameter Weibull, and Weibayes).
* Apply Median Rank Regression (MRR) and Maximum Likelihood Estimation (MLE) estimation methods to sample datasets.
* Interpret results using plotting methods, including probability plots and contour plots.

## What is Life Data Analysis?

Life data analysis is the study of how systems function over time. Life data helps explain how long a system will last, when it will likely fail, and how often it will need maintenance. For example, in the manufacturing industry, life data analysis can be used to understand the lifespan of machines and equipment, helping to optimize maintenance schedules and reduce downtime.

## Why is Life Data Analysis Important?

Understanding life data analysis helps in predicting the lifespan and maintenance needs of various systems, thereby improving reliability and efficiency. By collecting and modeling life data, we can make predictions about how long machines will last or how long before they will need to be repaired or replaced. 

## Life Distributions

### The Weibull Distribution

The Weibull distribution is a continuous probability distribution used in life data analysis and many other fields. Life data analysis is often referred to as Weibull analysis because the Weibull distribution is commonly used to model life data. While other distributions can be used to model life data, this tutorial focuses on the Weibull distribution because of its wide application.

### Why the Weibull Distribution?

The Weibull is a **flexible** distribution that can fit many different types of data. By adjusting the distribution parameters, the Weibull can mimic other distributions such as the normal, exponential, and log-normal distributions.

### Mimicking Other Distributions

To illustrate, take the following normal distribution. The normal distribution is another common distribution used in statistics. The normal is symmetric and characterized by its mean and standard deviation.

```{r echo=FALSE}
x <- seq(0, 10, by = .1) # Create a sequence of x values
y <- dnorm(x, mean = 5, sd = 1.5) # Calculate the normal density function
plot(x,y, type = "l", col = "blue", xlab = "Variable", ylab = "Probability") # Plot the normal distribution
```

The Weibull can *mimic* the normal distribution by adjusting the shape and scale parameters. Adjust the scale parameter in the slider until the Weibull below looks similar to the Normal above.

```{r, echo=FALSE}
sliderInput("eta", "Scale:", min = 1, max = 10, value = 1, step = 0.5) # Slider for eta
plotOutput("distPlot") # Plot output
```

```{r, context="server"}
# Plot the Weibull distribution
output$distPlot <- renderPlot({
  x <- seq(0, 10, by = .1) # Create a sequence of x values
  y <- dweibull(x, shape = 3.67, scale = input$eta) # Calculate the Weibull density function
  plot(x,y, type = "l", col = "red", xlab = "Variable", ylab = "Probability") # Plot the Weibull distribution
})
```

<div id="filter-hint">
**Hint:** Set Eta equal to 5 as a starting point.
</div>

By adjusting the scale parameter, the Weibull can closely resemble the normal distribution.

### The Cumulative Distribution Function

In life data analysis, the cumulative distribution function (CDF) describes the probability of survival (or failure) prior to a certain time *t*. In other words, the CDF gives the probability that a unit will survive (or fail) up to time *t*.



For the Weibull distribution, the CDF is given by:

$$ R(t) = 1 - F(t) = e^{-(t/\eta)^\beta} $$

Where *R(t)* is the cumulative probability of *survival* prior to  time *t*,  *F(t)* is the cumulative probability of *failure* prior to time *t*, $\beta$ is the shape parameter and $\eta$ is the scale parameter.

Notice how *R(t)* and *F(t)* are complementary probabilities, meaning that they add up to 1. For example, if the probability of failure prior to time *t* is 30%, then the probability of survival prior to time *t* is 70%.

For the Weibull distribution, $\beta$ has a physical interpretation related to the failure rate, or the rate at which failures occur over time.

-   $\beta$ \< 1 - the failure rate is decreasing
-   $\beta$ = 1 - the failure rate is constant
-   $\beta$ \> 1 - the failure rate is increasing

By estimating $\beta$ from life data, we can gain insights into the failure behavior of the system being analyzed.

```{r quiz1, echo=FALSE}
quiz(caption = "Quiz: The Weibull Parameters",
  question("Which beta indicates an increasing failure rate?",
    answer("1", message = "Beta of 1 indicates a constant failure rate."),
    answer("0.5", message = "Beta less than 1 indicates a decreasing failure rate."),
    answer("2", correct = TRUE, message = "Correct! Beta greater than 1 indicates an increasing failure rate."),
    answer("0", message = "Beta cannot be 0."),
    random_answer_order = TRUE, 
    allow_retry = TRUE
  ),
  question("What does the parameter eta represent?",
    answer("shape parameter", message = "Eta is the scale parameter, not the shape parameter."),
    answer("scale parameter", correct = TRUE, message = "Correct! Eta is the scale parameter."),
    answer("failure rate", message = "Eta is not the failure rate."),
    answer("mean time between failures", message = "Eta is not the mean time between failures."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("What is the cumulative probability when t = eta?",
    answer("0%", message = "At t = eta, the cumulative probability is not 0%."),
    answer("50%", message = "At t = eta, the cumulative probability is not 50%."),
    answer("63.2%", correct = TRUE, message = "Correct! At t = eta, the cumulative probability is 63.2%."),
    answer("100%", message = "At t = eta, the cumulative probability is not 100%."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("What is the cumulative probability when t = 0?",
    answer("0%", correct = TRUE, message = "Correct! At t = 0, the cumulative probability is 0%."),
    answer("50%", message = "At t = 0, the cumulative probability is not 50%."),
    answer("63.2%", message = "At t = 0, the cumulative probability is not 63.2%."),
    answer("100%", message = "At t = 0, the cumulative probability is not 100%."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  )
)
```

## The Weibull Parameters

### Beta
 
Beta is the **shape** parameter of the Weibull distribution. Changing Beta changes 
the **shape** of the cumulative distribution function. To illustrate, use the slider 
below to adjust the shape of the Weibull. 

```{r, echo=FALSE}
sliderInput("beta", "Beta:", min = 0.5, max = 5, value = 1, step = 0.5) # Slider for beta
plotOutput("cumPlot") # Plot output
```

```{r, context="server"}
# Plot the cumulative distribution function
output$cumPlot <- renderPlot({
  x <- seq(0, 50, by = .1) # Create a sequence of x values
  plot(pweibull(x, shape = input$beta, scale = 10), col = "blue", type = "l", 
       xlab = "Variable", ylab = "Cumulative Probability") # Plot the CDF
})
```

As you adjust Beta, notice how the shape of the Weibull changes. Increasing Beta 
makes the curve steeper, indicating an increasing failure rate. Decreasing Beta 
makes the curve less steep, indicating a decreasing failure rate.

### Eta

Eta is the **scale** parameter of the Weibull distribution. Changing Eta changes 
the **scale** of the cumulative distribution function. To illustrate, use the slider 
below to adjust the scale of the Weibull. 

```{r, echo=FALSE}
sliderInput("eta2", "Eta:", min = 1, max = 20, value = 1) # Slider for eta
plotOutput("cummPlot2") # Plot output
```

```{r, context="server"}
# Plot the cumulative distribution function
output$cummPlot2 <- renderPlot({
  x <- seq(0, 50, by = .1) # Create a sequence of x values
  plot(pweibull(x, shape = 1, scale = input$eta2), col = "red", type = "l", xlab = "Variable", ylab = "Cumulative Probability") # Plot the CDF
})
```

As you adjust Eta, notice how the scale of the Weibull changes. Increasing Eta 
shifts the curve to the right, indicating a longer lifespan. Decreasing Eta shifts the
curve to the left, indicating a shorter lifespan. 

## WeibullR

### Getting Started with WeibullR

For this tutorial, we will use WeibullR: an R package for Weibull analysis.

First, check if WeibullR is installed in R and install if not.

```{r eval=FALSE}
pak::pkg_install("WeibullR")
```

As an example of Weibull analysis, suppose a factory has 5 machines that fail at different times. The factory wants to understand the reliability of the machines over time and predict when future failures may occur. The factory collects the failure data and uses Weibull analysis to model the reliability of the machines. The failure times are 30, 49, 82, 90, and 96 respectively. 

To run Weibull analysis in R, first create a vector with the failure data, then use the `MLEw2p` function with the failure data to fit a Weibull to the data. The argument `bounds=TRUE` adds confidence bounds to the fit, and `show=TRUE` displays the results.

```{r echo=TRUE}
failures<-c(30, 49, 82, 90, 96)
fit<-MLEw2p(failures, bounds=TRUE, show=TRUE)
```

### The Weibull Probability Plot

The Weibull Probability Plot above shows the time (or equivalent) on the horizontal axis and the cumulative probability of failure or unreliability on the vertical axis. We know from the cumulative distribution function that the complement of the failure probability is the survival probability or reliability. For example, if the unreliability at time 50 is 20%, then the reliability at time 50 is 100%-20% = 80%.

Note that the horizontal axis on a probability plot is log scale, and the vertical axis is log(log(1-p)) scale, where p is the cumulative probability of failure. By using this scale, the resulting plot of the Weibull data appears linear.

```{r quiz2, echo=FALSE}
quiz(caption = "Quiz: Interpreting the Probability Plot",
  question("In the previous example, what is the approximate unreliability at time 100?",
    answer("20%", message = "20% is too low."),
    answer("80%", message = "80% is slightly too low."),
    answer("10%", message = "10% is the reliability, not the unreliability."),
    answer("90%", correct = TRUE, message = "Correct! The unreliability at time 100 is approximately 90%."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("In the previous example, what is the approximate reliability at time 50?)",
    answer("20%", correct = TRUE, message = "Correct! The reliability at time 50 is approximately 20%."),
    answer("80%", message = "80% is the unreliability, not the reliability."),
    answer("10%", message = "10% is slightly too low."),
    answer("90%", message = "90% is too high."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("In the previous example, what is the approximate beta of the fit?",
    answer("1.5", message = "1.5 is too low."),
    answer("3.5", message = "3.5 is slightly too high."),
    answer("0.5", message = "0.5 is too low."),
    answer("3.2", correct = TRUE, message = "Correct! The beta of the fit is approximately 3.2."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("In the previous example, what is the approximate eta of the fit?",
    answer("77.81", correct = TRUE, message = "Correct! The eta of the fit is approximately 77.81."),
    answer("88.84", message = "88.84 is slightly too high."),
    answer("37.89", message = "37.89 is too low."),
    answer("389.7", message = "389.7 is too high."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  )
)
```

### Exercise: Fit a Weibull Distribution

A batch of industrial pumps failed at the following times (hours): 120, 205, 310, 445, 590, 780. Use `MLEw2p()` to fit a 2-parameter Weibull distribution and display the probability plot.

```{r ex-lda1, exercise=TRUE}
failures <- c(120, 205, 310, 445, 590, 780)

```

```{r ex-lda1-hint}
# Use MLEw2p() with show=TRUE to plot the result
# MLEw2p(failures, show = TRUE)
```

```{r ex-lda1-solution}
failures <- c(120, 205, 310, 445, 590, 780)
fit <- MLEw2p(failures, show = TRUE)
```

## Data Censoring

In life data analysis, data is often **censored**. Censoring occurs when the exact time to failure is not known for all units. There are several types of censoring, including right censoring and interval censoring.

### Right Censored Data

Right censored data includes **suspensions** or units that have operated for a period of time without failure. Suspensions are common in life data analysis because not all units fail during the analysis period.

To illustrate right censored data, suppose that the factory in the previous example has 3 additional machines that did not fail during the analysis period. The machines were suspended (i.e. removed from service) at times 100, 45, and 10 respectively.

To add suspensions to the previous example, create a vector with the suspension data and add it to the `MLEw2p` function.

```{r echo=TRUE}
suspensions<-c(100, 45, 10)
fit<-MLEw2p(failures, suspensions, bounds=TRUE, show=TRUE)
```

Note how the fit has changed slightly after adding suspensions. The fit now has a lower beta and higher eta.

### Exercise: Right-Censored Fit

Four compressors failed at times 50, 85, 130, and 200 hours. Two additional units were removed from service (suspended) at 250 hours each without failing. Fit a Weibull model using `MLEw2p()` with suspensions.

```{r ex-lda2, exercise=TRUE}
failures <- c(50, 85, 130, 200)
suspensions <- c(250, 250)

```

```{r ex-lda2-hint}
# Pass both failures and suspensions to MLEw2p()
# MLEw2p(failures, suspensions, show = TRUE)
```

```{r ex-lda2-solution}
failures <- c(50, 85, 130, 200)
suspensions <- c(250, 250)
fit <- MLEw2p(failures, suspensions, show = TRUE)
```

### Interval Censored Data

Interval censored data is data where the exact failure times are unknown, but are known to have occurred within certain intervals. An example of interval censored data is inspection data, where parts or machines are inspected periodically for failures. If a failure is detected during an inspection, the exact failure time is unknown, but it is known to have occurred between the last inspection time and the current inspection time.

To illustrate interval censored data, let's look at an example from Silkworth, 2020. The data consists of part cracks detected during periodic inspections. The left and right columns represent the inspection intervals, and the qty column represents the number of cracks detected in each interval. 

```{r echo=TRUE}
inspection_data<-data.frame(left=c(0, 6.12, 19.92, 29.64, 35.4, 39.72, 45.32, 52.32),
                            right=c(6.12, 19.92, 29.64, 35.4, 39.72, 45.32, 52.32, 63.48),
                            qty=c(5, 16, 12, 18, 18, 2, 6, 17))
```

A number of units did not fail during the inspection period. These units are considered suspensions and are added to the data set as a single entry with the time equal to the last inspection time, event equal to 0 (suspension), and qty equal to the number of suspensions.

```{r echo=TRUE}
suspensions<-data.frame(time=63.48, event=0, qty=73)
```

To run the interval censored example in WeibullR, use the `wblr` function with the suspension data and the `interval` argument to specify the inspection data. Then use the `wblr.fit` function with the `method.fit="mle"` argument to fit the model using maximum likelihood estimation. Next, use the `wblr.conf` function with the `method.conf="fm"` argument to add confidence bounds using the Fisher matrix method. The `col` and `lty` arguments specify the color and line type of the fit and confidence bounds respectively. Use the `plot` function to create the probability plot.

```{r echo=TRUE, warning=FALSE}
obj1<-wblr(suspensions, interval = inspection_data)
obj1<-wblr.fit(obj1, method.fit="mle", col="red") 
obj1<-wblr.conf(obj1, method.conf="fm", lty=2)
plot(obj1)
```

Instead of points, the interval censored data is shown as black horizontal lines representing the intervals where failures occurred. The solid red line is the model fit and the red dashed lines are the confidence bounds.

### Grouped Data

Life data is often compiled into groups (i.e. a group of machines failed or survived at a certain time). For example, in the interval censored example above, the inspection data is grouped into intervals with multiple failures in each interval. Likewise, the suspensions are grouped into a single entry with 73 suspensions at the last inspection time.

Let's look at the inspection data again. The knitr::kable function below displays the inspection data in a table format, and the head function shows the first few rows of the data.

``` {r echo=TRUE}
knitr::kable(
  head(inspection_data)
)
```

The `qty` column indicates how many units failed in each interval. For example, 5 units failed between time 0 and 6.12, 16 units failed between time 6.12 and 19.92, and so on. 

```{r quiz3, echo=FALSE}
quiz(caption = "Quiz: Interval Censored Data",
  question("In the interval censored example above, how many units are in the group of suspensions that did not fail?",
    answer("73", correct = TRUE, message = "Correct! There are 73 units in the group of suspensions that did not fail."),
    answer("17", message = "17 is the number of failures in the last interval."),
    answer("94", message = "94 is the total number of units (failures + suspensions)."),
    answer("63.48", message = "63.48 is the time of the last inspection."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("In the interval censored example, how many total failures occurred?",
    answer("73", message = "73 is the number of suspensions that did not fail."),
    answer("17", message = "17 is the number of failures in the last interval."),
    answer("94", correct = TRUE, message = "Correct! There are a total of 94 failures."),
    answer("63.48", message = "63.48 is the time of the last inspection."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("In the interval censored example, how many total units are in the data set?",
    answer("73", message = "73 is the number of suspensions that did not fail."),
    answer("17", message = "17 is the number of failures in the last interval."),
    answer("94", message = "94 is the total number of failures."),
    answer("167", correct = TRUE, message = "Correct! There are a total of 167 units (failures + suspensions)."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("In the interval censored example, how many failures occurred between time 19.92 and 35.4?",
    answer("12", message = "12 is the number of failures between time 6.12 and 19.92."),
    answer("18", correct = TRUE, message = "Correct! There are 18 failures between time 19.92 and 35.4."),
    answer("16", message = "16 is the number of failures between time 0 and 6.12."),
    answer("2", message = "2 is the number of failures between time 39.72 and 45.32."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  )
)
```

## Parameter Estimation Methods

Parameter estimation methods are used to estimate the parameters of a probability distribution (e.g. Beta and Eta for the Weibull distribution) based on the observed data. The two most common methods are Maximum Likelihood Estimation (MLE) and Median Rank Regression (MRR).

### Maximum Likelihood vs Rank Regression

-   **Median Rank Regression (MRR)** is based on estimating the parameters of the distribution by minimizing the sum of squared errors between the observed and predicted values. The MRR method is also known as the least squares method. 

-   **Maximum Likelihood Estimation (MLE)** is based on estimating the parameters of the distribution by maximizing the likelihood function. The MLE method finds the parameters that make the observed data most probable.

This tutorial does not go into the mathematical details of MRR and MLE, but both methods are widely used in life data analysis. WeibullR includes functions for both methods, such as the `MRRw2p`, `MRRw3p` functions for MRR and the `MLEw2p`, `MLEw3p` functions for MLE.

Let's rerun the right censored example and compare the MRR and MLE methods using the `wblr` function. In the first step, fit the MRR and model using the `wblr.fit` function with the `method.fit` argument set to "rr" for MRR. In the second step, fit the MLE model using the `method.fit` argument set to "mle" for MLE. Finally, use the `plot.wblr` function to create a multi-plot comparing both models.

```{r echo=TRUE}
MRRfit <- wblr.fit(wblr(failures, suspensions, col="blue"), method.fit="rr")
MLEfit <- wblr.fit(wblr(failures, suspensions, col="red"), method.fit="mle")
plot.wblr(list(MRRfit, MLEfit))
```

The blue model is the MRR fit and the red model is the MLE fit. While both models fit the data well,
there are some slight differences in the parameters and unreliability.

```{r quiz4, echo=FALSE}
quiz(caption = "Quiz: MRR vs MLE",
  question("What is the difference between the MRR and MLE fits (select all that apply)?",
    answer("The MLE fit has a higher beta", correct = TRUE),
    answer("The MRR fit has a higher beta"),
    answer("The MLE fit results in a higher unreliability at time 30"),
    answer("The MRR fit results in a higher unreliability at time 30", correct = TRUE),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("In the previous example, which fit has a slightly higher unreliability at time 100?",
    answer("MRR", message = "The MRR fit has a slightly lower unreliability at time 100."),
    answer("MLE", correct = TRUE, message = "The MLE fit has a slightly higher unreliability at time 100."),
    answer("Both are the same", message = "The fits are slightly different at time 100."),
    answer("Cannot be determined from the plot", message = "The unreliability can be determined from the plot."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  )
)
```

## Goodness of Fit

After fitting a Weibull model, it is important to assess how well the distribution fits the data. The **Anderson-Darling (AD) statistic** is a widely used goodness-of-fit metric in life data analysis.

The AD statistic measures the distance between the fitted cumulative distribution function (CDF) and the empirical distribution of the data. A **lower AD value** indicates a closer fit between the model and the data. WeibullR automatically computes and displays the AD statistic in the legend of every probability plot.

As an informal rule of thumb:

| AD Value | Interpretation |
|----------|---------------|
| < 0.3    | Good fit       |
| 0.3 – 0.6 | Acceptable fit |
| > 0.6    | Poor fit       |

The AD statistic is also useful for **comparing fitting methods**. For example, fitting the same data with both MRR and MLE allows you to select the method that produces a lower AD — that is, the better-fitting model for your data.

```{r quiz-gof, echo=FALSE}
quiz(caption = "Quiz: Goodness of Fit",
  question("What does a lower Anderson-Darling statistic indicate?",
    answer("The fitted distribution more closely matches the data", correct = TRUE,
           message = "Correct! AD measures the distance between the fitted CDF and the empirical data — lower is better."),
    answer("The fitted distribution has a higher beta", message = "Beta is a shape parameter and is unrelated to the AD statistic."),
    answer("The data has fewer censored observations", message = "Censoring affects the dataset, not the goodness-of-fit metric directly."),
    answer("The sample size is larger", message = "Sample size affects statistical power, not the AD value itself."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("You fit the same failure dataset with both MRR and MLE. The MRR fit has AD = 0.42 and the MLE fit has AD = 0.18. Which fit is preferred?",
    answer("MLE, because it has a lower AD statistic", correct = TRUE,
           message = "Correct! The lower AD for MLE means it more closely follows the empirical distribution of the data."),
    answer("MRR, because it has a higher AD statistic", message = "A higher AD means a worse fit, not a better one."),
    answer("They are equivalent — both are below 0.6", message = "Although both may be acceptable, the MLE fit with AD = 0.18 is meaningfully closer to the data."),
    answer("Cannot be determined without knowing beta and eta", message = "The AD statistic directly summarizes fit quality; no additional parameters are needed to compare."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  )
)
```

### Exercise: Compare MRR and MLE Goodness of Fit

Fit the following failure data using both MRR and MLE methods and compare their Anderson-Darling statistics on a combined probability plot.

```{r ex-lda-gof, exercise=TRUE}
failures <- c(15, 28, 44, 61, 83, 110, 145)

```

```{r ex-lda-gof-hint}
# Create two wblr objects with different fitting methods, then plot them together
# obj_mrr <- wblr.conf(wblr.fit(wblr(failures, col = "blue"), method.fit = "rr"), method.conf = "fm", lty = 2)
# obj_mle <- wblr.conf(wblr.fit(wblr(failures, col = "red"),  method.fit = "mle"), method.conf = "fm", lty = 2)
# plot.wblr(list(obj_mrr, obj_mle))
```

```{r ex-lda-gof-solution}
failures <- c(15, 28, 44, 61, 83, 110, 145)
obj_mrr <- wblr.conf(wblr.fit(wblr(failures, col = "blue"), method.fit = "rr"),  method.conf = "fm", lty = 2)
obj_mle <- wblr.conf(wblr.fit(wblr(failures, col = "red"),  method.fit = "mle"), method.conf = "fm", lty = 2)
plot.wblr(list(obj_mrr, obj_mle))
```

## Other Weibull Models

### The WeiBayes Model

A WeiBayes or one parameter (1P) Weibull has a fixed $\beta$ or shape parameter based on *prior* knowledge or experience. Weibayes is a Bayesian approach to Weibull analysis, where prior knowledge is combined with observed data to estimate the parameters of the Weibull distribution. Weibayes is useful when there is limited failure data or when the data does not fit a standard Weibull model well.

As an example, let's rerun the right censored example of a machine that failed at times 30, 49, 82, 90, and 96. In this case, the data is limited, so a WeiBayes model may be appropriate. First, recreate the failure and suspension data. Then fit the WeiBayes model while assuming a Beta of 2 and plot the results. In the code below, the `method.fit` argument specifies the WeiBayes model, and the `weibayes.beta` argument specifies the fixed Beta value.

```{r echo=TRUE, warning=FALSE}
failures<-c(30, 49, 82, 90, 96)
suspensions<-c(100, 45, 10)
obj <- wblr.fit(wblr(failures, suspensions, col="blue"), method.fit="weibayes", weibayes.beta=2)
plot(obj)
```

Note that confidence bounds are not shown for the WeiBayes model. Confidence bounds for the WeiBayes model are not well defined because the shape parameter is fixed.

### The 3-Parameter Weibull Model

The 3P Weibull has an additional parameter *t~0~* to represent a failure free period. The failure free period is the time before which no failures can occur. Often parts or machines must age, wear, fatigue, etc. before they can actually fail, and the 3P Weibull can model this behavior. The cumulative distribution function for the 3P Weibull is given below:

$$ R(t) = 1 - F(t) = e^-((t-t~0~)/\eta)^\beta $$
Where *t~0~* is the failure free period, $\beta$ is the shape parameter, and $\eta$ is the scale parameter.

Here is an example from "The New Weibull Handbook" by Robert Abernethy (2004). This example consists of 25 failure times from a machine that has a failure free period. Let's run this example as a 3P Weibull. First, create a vector of failure data. Then fit the 3P Weibull model and plot the results. The `dist="weibull3p"` argument specifies the 3P Weibull model.

```{r echo=TRUE}
failures<-c(3.46623, 3.732711, 4.052996, 4.628703, 4.8157, 5.84517, 5.888313, 5.892967,
            8.168362, 10.02799, 10.06062, 10.49785, 11.11493, 11.87369, 12.21122, 12.51854, 
            12.91357, 18.04246, 18.20712, 19.57305, 21.20873, 30.03917, 34.88001, 36.87355,
            53.91168)
fit<- wblr.conf(wblr.fit(wblr(failures), dist="weibull3p"), col="darkgreen")
plot(fit)
```

Note that the 3P Weibull model has a failure free period of approximately 3.3 time units.
The model appears curved at lower probabilities, which is typical of a 3P Weibull.
The curvature is due to the failure free period, which shifts the distribution to the right.

```{r quiz5, echo=FALSE}
quiz(caption = "Quiz: Other Weibull Models",
  question("In the Weibayes example, what is eta?",
    answer("88.84", correct = TRUE, message = "Correct! Eta is approximately 88.84."),
    answer("389.7", message = "389.7 is too high."),
    answer("87.89", message = "87.89 is slightly too low."),
    answer("37.89", message = "37.89 is too low."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("What is the failure free period in the 3P Weibull example?",
    answer("3.3", correct = TRUE, message = "Correct! The failure free period is approximately 3.3 time units."),
    answer("0.85", message = "0.85 is the beta parameter, not the failure free period."),
    answer("11.16", message = "11.16 is the eta parameter, not the failure free period."),
    answer("0", message = "The failure free period is greater than 0."),
    random_answer_order = TRUE
  ),
  question("In the 3P Weibull example, what is beta?",
    answer("0.85", correct = TRUE, message = "Correct! Beta is approximately 0.85."),
    answer("3.3", message = "3.3 is the failure free period, not beta."),
    answer("11.16", message = "11.16 is the eta parameter, not beta."),
    answer("1", message = "Beta is not equal to 1."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("In the 3P Weibull example, what is eta?",
    answer("11.16", correct = TRUE, message = "Correct! Eta is approximately 11.16."),
    answer("0.85", message = "0.85 is the beta parameter, not eta."),
    answer("3.3", message = "3.3 is the failure free period, not eta."),
    answer("1", message = "Eta is not equal to 1."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  )
)
```

## Multi-Plots

### Multi-Plots

Multi-plots are useful for comparing multiple Weibull models on the same plot. In this way, you can easily see the differences between the models and how they fit the data. To build a multi-plot in WeibullR, create multiple `wblr` objects and add them to a list. Then use the `plot.wblr` function to create the multi-plot.

First, recreate the failure data from the first example of a machine that failed at times 30, 49, 82, 90, and 96.

```{r echo=TRUE}
failures<-c(30, 49, 82, 90, 96)
obj1<-wblr.fit(wblr(failures), col="red")
```

Next, recreate the failure and suspension data from the right censored example of a machine that failed at times 30, 49, 82, 90, and 96 with suspensions at times 100, 45, and 10.

```{r echo=TRUE}
failures<-c(30, 49, 82, 90, 96)
suspensions<-c(100, 45, 10)
obj2<-wblr.fit(wblr(failures, suspensions, col="purple"))
```

Finally, add the 2 `wblr` objects to a list and plot both objects in a single chart.

```{r echo=TRUE}
plot.wblr(list(obj1, obj2))
```

The red model is the fit without suspensions and the purple model is the fit with suspensions. 
Note how the fit with suspensions has a slightly lower beta and higher eta.

```{r quiz6, echo=FALSE}
quiz(caption = "Quiz: Multi-Plots",
  question("What is the difference between obj1 and obj2 in the previous example (select all that apply)?",
    answer("obj1 includes suspensions"),
    answer("obj1 has a fit with a lower beta"),
    answer("obj2 includes suspensions", correct=TRUE),
    answer("obj2 has a fit with a lower beta", correct=TRUE),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("In the previous example, which fit has a higher unreliability at time 100?",
    answer("obj2", message = "obj2 has a slightly lower unreliability at time 100."),
    answer("obj1", correct = TRUE, message = "Correct! obj1 has a slightly higher unreliability at time 100."),
    answer("Both are the same", message = "The fits are slightly different at time 100."),
    answer("Cannot be determined from the plot", message = "The unreliability can be determined from the plot."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  )
)
```

## Competing Failure Modes

A part or a system may fail in different ways. A failure mode is a specific way in which a part or machine can fail. For example, a machine may fail due to corrosion, fatigue, or overload. Each failure mode may have a different distribution of time to failure. Competing failure modes occur when multiple failure modes are present in the same data set. 

When competing failure modes are present, a basic Weibull model tends to fit the data poorly. This is because the basic Weibull assumes that all failures are due to a single failure mode. 

To illustrate, let's simulate some data with 3 different failure modes, where each failure mode has a different beta and eta. The data.frame below contains 120 observations, where 20 observations are failures and 100 observations are suspensions. The failure_mode column indicates the failure mode for each observation.

```{r echo=TRUE}
set.seed(123)
data <- data.frame(
  time = c(
    rweibull(5, 0.5, 20), # Simulate times for Failure Mode A
    rweibull(10, 1, 10), # Simulate times for Failure Mode B
    rweibull(5, 2, 5),  # Simulate times for Failure Mode C
    rweibull(100, 2, 10) # Simulate times for Suspensions
    ),
  event = c(
    rep(1, 20), # Label Failures
    rep(0, 100) # Label Suspensions
    ),
  failure_mode = c(
    rep("A", 5), # Label Failure Mode A
    rep("B", 10), # Label Failure Mode B
    rep("C", 5), # Label Failure Mode C
    rep("", 100)) # No label for Suspensions
  )
```

Next, fit a basic Weibull model to the combined data and plot the results.

```{r echo=TRUE}
obj <- wblr.conf(wblr.fit(wblr(data)))
plot(obj, col="darkgreen", is.plot.legend = FALSE)
```

When the data is combined, the overall fit has a beta of approximately 1, which indicates random failures. However, we know that the data actually contains 3 different failure modes.

Now let's create a separate data set for each failure mode. The code below creates 3 copies of the original data set and sets the event column to 0 for all observations that do not belong to the respective failure mode.

```{r echo=TRUE}
dat1 <- data
dat2 <- data
dat3 <- data
dat1$event[dat1$failure_mode != "A"] <- 0
dat2$event[dat2$failure_mode != "B"] <- 0
dat3$event[dat3$failure_mode != "C"] <- 0
```

Finally, fit a `wblr` object for each failure mode and create a multi-plot. The `is.plot.legend=FALSE` argument removes the legend from the plot.

```{r echo=TRUE}
obj1 <- wblr.fit(wblr(dat1, col="blue"))
obj2 <- wblr.fit(wblr(dat2, col="red"))
obj3 <- wblr.fit(wblr(dat3, col="orange"))
plot.wblr(list(obj1, obj2, obj3), is.plot.legend = FALSE)
```

Note how each failure mode has a different beta and eta. Failure mode A (blue) has a beta less than 1, which means the failure rate is decreasing. Failure mode B (red) has a beta greater than 1, which means that the failure rate is increasing. Failure mode C (orange) has a beta close to 1, which means that the failure rate is approximately constant.

## Contour Plots

Another way to compare Weibull models is with contour plots. A contour plot is a graphical representation of the relationship between the Weibull parameters Beta and Eta. 
 
Once again, let's look at the first example of a machine that failed at times 30, 49, 82, 90, and 96. First, recreate the failure data. Then fit the Weibull model and build the contour plot at a probability level of 90%. The `wblr.conf` function with the `method="lrb"` argument computes the likelihood ratio bounds for the contour plot, and the `plot_contour` function creates the contour plot. The `CL=0.9` argument specifies a probability level of 90%.

```{r echo=TRUE, results='hide'}
failures<-c(30, 49, 82, 90, 96)
obj <- wblr.conf(wblr.fit(wblr(failures, col="blue"), method="mle"), method="lrb")
plot_contour(obj, CL=0.9)
```

The Weibull contour plot shows the confidence region for the Weibull parameters Beta and Eta at a probability level of 90%. The x-axis represents Eta, the y-axis represents Beta, and the contour lines represent the confidence levels. 

### Comparing Contour Plots

To compare multiple Weibull models, we can build contour plots for each model and overlay them on the same chart. If the models are similar, then the contours will overlap. If the models are different, then the contours will not overlap.

To visualize this concept, let's revisit the previous competing failure mode example. First, recreate the failure data with 3 different failure modes. Then create a separate `wblr` object for each failure mode and build the contour plot for each model at a probability level of 90%. The contours are then overlaid on the same chart. Note the use of the `xlim` argument to set the x-axis limits for better visualization.


```{r echo=TRUE, results='hide'}
obj1 <- wblr.conf(wblr.fit(wblr(dat1, col="blue"), method="mle"), method="lrb")
obj2 <- wblr.conf(wblr.fit(wblr(dat2, col="red"), method="mle"), method="lrb")
obj3 <- wblr.conf(wblr.fit(wblr(dat3, col="orange"), method="mle"), method="lrb")
plot_contour(list(obj1, obj2, obj3), CL=0.9, xlim = c(1, 1500))
```

The blue, red, and orange contours represent the 3 different failure modes. Note how the contours overlap, which indicates that the failure modes are not significantly different. If the contours did not overlap, then the failure modes would be significantly different.

```{r quiz7, echo=FALSE}
quiz(caption = "Quiz: Contour Plots",
  question("In the single contour plot example, what is the approximate beta at the center of the contour?",
    answer("Between 2 and 3", message = "3 is slightly too low."),
    answer("Between 75 and 80", message = "The approximate eta is between 75 and 80."),
    answer("Between 4 and 4.5", message = "4 is slightly too high."),
    answer("Between 3 and 3.5", correct = TRUE, message = "Correct! The approximate beta at the center of the contour is between 3 and 3.5."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("In the multi-contour plot example, at a probability level of 90%...",
    answer("obj3 (orange) is significantly different than the others", message = "obj3 (orange) overlaps with obj1 (blue) and obj2 (red), so they are not significantly different."),
    answer("obj2 (red) is significantly different than the others", message = "obj2 (red) overlaps with obj3 (orange), so they are not significantly different."),
    answer("None of the distributions are significantly different", correct=TRUE, message = "Correct! All of the distributions overlap, so they are not significantly different."),
    answer("All of the distributions are significantly different", message = "The distributions overlap, so they are not significantly different."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  )
)
```

## Summary

Congratulations on completing the tutorial on life data analysis!

In this tutorial, we've introduced the concepts of life data analysis and reliability. You have learned to calculate reliability using the Weibull distribution and visualize life data.

### To Get Help

To get help with a specific function in WeibullR, type a question mark before the function name.

```{r echo=TRUE, eval=FALSE}
?MRRw2p
```

For more help with the WeibullR package.

```{r echo=TRUE, eval=FALSE}
help(package="WeibullR")
```

### Additional Resources

- The WeibullR package documentation: <https://cran.r-project.org/package=WeibullR>

### References

- Abernethy, R.B. (2004) The New Weibull Handbook. Fifth Edition.

-   Aden-Buie G, Schloerke B, Allaire J, Rossell Hayes A (2023). learnr: Interactive Tutorials for R. https://rstudio.github.io/learnr/, https://github.com/rstudio/learnr.

-   Silkworth, David. (2020). WeibullR: An R Package for Weibull Analysis for Reliability Engineers. 43-53. <https://doi.org/10.35566/isdsa2019c3>.

-   Silkworth D, Symynck J (2022). *WeibullR: Weibull Analysis for Reliability Engineering*. R package version 1.2.1, <https://CRAN.R-project.org/package=WeibullR>.

---
title: "Helper Functions"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{helpers}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup, include=FALSE}
library(ReliaLearnR)
```

The `ReliaLearnR` package includes several helper functions to calculate common reliability metrics. These functions include:

- `rel(failures, total_time)`: Calculates reliability given the number of failures and total time.
- `avail(downtime, total_time)`: Calculates availability given the downtime and total time.
- `mttf(downtime, total_time)`: Estimates the Mean Time To Failure.
- `mtbf(failures, total_time)`: Estimates the Mean Time Between Failures.
- `fr(failures, total_time)`: Estimates the failure rate.

This vignette provides examples of how to use these functions.

## Examples

To calculate the reliability of an item that ran for 3 years total and was failed for 5 of those days:

```{r echo=TRUE, results='asis'}
result <- rel(5, 3 * 365)
cat(result)
```

To calculate the availability of an item that ran 3 years total, was failed for 5 days, and had scheduled maintenance for 14 days:

```{r echo=TRUE, results='asis'}
result <- avail(5 + 14, 3 * 365)
cat(result)
```

The MTTR can be estimated with the base function `mean`. The MTTR for 5 failures with repair times in days of 5, 10, 15, 8, and 12:

```{r echo=TRUE, results='asis'}
result <- mean(c(5, 10, 15, 8, 12))
cat(result)
```

To estimate the MTTF for 1000 items that ran for 3 years total:

```{r echo=TRUE, results='asis'}
result <- mttf(5 + 14, 3 * 365)
cat(result)
```

To estimate the MTBF for an item that failed 5 times over a total time of 45,000 hours:

```{r echo=TRUE, results='asis'}
result <- mtbf(5, 45000)
cat(result)
```

To estimate the failure rate for 100 items that ran for 5000 hours and had 75 failures:

```{r echo=TRUE, results='asis'}
result <- fr(75, 100 * 5000)
cat(result)
```

The Exponential failure probability can be estimated with the base function `pexp`. To estimate the probability of survival at time 5 for an item with a failure rate of 0.1:

```{r echo=TRUE, results='asis'}
result <- 1 - pexp(5, 0.1)
cat(result)
```

The $B_n$ life for the Exponential distribution can be estimated with the base function `qexp`. To estimate the B10 life for an item with a failure rate of 0.1:

```{r echo=TRUE, results='asis'}
result <- qexp(0.1, 0.1)
cat(result)
```

---
title: "Repairable Systems Analysis"
output:
  learnr::tutorial:
    progressive: true
    allow_skip: true
runtime: shiny_prerendered
---

```{r setup, include=FALSE}
library(learnr)
library(ReliaGrowR)
```

## Introduction

Welcome to the tutorial on **Repairable Systems Analysis**! In this tutorial, we will explore models and methods used to analyze systems that are repaired and returned to service after each failure. These methods are widely used in fleet management, industrial maintenance, and infrastructure planning.

Unlike life data analysis, which focuses on the time to first failure of a non-repairable component,  repairable systems analysis models the entire history of recurring failures on a single system or fleet. The goal is to characterize the failure process, detect trends, and predict future failure behavior.

## Learning Objectives

By the end of this tutorial, you will be able to:

* Define the distinction between repairable and non-repairable systems.
* Explain the Homogeneous Poisson Process (HPP) and Non-Homogeneous Poisson Process (NHPP) as failure process models.
* Describe the Power Law Process and its parameters.
* Fit and interpret the Crow-AMSAA model using `nhpp()` from ReliaGrowR.
* Apply the Piecewise NHPP model to detect changes in failure behavior.
* Compute fleet-level exposure using `exposure()` from ReliaGrowR.
* Estimate the Mean Cumulative Function (MCF) using `mcf()` from ReliaGrowR.
* Interpret model parameters to characterize deterioration, stability, or improvement.
* Use R to predict future failure counts and recurrence rates.

## Repairable vs Non-Repairable Systems

Before applying any model, it is essential to classify the system correctly.

A **non-repairable system** is discarded or replaced with a new, statistically identical unit after each failure. The unit of analysis is the *time to first failure*, and Weibull analysis applies. Examples include ball bearings, fuses, and light bulbs.

A **repairable system** is restored to operating condition after each failure and continues accumulating operating time. The unit of analysis is the *sequence of inter-failure times* or cumulative failure counts over the system's life. Examples include diesel generators, industrial pumps, and aircraft engines.

Two idealized repair assumptions are common:

* **Perfect repair** ("as-good-as-new"): The system is restored to the same condition as a new unit. This is modeled by a renewal process.
* **Minimal repair** ("as-bad-as-old"): The system is restored to the same condition it was in just before the failure — the failure is fixed, but nothing else changes. This is modeled by a Non-Homogeneous Poisson Process (NHPP).

Most field maintenance data is best described by the minimal repair assumption, making NHPP the standard framework for repairable systems analysis.

The recurrence rate (intensity function) $\rho(t)$ describes the instantaneous rate of failure occurrence at time $t$:

$$\rho(t) = \frac{dE[N(t)]}{dt}$$

where $E[N(t)]$ is the expected cumulative number of failures by time $t$.

## Failure Processes: HPP and NHPP

### Homogeneous Poisson Process (HPP)

The **Homogeneous Poisson Process** assumes a *constant* failure rate $\lambda$ throughout the system's life:

$$E[N(t)] = \lambda t \qquad \rho(t) = \lambda$$

The mean time between failures is simply $\text{MTBF} = 1/\lambda$. The HPP is appropriate when failures occur at random, with no trend toward improvement or deterioration.

### Non-Homogeneous Poisson Process (NHPP)

The **Non-Homogeneous Poisson Process** allows the intensity function to change with time:

$$E[N(t)] = \Lambda(t) \qquad \rho(t) = \frac{d\Lambda(t)}{dt}$$

When $\rho(t)$ increases with time the system is deteriorating; when it decreases the system is improving. The Power Law Process is the most widely used NHPP for repairable systems.

Use the controls below to explore how the shape of the NHPP intensity function changes with the beta parameter, compared to a constant HPP rate.

```{r echo=FALSE}
sliderInput("lambda_hpp", label = h3("HPP Rate (λ):"),
            value = 0.05, min = 0.01, max = 0.20, step = 0.01, width = "50%")
numericInput("beta_nhpp", label = h3("NHPP Beta (β):"),
             value = 0.8, min = 0.1, max = 3.0, step = 0.1, width = "50%")
plotOutput("processPlot")
```

```{r context="server"}
output$processPlot <- renderPlot({
  t <- seq(1, 100, by = 1)
  hpp_intensity <- rep(input$lambda_hpp, length(t))
  nhpp_intensity <- 0.01 * input$beta_nhpp * t^(input$beta_nhpp - 1)
  ylim_max <- max(c(hpp_intensity, nhpp_intensity)) * 1.2
  plot(t, hpp_intensity, type = "l", col = "blue", lwd = 2,
       ylim = c(0, ylim_max),
       xlab = "Time", ylab = "Intensity ρ(t)",
       main = "HPP vs NHPP Intensity Function")
  lines(t, nhpp_intensity, col = "red", lwd = 2)
  legend("topright",
         legend = c(paste0("HPP (constant λ = ", input$lambda_hpp, ")"),
                    paste0("NHPP Power Law (β = ", input$beta_nhpp, ")")),
         col = c("blue", "red"), lwd = 2)
})
```

When $\beta < 1$ the NHPP intensity decreases over time (improving system). When $\beta = 1$ the NHPP reduces to the HPP (constant rate). When $\beta > 1$ the intensity increases (aging or deteriorating system).

## The Power Law Process

The **Power Law Process** is an NHPP whose mean value function (expected cumulative failures) follows a power law:

$$E[N(t)] = \lambda t^{\beta}$$

The corresponding intensity function is:

$$\rho(t) = \lambda \beta t^{\beta - 1}$$

**Parameter interpretation:**

* $\lambda > 0$ — scale parameter; governs the overall magnitude of the failure rate.
* $\beta$ — shape parameter:
  * $\beta < 1$: failure intensity is decreasing (system improving).
  * $\beta = 1$: constant intensity, equivalent to HPP.
  * $\beta > 1$: failure intensity is increasing (system deteriorating or aging).

The **instantaneous MTBF** at time $t$ is the reciprocal of the intensity:

$$\text{MTBF}(t) = \frac{1}{\rho(t)} = \frac{1}{\lambda \beta t^{\beta - 1}}$$

Given $n$ failures observed at times $t_1 < t_2 < \cdots < t_n$ over a total operating time $T$, the maximum likelihood estimates are:

$$\hat{\beta} = \frac{n}{\displaystyle\sum_{i=1}^{n} \ln(T/t_i)} \qquad \hat{\lambda} = \frac{n}{T^{\hat{\beta}}}$$

```{r quiz1, echo=FALSE}
quiz(caption = "Quiz: Failure Processes and the Power Law",
  question("A repairable system is observed to have an increasing failure rate over time. Which model is most appropriate?",
    answer("Homogeneous Poisson Process (HPP)",
           message = "HPP assumes a constant failure rate and cannot capture increasing trends."),
    answer("NHPP Power Law with β > 1", correct = TRUE,
           message = "An NHPP with β > 1 produces an increasing intensity function, consistent with a deteriorating system."),
    answer("NHPP Power Law with β < 1",
           message = "β < 1 indicates a decreasing failure rate (improving system), not an increasing one."),
    answer("Weibull life data analysis",
           message = "Weibull analysis applies to time-to-first-failure data on non-repairable components, not to recurring failures on a repairable system."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("In the Power Law Process, what does the scale parameter λ control?",
    answer("Whether the failure rate is increasing or decreasing",
           message = "The direction of the trend is controlled by β, not λ."),
    answer("The overall magnitude (rate) of failures", correct = TRUE,
           message = "λ scales the overall rate of occurrence; β controls whether the rate is increasing, constant, or decreasing."),
    answer("The time between the first and second failure only",
           message = "λ influences the expected number of failures over any interval, not just the first."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("For a Power Law Process with β = 1.0, what is the equivalent model?",
    answer("Weibull distribution with shape parameter 1",
           message = "That describes an exponential life distribution for a single component. When β = 1 the Power Law process becomes a constant-rate recurrent failure process."),
    answer("Homogeneous Poisson Process (HPP)", correct = TRUE,
           message = "When β = 1 the intensity ρ(t) = λ is constant, which is exactly an HPP."),
    answer("Piecewise NHPP with one segment",
           message = "A single-segment Piecewise NHPP with β = 1 would equal HPP, but the standard way to say this is simply HPP."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  )
)
```

## NHPP Analysis

The `nhpp()` function fits the Power Law Process (Crow-AMSAA model) to recurrent failure data, returning estimates of $\lambda$ and $\beta$. The fitted mean value function is plotted against time:

$$E[N(t)] = \hat{\lambda}\, t^{\hat{\beta}}$$

A curve that bends downward (concave) on an MCF vs. time plot indicates $\hat{\beta} < 1$ — the failure rate is falling. A curve that bends upward (convex) indicates $\hat{\beta} > 1$ — the failure rate is rising.

The `nhpp()` function can accept an `mcf` object directly as its first argument. When passed an MCF object, the Power Law is fit to the non-parametric per-system MCF estimate rather than raw cumulative counts — this accounts for fleet size and unequal observation windows automatically.

The recommended two-step workflow is:

1. Estimate the MCF non-parametrically with `mcf()`.
2. Pass the result directly to `nhpp()`.

The following data represent per-system failure records from a fleet of five industrial pumps, each observed for 3,000 hours.

```{r echo=TRUE}
pump_data <- data.frame(
  id    = c(1, 1, 1,  2, 2, 2,  3, 3,  4, 4, 4, 4,  5, 5),
  time  = c(310, 850, 1620,  420, 1050, 2100,  580, 1890,
            240, 710, 1380, 2400,  530, 1740),
  event = rep(1, 14)
)
end_times <- c("1" = 3000, "2" = 3000, "3" = 3000, "4" = 3000, "5" = 3000)

mcf_result <- mcf(id = pump_data$id, time = pump_data$time,
                  event = pump_data$event, end_time = end_times)
fit <- nhpp(time = mcf_result)
plot(fit,
     main = "Crow-AMSAA Model: Industrial Pump Fleet",
     xlab = "Operating Hours")
```

The fitted curve shape directly reveals the failure trend: a concave curve ($\hat{\beta} < 1$) means each successive failure takes longer to arrive — the system is improving.

```{r quiz2, echo=FALSE}
quiz(caption = "Quiz: NHPP Analysis",
  question("In a Crow-AMSAA plot, what does a curve that bends downward (concave shape) indicate?",
    answer("The failure rate is increasing over time",
           message = "A concave curve means failures accumulate more slowly as time passes — the rate is decreasing, not increasing."),
    answer("The failure rate is constant",
           message = "A straight line on the MCF plot would indicate a constant rate (HPP), not a concave curve."),
    answer("The failure rate is decreasing over time", correct = TRUE,
           message = "A concave (downward-bending) MCF curve means β < 1: each successive failure takes longer to arrive, so the system is improving."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("For the pump fleet example, which statement best characterizes the system based on the nhpp() fit?",
    answer("The pumps are deteriorating — failures are becoming more frequent",
           message = "If the fitted curve bends downward (concave), the rate is actually decreasing, not increasing."),
    answer("The pumps show a perfectly constant failure rate (HPP)",
           message = "A straight MCF line would indicate HPP. The concave shape indicates a trend."),
    answer("The pumps show a decreasing failure rate — the fleet appears to be in a favorable operating regime", correct = TRUE,
           message = "The concave curve and β < 1 indicate the failure rate is falling, consistent with stable or improving operational conditions."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("What is the recommended workflow for fitting a Crow-AMSAA model to fleet data with nhpp()?",
    answer("Pass raw cumulative failure counts directly to nhpp() via the event argument",
           message = "Passing raw counts does not account for fleet size or unequal observation windows. The recommended approach uses mcf() first."),
    answer("Estimate the MCF non-parametrically with mcf(), then pass the mcf object to nhpp()", correct = TRUE,
           message = "nhpp() detects an mcf object passed as its first argument and fits the Power Law to the per-system normalized MCF, automatically accounting for fleet size and censoring."),
    answer("Convert individual failure times to a single cumulative total before calling nhpp()",
           message = "Summing across systems without normalizing by the risk set ignores censoring and unequal observation windows. Use mcf() to handle this correctly."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  )
)
```

### Exercise: Fit a Crow-AMSAA Model

A compressor fleet recorded per-system failure times below. Each of the three systems was observed for 5,000 hours. Use `mcf()` to estimate the MCF, then pass the result to `nhpp()` with `method = "LS"` to fit a Power Law model and plot the result.

```{r ex-rs1, exercise=TRUE}
fleet <- data.frame(
  id    = c(rep(1, 3), rep(2, 4), rep(3, 3)),
  time  = c(600, 1500, 3200,  400, 1100, 2300, 4000,  800, 2000, 3800),
  event = rep(1, 10)
)
end_times_ex <- c("1" = 5000, "2" = 5000, "3" = 5000)

```

```{r ex-rs1-hint}
# Step 1: estimate the MCF
# mcf_ex <- mcf(id = fleet$id, time = fleet$time, event = fleet$event,
#               end_time = end_times_ex)
# Step 2: fit the Power Law to the MCF
# fit <- nhpp(time = mcf_ex, method = "LS")
# plot(fit)
```

```{r ex-rs1-solution}
fleet <- data.frame(
  id    = c(rep(1, 3), rep(2, 4), rep(3, 3)),
  time  = c(600, 1500, 3200,  400, 1100, 2300, 4000,  800, 2000, 3800),
  event = rep(1, 10)
)
end_times_ex <- c("1" = 5000, "2" = 5000, "3" = 5000)
mcf_ex <- mcf(id = fleet$id, time = fleet$time, event = fleet$event,
              end_time = end_times_ex)
fit <- nhpp(time = mcf_ex, method = "LS")
plot(fit, main = "Crow-AMSAA: Compressor Fleet",
     xlab = "Operating Hours", ylab = "MCF")
```

## Piecewise NHPP

Sometimes a system's failure behavior changes at a specific point in its operating life, for example after a major overhaul, a design modification, or a change in operating environment. The **Piecewise NHPP** model fits a separate Power Law to each segment of the time axis, separated by one or more **breakpoints** $\tau$:

$$E[N(t)] = \begin{cases} \lambda_1 t^{\beta_1} & 0 < t \leq \tau \\ E[N(\tau)] + \lambda_2 (t - \tau)^{\beta_2} & t > \tau \end{cases}$$

Each segment has its own $\lambda_i$ and $\beta_i$, allowing the model to capture, for example, a deteriorating early period followed by improvement after an overhaul.

Pass a known breakpoint time as the `breaks` argument to `nhpp()`. The function will fit a separate Power Law to each segment. The same MCF-first workflow applies: compute the non-parametric MCF from per-system records, then pass it to `nhpp()`.

```{r echo=TRUE}
# Extended pump fleet: 5 systems observed for 20,000 hours each.
# The fleet underwent a major overhaul at hour 8,000.
pump2 <- data.frame(
  id = c(rep(1, 15), rep(2, 15), rep(3, 13), rep(4, 17), rep(5, 12)),
  time = c(
    # Pump 1: intervals shrink before overhaul (deteriorating), grow after (improving)
    2000, 3400, 4600, 5500, 6200, 6800, 7300, 7700,
    8200, 9000, 10100, 11500, 13300, 15500, 18200,
    # Pump 2
    1800, 3100, 4200, 5100, 5800, 6400, 6900, 7300, 7700,
    8400, 9300, 10500, 12100, 14200, 16900,
    # Pump 3
    2500, 4000, 5200, 6100, 6900, 7500, 7900,
    8600, 9600, 10900, 12500, 14700, 17500,
    # Pump 4
    1500, 2700, 3700, 4500, 5200, 5800, 6300, 6700, 7100, 7500,
    8100, 9000, 10200, 11800, 13800, 16300, 19300,
    # Pump 5
    2200, 3700, 5000, 6000, 6800, 7400, 7800,
    8500, 9600, 11100, 13100, 15700
  ),
  event = rep(1, 72)
)
end_times2 <- c("1" = 20000, "2" = 20000, "3" = 20000,
                "4" = 20000, "5" = 20000)

mcf2   <- mcf(id = pump2$id, time = pump2$time,
              event = pump2$event, end_time = end_times2)
fit_pw <- nhpp(time = mcf2, breaks = 8000, method = "LS")
plot(fit_pw,
     main = "Piecewise NHPP: Fleet Overhaul at Hour 8000",
     xlab = "Operating Hours")
```

The first segment (before the estimated breakpoint) has a steeper slope ($\hat{\beta}_1 > 1$, increasing failure rate), while the second segment (post-overhaul) shows a flatter or downward-bending MCF curve, confirming the overhaul was effective in reducing the failure rate.

Note that the `breaks` argument serves as an *initial estimate* for the breakpoint location. The `segmented` algorithm optimizes the breakpoint position from the data, so the fitted breakpoint in the plot may differ slightly from the value passed to `breaks`. In the example above, the fitted breakpoint is estimated near hour 8,800, close to the known overhaul time of 8,000.

```{r quiz3, echo=FALSE}
quiz(caption = "Quiz: Piecewise NHPP",
  question("What is the primary reason to use a Piecewise NHPP model instead of a single Crow-AMSAA model?",
    answer("The Piecewise model always produces a better statistical fit",
           message = "A more complex model fits observed data better by definition, but is only appropriate when a genuine structural change in the failure process exists."),
    answer("To account for a known or suspected change in the failure behavior at one or more points in time", correct = TRUE,
           message = "The Piecewise NHPP is most appropriate when a real event (overhaul, design change, operational change) is believed to have altered the failure intensity."),
    answer("Because the standard Crow-AMSAA model cannot handle more than 10 data points",
           message = "The standard Crow-AMSAA model handles any number of data points. The Piecewise model is chosen based on physical knowledge, not data volume."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("In the pump fleet example, the post-overhaul segment shows a lower failure rate than the pre-overhaul segment. What does this suggest?",
    answer("The overhaul made the system worse",
           message = "A lower failure rate after the overhaul indicates improvement, not deterioration."),
    answer("The overhaul had no measurable effect on the failure process",
           message = "If the second-segment β were identical to the first, the overhaul had no effect. A lower intensity in the second segment is evidence of improvement."),
    answer("The overhaul was effective in reducing the failure rate", correct = TRUE,
           message = "A lower β or decreasing intensity in the post-overhaul segment is evidence that the maintenance intervention improved the system's failure behavior."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("In nhpp(), what role does the breaks argument play for piecewise Power Law fitting?",
    answer("It fixes the breakpoint at exactly that time value",
           message = "breaks provides an initial estimate to the segmented algorithm, which then optimizes the breakpoint position from the data. The fitted breakpoint may differ from the value passed."),
    answer("It provides an initial estimate of the breakpoint; the algorithm optimizes its position from the data", correct = TRUE,
           message = "nhpp() passes breaks to segmented::segmented() as a starting value. The algorithm finds the maximum-likelihood breakpoint location, which may shift from the initial estimate."),
    answer("It has no effect — nhpp() always uses a single Power Law",
           message = "Providing breaks triggers piecewise fitting. Without breaks, a single Power Law is fit."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  )
)
```

## Exposure Analysis

When analyzing a *fleet* of systems rather than a single system, individual units enter and leave observation at different times. **Exposure** is the total operating time accumulated across all systems that are currently at risk. It answers the question: "How much system-time has been observed at each point in calendar time?"

The `exposure()` function computes:

* **Cumulative exposure** — total system-time at risk up to each event time.
* **Number of systems at risk** — how many units were under observation at each event time.
* **Event rate** — cumulative events divided by cumulative exposure, providing a fleet-wide failure rate estimate.

The function accepts per-event records with three columns:

* `id` — system identifier.
* `time` — cumulative event or end-of-observation time for that system.
* `event` — 1 for a failure event, 0 for censoring (end of observation without a failure).

The dataset below records individual failure events for a fleet of five industrial pumps, each observed for 3,000 hours.

```{r echo=TRUE}
pump_data <- data.frame(
  id    = c(1, 1, 1,  2, 2, 2,  3, 3,  4, 4, 4, 4,  5, 5),
  time  = c(310, 850, 1620,  420, 1050, 2100,  580, 1890,  240, 710, 1380, 2400,  530, 1740),
  event = c(1, 1, 1,  1, 1, 1,  1, 1,  1, 1, 1, 1,  1, 1)
)
```

```{r echo=TRUE}
exp_result <- exposure(id = pump_data$id, time = pump_data$time,
                       event = pump_data$event)
plot(exp_result)
```

The exposure plot shows how the total observed system-time accumulates across the fleet. The event rate curve reveals whether the fleet-wide failure rate is increasing, stable, or decreasing over time.

```{r quiz4, echo=FALSE}
quiz(caption = "Quiz: Exposure Analysis",
  question("What does 'exposure' represent in a fleet-level repairable systems analysis?",
    answer("The total number of failures observed across all systems",
           message = "That is the cumulative event count. Exposure measures observation time, not event counts."),
    answer("The total operating time accumulated across all systems currently under observation", correct = TRUE,
           message = "Exposure is the sum of operating time contributed by each system still in the risk set. It is used to normalize event counts into event rates."),
    answer("The probability that any one system fails before a given time",
           message = "That describes a reliability function, not exposure. Exposure is a time-at-risk quantity."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("In the exposure() function, what does event = 0 indicate for a record?",
    answer("The system experienced a failure at that time",
           message = "event = 1 marks a failure. event = 0 marks the end of observation without a failure."),
    answer("The system's observation ended without a failure at that time (censoring)", correct = TRUE,
           message = "event = 0 is a censoring indicator: the system was removed from observation at that time, contributing exposure up to that point but no failure event."),
    answer("The system was not yet in service at that time",
           message = "Systems not yet in service simply have no records before their entry time. event = 0 marks an observation end."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("Why is it important to account for exposure when comparing failure rates across systems with different observation periods?",
    answer("Systems observed longer always have higher failure rates",
           message = "Longer observation periods produce more total failures, but the rate (failures per unit time) may be identical."),
    answer("Without normalizing by exposure, a system observed for twice as long will appear to have twice as many failures even if its underlying rate is the same", correct = TRUE,
           message = "Exposure provides the denominator for the event rate. Without it, raw failure counts confound the length of observation with the underlying failure intensity."),
    answer("Exposure only matters for non-repairable systems",
           message = "Exposure is especially important for repairable systems analyzed at the fleet level, where units have different observation windows."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  )
)
```

## Mean Cumulative Function

The **Mean Cumulative Function (MCF)** is a non-parametric estimate of the expected cumulative number of failures per system as a function of time. It is the repairable-systems analogue of the Kaplan-Meier estimator in survival analysis: it makes no distributional assumptions about the failure process and can accommodate systems with different observation periods.

The MCF at time $t$ is estimated using the Nelson-Aalen estimator:

$$\hat{M}(t) = \sum_{t_j \leq t} \frac{d_j}{n_j}$$

where $d_j$ is the number of failures at time $t_j$ and $n_j$ is the number of systems still under observation at $t_j$. The MCF provides a visual summary of the recurrence trend without requiring a parametric model.

The `mcf()` function accepts the same per-event data format as `exposure()`. Passing `end_time` ensures that systems remaining under observation beyond their last failure are correctly kept in the risk set.

```{r echo=TRUE}
end_times <- c("1" = 3000, "2" = 3000, "3" = 3000, "4" = 3000, "5" = 3000)

mcf_result <- mcf(id = pump_data$id, time = pump_data$time,
                  event = pump_data$event,
                  end_time = end_times)
plot(mcf_result,
     main = "Mean Cumulative Function: Industrial Pump Fleet",
     xlab = "Operating Hours",
     ylab = "Expected Cumulative Failures per System")
```

The slope of the MCF curve reveals the recurrence trend: a steepening slope indicates an increasing failure rate; a flattening slope indicates improvement. Confidence bounds (shown by default) reflect uncertainty in the estimate.

```{r quiz5, echo=FALSE}
quiz(caption = "Quiz: Mean Cumulative Function",
  question("What does the MCF estimate for a fleet of repairable systems?",
    answer("The probability that a randomly selected system survives to time t",
           message = "That is the reliability function, used in life data analysis. The MCF estimates cumulative expected failures, not survival probability."),
    answer("The expected cumulative number of failures per system as a function of time", correct = TRUE,
           message = "The MCF tracks how many failures, on average, a system is expected to have accumulated by each point in time, accounting for systems that enter or leave observation at different times."),
    answer("The total number of failures across the entire fleet up to time t",
           message = "That is the cumulative event count. The MCF normalizes by the risk set to give a per-system estimate."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("Why should end_time be provided to mcf() when systems were observed beyond their last event?",
    answer("It makes the MCF plot smoother",
           message = "end_time affects the risk set, not the smoothness of the curve."),
    answer("Without end_time, systems with no late events are assumed to have left observation at their last failure, causing the MCF to overestimate the recurrence rate", correct = TRUE,
           message = "If end_time is omitted, mcf() infers observation end from the last recorded time per system. A system observed but failure-free after hour 2000 would be dropped from the risk set too early, inflating the estimated rate."),
    answer("end_time is only needed when some systems have more failures than others",
           message = "end_time is needed whenever systems were observed beyond their last recorded failure, regardless of the total failure count."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("If the MCF curve has a steepening slope over time, what does this indicate about the fleet?",
    answer("The fleet's failure rate is decreasing — systems are improving",
           message = "A steepening slope means the curve is climbing more steeply, indicating more failures per unit time — an increasing rate."),
    answer("The fleet's failure rate is increasing — systems may be deteriorating or aging", correct = TRUE,
           message = "A steepening MCF slope means each additional unit of time is associated with more expected failures, consistent with a deteriorating or aging fleet."),
    answer("The MCF has reached a steady-state plateau",
           message = "A plateau would indicate a decreasing or near-zero failure rate. A steepening slope is the opposite."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  )
)
```

### Exercise: Mean Cumulative Function

Three systems were observed with the following failure times (hours). Each system was observed until hour 2000. Compute and plot the MCF using `mcf()`.

```{r ex-rs2, exercise=TRUE}
id    <- c(rep("1", 3), rep("2", 4), rep("3", 2))
time  <- c(400, 900, 1500,   300, 700, 1200, 1800,   600, 1400)
event <- rep(1, 9)
end_times <- c("1" = 2000, "2" = 2000, "3" = 2000)

```

```{r ex-rs2-hint}
# Use mcf() with the id, time, event, and end_time arguments, then plot
# fit <- mcf(id = id, time = time, event = event, end_time = end_times)
# plot(fit)
```

```{r ex-rs2-solution}
id    <- c(rep("1", 3), rep("2", 4), rep("3", 2))
time  <- c(400, 900, 1500,   300, 700, 1200, 1800,   600, 1400)
event <- rep(1, 9)
end_times <- c("1" = 2000, "2" = 2000, "3" = 2000)
fit <- mcf(id = id, time = time, event = event, end_time = end_times)
plot(fit, main = "Mean Cumulative Function", xlab = "Time (hours)", ylab = "MCF")
```

## Interpreting Results and Repair Effectiveness

### Reading the Beta Parameter

The estimated $\hat{\beta}$ from any Power Law fit gives immediate operational insight:

| $\hat{\beta}$ | Interpretation | Typical cause |
|:---:|---|---|
| $< 1$ | Failure rate decreasing | Effective maintenance, infant mortality resolved, improving conditions |
| $= 1$ | Constant failure rate | Random external shocks dominate, stable operating regime |
| $> 1$ | Failure rate increasing | Wear-out, accumulating damage, aging components, deteriorating environment |

### Cumulative vs Instantaneous MTBF

Two distinct MTBF metrics arise in repairable systems analysis:

* **Cumulative MTBF**: total operating time divided by total number of failures — a historical average.
* **Instantaneous MTBF**: the expected time to the *next* failure if the system continues from its current state.

$$\text{Instantaneous MTBF}(t) = \frac{1}{\hat{\lambda}\,\hat{\beta}\,t^{\hat{\beta}-1}}$$

For $\hat{\beta} > 1$ the instantaneous MTBF falls *below* the cumulative average at late times. The system's near-term reliability is worse than its history suggests.

### Predicting Future Failures

The expected number of additional failures from current time $T$ to a future time $T + \Delta t$ is:

$$E[\Delta N] = \hat{\lambda}\,(T + \Delta t)^{\hat{\beta}} - \hat{\lambda}\,T^{\hat{\beta}}$$

Use the controls below to explore how the forecast changes with model parameters and horizon.

```{r echo=FALSE}
numericInput("current_t",  label = h3("Current cumulative time (hours):"),
             value = 15000, min = 1000, max = 100000, step = 1000, width = "50%")
numericInput("future_t",   label = h3("Forecast horizon (additional hours):"),
             value = 2000, min = 100, max = 20000, step = 100, width = "50%")
numericInput("beta_pred",  label = h3("Beta (β) from model fit:"),
             value = 0.85, min = 0.1, max = 3.0, step = 0.05, width = "50%")
numericInput("lambda_pred", label = h3("Lambda (λ) from model fit:"),
             value = 0.0003, min = 0.00001, max = 0.01, step = 0.00001, width = "50%")
plotOutput("forecastPlot")
```

```{r context="server"}
output$forecastPlot <- renderPlot({
  T0   <- input$current_t
  dT   <- input$future_t
  beta <- input$beta_pred
  lam  <- input$lambda_pred
  t_seq <- seq(T0, T0 + dT, length.out = 200)
  expected_add <- lam * t_seq^beta - lam * T0^beta
  plot(t_seq - T0, expected_add, type = "l", col = "darkred", lwd = 2,
       xlab = "Additional Operating Hours",
       ylab = "Expected Additional Failures",
       main = paste0("Failure Forecast from Hour ", T0))
  grid()
})
```

```{r quiz6, echo=FALSE}
quiz(caption = "Quiz: Interpreting Results",
  question("A repairable system has a fitted Power Law model with β = 1.35. What is the most likely operational implication?",
    answer("The failure rate is decreasing — repairs are effective",
           message = "β > 1 means the failure rate is increasing, not decreasing."),
    answer("The system has a constant failure rate — no trend is evident",
           message = "β = 1 would indicate a constant rate. β = 1.35 indicates an increasing trend."),
    answer("The failure rate is increasing — wear-out or aging may be occurring", correct = TRUE,
           message = "β > 1 means ρ(t) is increasing with time. This is characteristic of aging or wear-out and may indicate the system is approaching a major overhaul threshold."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("What is the difference between cumulative MTBF and instantaneous MTBF for a repairable system?",
    answer("They are the same quantity calculated two different ways",
           message = "They are equal only when β = 1 (HPP). For any other β they diverge over time."),
    answer("Cumulative MTBF is a historical average over all observed failures; instantaneous MTBF reflects the current failure rate at a specific point in time", correct = TRUE,
           message = "Cumulative MTBF = T / N(T). Instantaneous MTBF = 1/ρ(t) and changes with time when β ≠ 1. For β > 1, instantaneous MTBF is lower than the cumulative average at late times."),
    answer("Instantaneous MTBF is always larger than cumulative MTBF",
           message = "For β > 1 (aging) the instantaneous MTBF is actually smaller than the cumulative average at late times."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("A fleet has been operating for 10,000 cumulative hours with a Power Law fit of λ = 0.0002 and β = 1.1. What is the expected number of additional failures in the next 1,000 hours?",
    answer("Approximately 1",
           message = "Try computing: 0.0002 × (11000^1.1 − 10000^1.1). The result is higher than 1."),
    answer("Approximately 3", correct = TRUE,
           message = "E[ΔN] = 0.0002 × (11000^1.1 − 10000^1.1) ≈ 3. With β slightly above 1 the failure rate is above its historical average."),
    answer("Approximately 10",
           message = "10 is too high. With β = 1.1 the rate is only modestly above the historical average."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  )
)
```

### Exercise: Predict Future Failures

A system has been operating for 5,000 cumulative hours. A Power Law fit yielded λ = 0.0005 and β = 1.2. Calculate the expected number of additional failures in the next 1,000 hours (from hour 5,000 to hour 6,000).

```{r ex-rs3, exercise=TRUE}
lambda    <- 0.0005
beta      <- 1.2
T_current <- 5000
T_future  <- 6000

```

```{r ex-rs3-hint}
# E[ΔN] = lambda * T_future^beta - lambda * T_current^beta
```

```{r ex-rs3-solution}
lambda    <- 0.0005
beta      <- 1.2
T_current <- 5000
T_future  <- 6000
expected_failures <- lambda * T_future^beta - lambda * T_current^beta
cat("Expected additional failures:", round(expected_failures, 2), "\n")
```

## Summary

Congratulations on completing the Repairable Systems Analysis tutorial! You have learned how to:

* Distinguish repairable from non-repairable systems and select the appropriate analytical framework.
* Apply the HPP and NHPP as models for recurrent failure data.
* Interpret the Power Law Process parameters $\lambda$ and $\beta$ to characterize system failure trends.
* Use `nhpp()` to fit and plot the Crow-AMSAA model.
* Use `nhpp()` with `breaks` to model systems with structural changes in failure behavior.
* Use `exposure()` to compute fleet-level operating time at risk and event rates.
* Use `mcf()` to non-parametrically estimate the expected cumulative failures per system.
* Forecast future failure counts using the fitted Power Law model.

```{r quiz7, echo=FALSE}
quiz(caption = "Quiz: Repairable Systems — Review",
  question("Which of the following datasets is most appropriate for Repairable Systems Analysis using the Power Law Process?",
    answer("Times to first failure for 50 identical ball bearings",
           message = "Times to first failure on non-repairable components is the domain of Life Data Analysis (Weibull), not repairable systems analysis."),
    answer("Cumulative failure counts recorded at regular intervals for a fleet of diesel generators over 5 years", correct = TRUE,
           message = "Recurring failures on systems that are repaired and returned to service is exactly the domain of repairable systems analysis."),
    answer("Failure probabilities derived from a reliability block diagram",
           message = "Reliability block diagrams describe system-level reliability from component reliabilities — a RAM analysis topic, not a recurrent failure model."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("Which ReliaGrowR function fits the Crow-AMSAA (Power Law) model to recurrent failure data?",
    answer("duane()", message = "duane() fits the Duane graphical model and is not the primary parametric fitting function in ReliaGrowR."),
    answer("nhpp()", correct = TRUE, message = "nhpp() fits the Power Law NHPP (Crow-AMSAA) model and, when breaks is provided, a piecewise segmented model."),
    answer("alt.data()", message = "alt.data() is from WeibullR.ALT and is used for Accelerated Life Testing data preparation, not repairable systems analysis."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  ),
  question("A system with β = 0.72 from a Power Law fit is scheduled for a major overhaul at 20,000 hours. Based on the model alone, what does the trend suggest about the urgency of the overhaul?",
    answer("The overhaul is urgently needed — β < 1 means the system is deteriorating rapidly",
           message = "β < 1 indicates a decreasing failure rate (improving system), not deterioration."),
    answer("The failure rate is already decreasing, so the overhaul may be less urgently indicated by trend alone", correct = TRUE,
           message = "β < 1 indicates an already-improving system. The model suggests the current trend is favorable, which is relevant information for the maintenance decision."),
    answer("The model cannot say anything about the value of a scheduled overhaul",
           message = "The model directly characterizes the failure trend. A decreasing failure rate (β < 1) is directly relevant to maintenance scheduling decisions."),
    random_answer_order = TRUE,
    allow_retry = TRUE
  )
)
```

### To Get Help

```{r echo=TRUE, eval=FALSE}
?nhpp
```

```{r echo=TRUE, eval=FALSE}
?exposure
```

```{r echo=TRUE, eval=FALSE}
?mcf
```

```{r echo=TRUE, eval=FALSE}
help(package = "ReliaGrowR")
```

### Additional Resources

* ReliaGrowR package documentation: <https://cran.r-project.org/package=ReliaGrowR>
* ReliaSoft — Recurrent Event Data Analysis: <https://www.reliawiki.org/index.php/Repairable_Systems_Analysis>
* Crow (1975) technical report: <https://apps.dtic.mil/sti/citations/ADA020296>

### References

* Aden-Buie G, Schloerke B, Allaire J, Rossell Hayes A (2023). *learnr: Interactive Tutorials for R*. <https://rstudio.github.io/learnr/>

* Ascher H, Feingold H (1984). *Repairable Systems Reliability*. Marcel Dekker, New York.

* Crow, Larry H. (1975). Reliability Analysis for Complex, Repairable Systems. *Army Material Systems Analysis Activity*, 40. <https://apps.dtic.mil/sti/citations/ADA020296>

* Govan P (2024). *ReliaGrowR: Reliability Growth Analysis*. R package. <https://cran.r-project.org/package=ReliaGrowR>

* Guo H, Mettas A, Sarakakis G, Niu P (2010). Piecewise NHPP models with maximum likelihood estimation for repairable systems. *2010 Proceedings — Annual Reliability and Maintainability Symposium (RAMS)*, 1–7. <https://doi.org/10.1109/RAMS.2010.5448029>

* Meeker W. Q., Escobar L. A. (1998). *Statistical Methods for Reliability Data*. Wiley-Interscience, New York.

---
title: "Helper Functions"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{helpers}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup, include=FALSE}
library(ReliaLearnR)
```

The `ReliaLearnR` package includes several helper functions to calculate common reliability metrics. These functions include:

- `rel(outageTime, totalTime)`: Calculates reliability given the outage time and total time.
- `avail(unavailTime, totalTime)`: Calculates availability given the unavailable time and total time.
- `mttf(failures, totalTime)`: Estimates the Mean Time To Failure.
- `mtbf(failures, totalTime)`: Estimates the Mean Time Between Failures.
- `fr(failures, totalTime)`: Estimates the failure rate.

This vignette provides examples of how to use these functions.

## Examples

To calculate the reliability of an item that ran for 3 years total and was failed for 5 of those days:

```{r echo=TRUE, results='asis'}
result <- rel(5, 3 * 365)
cat(result)
```

To calculate the availability of an item that ran 3 years total, was failed for 5 days, and had scheduled maintenance for 14 days:

```{r echo=TRUE, results='asis'}
result <- avail(5 + 14, 3 * 365)
cat(result)
```

The MTTR can be estimated with the base function `mean`. The MTTR for 5 failures with repair times in days of 5, 10, 15, 8, and 12:

```{r echo=TRUE, results='asis'}
result <- mean(c(5, 10, 15, 8, 12))
cat(result)
```

To estimate the MTTF for an item that failed 5 times over a 3-year period:

```{r echo=TRUE, results='asis'}
result <- mttf(5, 3 * 365)
cat(result)
```

To estimate the MTBF for an item that failed 5 times over a total time of 45,000 hours:

```{r echo=TRUE, results='asis'}
result <- mtbf(5, 45000)
cat(result)
```

To estimate the failure rate for 100 items that ran for 5000 hours and had 75 failures:

```{r echo=TRUE, results='asis'}
result <- fr(75, 100 * 5000)
cat(result)
```

The Exponential failure probability can be estimated with the base function `pexp`. To estimate the probability of survival at time 5 for an item with a failure rate of 0.1:

```{r echo=TRUE, results='asis'}
result <- 1 - pexp(5, 0.1)
cat(result)
```

The $B_n$ life for the Exponential distribution can be estimated with the base function `qexp`. To estimate the B10 life for an item with a failure rate of 0.1:

```{r echo=TRUE, results='asis'}
result <- qexp(0.1, 0.1)
cat(result)
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ram.R
\name{ram}
\alias{ram}
\title{Launch the RAM Analysis Tutorial}
\usage{
ram()
}
\value{
This function does not return a value.
}
\description{
This function launches an interactive tutorial on Reliability, Availability,
and Maintainability (RAM) analysis.
}
\examples{
if (interactive()) {
  ram()
}
}
\seealso{
\url{https://paulgovan.github.io/ReliaLearnR/}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ReliaLearnR-package.R
\docType{package}
\name{ReliaLearnR-package}
\alias{ReliaLearnR}
\alias{ReliaLearnR-package}
\title{ReliaLearnR: Learning Modules for Reliability Analysis}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

Learning modules for reliability analysis including modules for Reliability, Availability, and Maintainability (RAM) Analysis, Life Data Analysis, Reliability Testing, Repairable Systems Analysis, and Reliability Block Diagrams.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://paulgovan.github.io/ReliaLearnR/}
  \item \url{https://github.com/paulgovan/ReliaLearnR}
  \item Report bugs at \url{https://github.com/paulgovan/ReliaLearnR/issues}
}

}
\author{
\strong{Maintainer}: Paul Govan \email{paul.govan2@gmail.com} (\href{https://orcid.org/0000-0002-1821-8492}{ORCID}) [copyright holder]

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rt.R
\name{rt}
\alias{rt}
\title{Launch the Reliability Testing Tutorial}
\usage{
rt()
}
\value{
This function does not return a value.
}
\description{
This function launches the Reliability Testing tutorial
}
\examples{
if (interactive()) {
  rt()
}
}
\seealso{
\url{https://paulgovan.github.io/ReliaLearnR/}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{mtbf}
\alias{mtbf}
\title{Mean Time Between Failures (MTBF) for repairable items.}
\usage{
mtbf(failures, totalTime)
}
\arguments{
\item{failures}{Numeric scalar or numeric vector of failure counts (non-negative).}

\item{totalTime}{Numeric scalar or numeric vector of total operating times.}
}
\value{
Numeric scalar: MTBF. If number of failures is zero, returns Inf (with a warning).
}
\description{
MTBF = total operating time / number of failures. The MTBF is the expected time
between consecutive failures. It is commonly used for repairable items. The
behavior is the same as mttf here; keep separate name for semantic clarity.
}
\examples{
mtbf(5, 1000)
mtbf(c(2,3), c(500,500))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{fr}
\alias{fr}
\title{Failure rate (lambda)}
\usage{
fr(failures, totalTime)
}
\arguments{
\item{failures}{Numeric scalar or numeric vector of failure counts (non-negative).}

\item{totalTime}{Numeric scalar or numeric vector of total operating times.}
}
\value{
Numeric scalar: failures per unit time (failures / totalTime).
}
\description{
Failure rate is the frequency with which an engineered system or component fails,
expressed in failures per unit of time. This function computes failure rate given
failure counts and total operating times.
}
\examples{
fr(75, 5000)
fr(c(10,5), c(1000,2000))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rs.R
\name{rs}
\alias{rs}
\title{Launch the Repairable Systems Analysis Tutorial}
\usage{
rs()
}
\value{
This function does not return a value.
}
\description{
This function launches an interactive tutorial on Repairable Systems Analysis.
}
\examples{
if (interactive()) {
  rs()
}
}
\seealso{
\url{https://paulgovan.github.io/ReliaLearnR/}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{mttf}
\alias{mttf}
\title{Mean Time To Failure (MTTF)}
\usage{
mttf(failures, totalTime)
}
\arguments{
\item{failures}{Numeric scalar or numeric vector of failure counts (non-negative).}

\item{totalTime}{Numeric scalar or numeric vector of total operating times.}
}
\value{
Numeric scalar: MTTF. If number of failures is zero, returns Inf (with a warning).
}
\description{
For non-repairable items MTTF = total operating time / number of failures. The
MTTF is the expected time to the first failure. It is commonly used for non-repairable items.
The behavior implemented is the same as mtbf here; keep separate name for semantic clarity.
}
\examples{
mttf(5, 1000)
mttf(c(2,3), c(500,500))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{avail}
\alias{avail}
\title{Availability (1 - unavailability / total)}
\usage{
avail(unavailTime, totalTime)
}
\arguments{
\item{unavailTime}{Numeric scalar or numeric vector of unavailable times.}

\item{totalTime}{Numeric scalar or numeric vector of total times (same units as unavailTime).}
}
\value{
Numeric scalar: availability for the period (between 0 and 1).
}
\description{
Availability is the proportion of time a system is in a functioning condition.
This function computes availability given unavailable times and total times.
}
\examples{
avail(100, 1000)
avail(c(5,10), c(500,600))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rbd.R
\name{rbd}
\alias{rbd}
\title{Launch the Reliability Block Diagram Tutorial}
\usage{
rbd()
}
\value{
This function does not return a value.
}
\description{
This function launches an interactive tutorial on Reliability Block Diagrams
and system reliability, covering series, parallel, mixed, and k-out-of-n
configurations as well as an introduction to Fault Tree Analysis.
}
\examples{
if (interactive()) {
  rbd()
}
}
\seealso{
\url{https://paulgovan.github.io/ReliaLearnR/}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{rel}
\alias{rel}
\title{Reliability (1 - outage / total)}
\usage{
rel(outageTime, totalTime)
}
\arguments{
\item{outageTime}{Numeric scalar or numeric vector of forced outage times.}

\item{totalTime}{Numeric scalar or numeric vector of total times (same units as outageTime).}
}
\value{
Numeric scalar: reliability for the period (between 0 and 1).
}
\description{
Reliability is the probability that an item will perform its intended function
without failure over a specified period under stated conditions. This function
computes reliability given outage times and total times.
}
\examples{
rel(100, 1000)
rel(c(10,20), c(500, 600))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lda.R
\name{lda}
\alias{lda}
\title{Launch the Life Data Analysis Tutorial}
\usage{
lda()
}
\value{
This function does not return a value.
}
\description{
This function launches an interactive tutorial for life data analysis.
}
\examples{
if (interactive()) {
  lda()
}
}
\seealso{
\url{https://paulgovan.github.io/ReliaLearnR/}
}
